@@ -1,5 +1,4 @@
-{-# LANGUAGE CPP #-}
-{-# LANGUAGE GADTs #-}
+{-# language GADTs #-}
 {-# LANGUAGE TupleSections #-}
 {-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE BinaryLiterals #-}
@@ -12,8 +11,6 @@ module GHC.CmmToAsm.AArch64.CodeGen (
 
 where
 
-#include "HsVersions.h"
-
 -- NCG stuff:
 import GHC.Prelude hiding (EQ)
 
@@ -65,6 +62,7 @@ import GHC.Utils.Misc
 import GHC.Utils.Panic
 
 -- Note [General layout of an NCG]
+-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 -- @cmmTopCodeGen@ will be our main entry point to code gen.  Here we'll get
 -- @RawCmmDecl@; see GHC.Cmm
 --
@@ -173,7 +171,7 @@ basicBlockCodeGen block = do
 -- -----------------------------------------------------------------------------
 -- | Utilities
 ann :: SDoc -> Instr -> Instr
-ann doc instr {- | debugIsOn -} = ANN doc instr
+ann doc instr {- debugIsOn -} = ANN doc instr
 -- ann _ instr = instr
 {-# INLINE ann #-}
 
@@ -201,8 +199,8 @@ ann doc instr {- | debugIsOn -} = ANN doc instr
 -- forced until we actually force them, and without -dppr-debug they should
 -- never end up being forced.
 annExpr :: CmmExpr -> Instr -> Instr
-annExpr e instr {- | debugIsOn -} = ANN (text . show $ e) instr
--- annExpr e instr {- | debugIsOn -} = ANN (pprExpr genericPlatform e) instr
+annExpr e instr {- debugIsOn -} = ANN (text . show $ e) instr
+-- annExpr e instr {- debugIsOn -} = ANN (pprExpr genericPlatform e) instr
 -- annExpr _ instr = instr
 {-# INLINE annExpr #-}
 
@@ -478,7 +476,7 @@ opRegWidth w   = pprPanic "opRegWidth" (text "Unsupported width" <+> ppr w)
 getRegister' :: NCGConfig -> Platform -> CmmExpr -> NatM Register
 -- OPTIMIZATION WARNING: CmmExpr rewrites
 -- 1. Rewrite: Reg + (-n) => Reg - n
---    TODO: this expression souldn't even be generated to begin with.
+--    TODO: this expression shouldn't even be generated to begin with.
 getRegister' config plat (CmmMachOp (MO_Add w0) [x, CmmLit (CmmInt i w1)]) | i < 0
   = getRegister' config plat (CmmMachOp (MO_Sub w0) [x, CmmLit (CmmInt (-i) w1)])
 
@@ -664,11 +662,10 @@ getRegister' config plat expr
         -- See Note [Signed arithmetic on AArch64].
         negate code w reg = do
             let w' = opRegWidth w
-            (reg', code_sx) <- signExtendReg w w' reg
             return $ Any (intFormat w) $ \dst ->
                 code `appOL`
-                code_sx `snocOL`
-                NEG (OpReg w' dst) (OpReg w' reg') `appOL`
+                signExtendReg w w' reg `snocOL`
+                NEG (OpReg w' dst) (OpReg w' reg) `appOL`
                 truncateReg w' w dst
 
         ss_conv from to reg code =
@@ -797,7 +794,7 @@ getRegister' config plat expr
             -- <OP> x<n>, x<m>, x<o>
             (reg_x, format_x, code_x) <- getSomeReg x
             (reg_y, format_y, code_y) <- getSomeReg y
-            MASSERT2(isIntFormat format_x == isIntFormat format_y, text "bitOp: incompatible")
+            massertPpr (isIntFormat format_x == isIntFormat format_y) $ text "bitOp: incompatible"
             return $ Any (intFormat w) (\dst ->
                 code_x `appOL`
                 code_y `appOL`
@@ -813,35 +810,33 @@ getRegister' config plat expr
               -- <OP> x<n>, x<m>, x<o>
               (reg_x, format_x, code_x) <- getSomeReg x
               (reg_y, format_y, code_y) <- getSomeReg y
-              MASSERT2(isIntFormat format_x && isIntFormat format_y, text "intOp: non-int")
+              massertPpr (isIntFormat format_x && isIntFormat format_y) $ text "intOp: non-int"
               -- This is the width of the registers on which the operation
               -- should be performed.
               let w' = opRegWidth w
                   signExt r
-                    | not is_signed  = return (r, nilOL)
+                    | not is_signed  = nilOL
                     | otherwise      = signExtendReg w w' r
-              (reg_x_sx, code_x_sx) <- signExt reg_x
-              (reg_y_sx, code_y_sx) <- signExt reg_y
               return $ Any (intFormat w) $ \dst ->
                   code_x `appOL`
                   code_y `appOL`
                   -- sign-extend both operands
-                  code_x_sx `appOL`
-                  code_y_sx `appOL`
-                  op (OpReg w' dst) (OpReg w' reg_x_sx) (OpReg w' reg_y_sx) `appOL`
+                  signExt reg_x `appOL`
+                  signExt reg_y `appOL`
+                  op (OpReg w' dst) (OpReg w' reg_x) (OpReg w' reg_y) `appOL`
                   truncateReg w' w dst -- truncate back to the operand's original width
 
           floatOp w op = do
             (reg_fx, format_x, code_fx) <- getFloatReg x
             (reg_fy, format_y, code_fy) <- getFloatReg y
-            MASSERT2(isFloatFormat format_x && isFloatFormat format_y, text "floatOp: non-float")
+            massertPpr (isFloatFormat format_x && isFloatFormat format_y) $ text "floatOp: non-float"
             return $ Any (floatFormat w) (\dst -> code_fx `appOL` code_fy `appOL` op (OpReg w dst) (OpReg w reg_fx) (OpReg w reg_fy))
 
           -- need a special one for conditionals, as they return ints
           floatCond w op = do
             (reg_fx, format_x, code_fx) <- getFloatReg x
             (reg_fy, format_y, code_fy) <- getFloatReg y
-            MASSERT2(isFloatFormat format_x && isFloatFormat format_y, text "floatCond: non-float")
+            massertPpr (isFloatFormat format_x && isFloatFormat format_y) $ text "floatCond: non-float"
             return $ Any (intFormat w) (\dst -> code_fx `appOL` code_fy `appOL` op (OpReg w dst) (OpReg w reg_fx) (OpReg w reg_fy))
 
       case op of
@@ -852,7 +847,7 @@ getRegister' config plat expr
         MO_Sub w -> intOp False w (\d x y -> unitOL $ annExpr expr (SUB d x y))
 
         -- Note [CSET]
-        --
+        -- ~~~~~~~~~~~
         -- Setting conditional flags: the architecture internally knows the
         -- following flag bits.  And based on thsoe comparisons as in the
         -- table below.
@@ -1024,21 +1019,16 @@ getRegister' config plat expr
 
 -- | Instructions to sign-extend the value in the given register from width @w@
 -- up to width @w'@.
-signExtendReg :: Width -> Width -> Reg -> NatM (Reg, OrdList Instr)
+signExtendReg :: Width -> Width -> Reg -> OrdList Instr
 signExtendReg w w' r =
     case w of
-      W64 -> noop
+      W64 -> nilOL
       W32
-        | w' == W32 -> noop
-        | otherwise -> extend SXTH
-      W16           -> extend SXTH
-      W8            -> extend SXTB
+        | w' == W32 -> nilOL
+        | otherwise -> unitOL $ SXTH (OpReg w' r) (OpReg w' r)
+      W16           -> unitOL $ SXTH (OpReg w' r) (OpReg w' r)
+      W8            -> unitOL $ SXTB (OpReg w' r) (OpReg w' r)
       _             -> panic "intOp"
-  where
-    noop = return (r, nilOL)
-    extend instr = do
-        r' <- getNewRegNat II64
-        return (r', unitOL $ instr (OpReg w' r') (OpReg w' r))
 
 -- | Instructions to truncate the value in the given register from width @w@
 -- down to width @w'@.
@@ -1316,7 +1306,7 @@ genCondBranch _ true false expr = do
 --
 -- To actually get the value of <symbol>, we'd need to ldr x0, x0 still, which
 -- for the first case can be optimized to use ldr x0, [x0, #:lo12:<symbol>]
--- instaed of the add instruction.
+-- instead of the add instruction.
 --
 -- As the memory model for AArch64 for PIC is considered to be +/- 4GB, we do
 -- not need to go through the GOT, unless we want to address the full address
@@ -1524,6 +1514,9 @@ genCCall target dest_regs arg_regs bid = do
         MO_Memmove _align   -> mkCCall "memmove"
         MO_Memcmp  _align   -> mkCCall "memcmp"
 
+        MO_SuspendThread    -> mkCCall "suspendThread"
+        MO_ResumeThread     -> mkCCall "resumeThread"
+
         MO_PopCnt w         -> mkCCall (popCntLabel w)
         MO_Pdep w           -> mkCCall (pdepLabel w)
         MO_Pext w           -> mkCCall (pextLabel w)
@@ -1546,11 +1539,11 @@ genCCall target dest_regs arg_regs bid = do
     unsupported :: Show a => a -> b
     unsupported mop = panic ("outOfLineCmmOp: " ++ show mop
                           ++ " not supported here")
-    mkCCall :: String -> NatM (InstrBlock, Maybe BlockId)
+    mkCCall :: FastString -> NatM (InstrBlock, Maybe BlockId)
     mkCCall name = do
       config <- getConfig
       target <- cmmMakeDynamicReference config CallReference $
-          mkForeignLabel (fsLit name) Nothing ForeignLabelInThisPackage IsFunction
+          mkForeignLabel name Nothing ForeignLabelInThisPackage IsFunction
       let cconv = ForeignConvention CCallConv [NoHint] [NoHint] CmmMayReturn
       genCCall (ForeignTarget target cconv) dest_regs arg_regs bid
 
