@@ -3,7 +3,7 @@
 (c) The GRASP/AQUA Project, Glasgow University, 1992-1998
 -}
 
-{-# LANGUAGE CPP, DeriveDataTypeable, FlexibleContexts #-}
+{-# LANGUAGE DeriveDataTypeable, FlexibleContexts #-}
 {-# LANGUAGE NamedFieldPuns #-}
 {-# LANGUAGE BangPatterns #-}
 
@@ -46,8 +46,8 @@ module GHC.Core (
         collectNBinders,
         collectArgs, stripNArgs, collectArgsTicks, flattenBinds,
 
-        exprToType, exprToCoercion_maybe,
-        applyTypeToArg,
+        exprToType,
+        wrapLamBody,
 
         isValArg, isTypeArg, isCoArg, isTyCoArg, valArgCount, valBndrCount,
         isRuntimeArg, isRuntimeVar,
@@ -64,8 +64,8 @@ module GHC.Core (
         maybeUnfoldingTemplate, otherCons,
         isValueUnfolding, isEvaldUnfolding, isCheapUnfolding,
         isExpandableUnfolding, isConLikeUnfolding, isCompulsoryUnfolding,
-        isStableUnfolding, hasCoreUnfolding, hasSomeUnfolding,
-        isBootUnfolding,
+        isStableUnfolding, isInlineUnfolding, isBootUnfolding,
+        hasCoreUnfolding, hasSomeUnfolding,
         canUnfold, neverUnfoldGuidance, isStableSource,
 
         -- * Annotated expression data types
@@ -92,8 +92,6 @@ module GHC.Core (
         isBuiltinRule, isLocalRule, isAutoRule,
     ) where
 
-#include "HsVersions.h"
-
 import GHC.Prelude
 import GHC.Platform
 
@@ -103,7 +101,7 @@ import GHC.Core.Type
 import GHC.Core.Coercion
 import GHC.Types.Name
 import GHC.Types.Name.Set
-import GHC.Types.Name.Env( NameEnv, emptyNameEnv )
+import GHC.Types.Name.Env( NameEnv )
 import GHC.Types.Literal
 import GHC.Types.Tickish
 import GHC.Core.DataCon
@@ -115,8 +113,7 @@ import GHC.Utils.Binary
 import GHC.Utils.Misc
 import GHC.Utils.Outputable
 import GHC.Utils.Panic
-
-import GHC.Driver.Ppr
+import GHC.Utils.Panic.Plain
 
 import Data.Data hiding (TyCon)
 import Data.Int
@@ -179,10 +176,10 @@ These data types are the heart of the compiler
 --
 -- *  Applications: note that the argument may be a 'Type'.
 --    See Note [Core let/app invariant]
---    See Note [Levity polymorphism invariants]
+--    See Note [Representation polymorphism invariants]
 --
 -- *  Lambda abstraction
---    See Note [Levity polymorphism invariants]
+--    See Note [Representation polymorphism invariants]
 --
 -- *  Recursive and non recursive @let@s. Operationally
 --    this corresponds to allocating a thunk for the things
@@ -190,7 +187,7 @@ These data types are the heart of the compiler
 --
 --    See Note [Core letrec invariant]
 --    See Note [Core let/app invariant]
---    See Note [Levity polymorphism invariants]
+--    See Note [Representation polymorphism invariants]
 --    See Note [Core type and coercion invariant]
 --
 -- *  Case expression. Operationally this corresponds to evaluating
@@ -300,7 +297,7 @@ data AltCon
 -- The instance adheres to the order described in [Core case invariants]
 instance Ord AltCon where
   compare (DataAlt con1) (DataAlt con2) =
-    ASSERT( dataConTyCon con1 == dataConTyCon con2 )
+    assert (dataConTyCon con1 == dataConTyCon con2) $
     compare (dataConTag con1) (dataConTag con2)
   compare (DataAlt _) _ = GT
   compare _ (DataAlt _) = LT
@@ -510,7 +507,7 @@ checked by Core Lint.
 5. Floating-point values must not be scrutinised against literals.
    See #9238 and Note [Rules for floating-point comparisons]
    in GHC.Core.Opt.ConstantFold for rationale.  Checked in lintCaseExpr;
-   see the call to isFloatingTy.
+   see the call to isFloatingPrimTy.
 
 6. The 'ty' field of (Case scrut bndr ty alts) is the type of the
    /entire/ case expression.  Checked in lintAltExpr.
@@ -548,25 +545,72 @@ Note [Core case invariants]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~
 See Note [Case expression invariants]
 
-Note [Levity polymorphism invariants]
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-The levity-polymorphism invariants are these (as per "Levity Polymorphism",
-PLDI '17):
+Note [Representation polymorphism invariants]
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+GHC allows us to abstract over calling conventions using **representation polymorphism**.
+For example, we have:
 
-* The type of a term-binder must not be levity-polymorphic,
-  unless it is a let(rec)-bound join point
-     (see Note [Invariants on join points])
+  ($) :: forall (r :: RuntimeRep) (a :: Type) (b :: TYPE r). a -> b -> b
 
-* The type of the argument of an App must not be levity-polymorphic.
+In this example, the type `b` is representation-polymorphic: it has kind `TYPE r`,
+where the type variable `r :: RuntimeRep` abstracts over the runtime representation
+of values of type `b`.
 
-A type (t::TYPE r) is "levity polymorphic" if 'r' has any free variables.
+To ensure that programs containing representation-polymorphism remain compilable,
+we enforce the following representation-polymorphism invariants:
+
+The paper "Levity Polymorphism" [PLDI'17] states the first two invariants:
+
+  I1. The type of a bound variable must have a fixed runtime representation
+      (except for join points: See Note [Invariants on join points])
+  I2. The type of a function argument must have a fixed runtime representation.
+
+On top of these two invariants, GHC's internal eta-expansion mechanism also requires:
+
+  I3. In any partial application `f e_1 .. e_n`, where `f` is `hasNoBinding`,
+      it must be the case that the application can be eta-expanded to match
+      the arity of `f`.
+      See Note [checkCanEtaExpand] in GHC.Core.Lint for more details.
+
+Example of I1:
 
-For example
   \(r::RuntimeRep). \(a::TYPE r). \(x::a). e
-is illegal because x's type has kind (TYPE r), which has 'r' free.
 
-See Note [Levity polymorphism checking] in GHC.HsToCore.Monad to see where these
-invariants are established for user-written code.
+    This contravenes I1 because x's type has kind (TYPE r), which has 'r' free.
+    We thus wouldn't know how to compile this lambda abstraction.
+
+Example of I2:
+
+  f (undefined :: (a :: TYPE r))
+
+    This contravenes I2: we are applying the function `f` to a value
+    with an unknown runtime representation.
+
+Examples of I3:
+
+  myUnsafeCoerce# :: forall {r1} (a :: TYPE r1) {r2} (b :: TYPE r2). a -> b
+  myUnsafeCoerce# = unsafeCoerce#
+
+    This contravenes I3: we are instantiating `unsafeCoerce#` without any
+    value arguments, and with a remaining argument type, `a`, which does not
+    have a fixed runtime representation.
+    But `unsafeCorce#` has no binding (see Note [Wiring in unsafeCoerce#]
+    in GHC.HsToCore).  So before code-generation we must saturate it
+    by eta-expansion (see GHC.CoreToStg.Prep.maybeSaturate), thus
+       myUnsafeCoerce# = \x. unsafeCoerce# x
+    But we can't do that because now the \x binding would violate I1.
+
+  bar :: forall (a :: TYPE) r (b :: TYPE r). a -> b
+  bar = unsafeCoerce#
+
+    OK: eta expand to `\ (x :: Type) -> unsafeCoerce# x`,
+    and `x` has a fixed RuntimeRep.
+
+Note that we currently require something slightly stronger than a fixed runtime
+representation: we check whether bound variables and function arguments have a
+/fixed RuntimeRep/ in the sense of Note [Fixed RuntimeRep] in GHC.Tc.Utils.Concrete.
+See Note [Representation polymorphism checking] in GHC.Tc.Utils.Concrete
+for an overview of how we enforce these invariants in the typechecker.
 
 Note [Core let goal]
 ~~~~~~~~~~~~~~~~~~~~
@@ -695,7 +739,6 @@ Join points must follow these invariants:
 
          The arity of a join point isn't very important; but short of setting
          it to zero, it is helpful to have an invariant.  E.g. #17294.
-         See also Note [Do not eta-expand join points] in GHC.Core.Opt.Simplify.Utils.
 
   3. If the binding is recursive, then all other bindings in the recursive group
      must also be join points.
@@ -709,7 +752,7 @@ However, join points have simpler invariants in other ways
      ok-for-speculation (i.e. drop the let/app invariant)
      e.g.  let j :: Int# = factorial x in ...
 
-  6. A join point can have a levity-polymorphic RHS
+  6. The RHS of join point is not required to have a fixed runtime representation,
      e.g.  let j :: r :: TYPE l = fail void# in ...
      This happened in an intermediate program #13394
 
@@ -939,7 +982,7 @@ type MOutCoercion = MCoercion
 -- See Note [Orphans]
 data IsOrphan
   = IsOrphan
-  | NotOrphan OccName -- The OccName 'n' witnesses the instance's non-orphanhood
+  | NotOrphan !OccName -- The OccName 'n' witnesses the instance's non-orphanhood
                       -- In that case, the instance is fingerprinted as part
                       -- of the definition of 'n's definition
     deriving Data
@@ -1041,15 +1084,40 @@ type RuleBase = NameEnv [CoreRule]
 -- but it also includes the set of visible orphans we use to filter out orphan
 -- rules which are not visible (even though we can see them...)
 data RuleEnv
-    = RuleEnv { re_base          :: RuleBase
+    = RuleEnv { re_base          :: [RuleBase] -- See Note [Why re_base is a list]
               , re_visible_orphs :: ModuleSet
               }
 
 mkRuleEnv :: RuleBase -> [Module] -> RuleEnv
-mkRuleEnv rules vis_orphs = RuleEnv rules (mkModuleSet vis_orphs)
+mkRuleEnv rules vis_orphs = RuleEnv [rules] (mkModuleSet vis_orphs)
 
 emptyRuleEnv :: RuleEnv
-emptyRuleEnv = RuleEnv emptyNameEnv emptyModuleSet
+emptyRuleEnv = RuleEnv [] emptyModuleSet
+
+{-
+Note [Why re_base is a list]
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+In Note [Overall plumbing for rules], it is explained that the final
+RuleBase which we must consider is combined from 4 different sources.
+
+During simplifier runs, the fourth source of rules is constantly being updated
+as new interfaces are loaded into the EPS. Therefore just before we check to see
+if any rules match we get the EPS RuleBase and combine it with the existing RuleBase
+and then perform exactly 1 lookup into the new map.
+
+It is more efficient to avoid combining the environments and store the uncombined
+environments as we can instead perform 1 lookup into each environment and then combine
+the results.
+
+Essentially we use the identity:
+
+> lookupNameEnv n (plusNameEnv_C (++) rb1 rb2)
+>   = lookupNameEnv n rb1 ++ lookupNameEnv n rb2
+
+The latter being more efficient as we don't construct an intermediate
+map.
+-}
 
 -- | A 'CoreRule' is:
 --
@@ -1114,7 +1182,7 @@ data CoreRule
                 -- arguments, it simply discards them; the returned 'CoreExpr'
                 -- is just the rewrite of 'ru_fn' applied to the first 'ru_nargs' args
     }
-                -- See Note [Extra args in rule matching] in GHC.Core.Rules
+                -- See Note [Extra args in the target] in GHC.Core.Rules
 
 -- | Rule options
 data RuleOpts = RuleOpts
@@ -1124,6 +1192,8 @@ data RuleOpts = RuleOpts
    , roBignumRules             :: !Bool     -- ^ Enable rules for bignums
    }
 
+-- | The 'InScopeSet' in the 'InScopeEnv' is a /superset/ of variables that are
+-- currently in scope. See Note [The InScopeSet invariant].
 type RuleFun = RuleOpts -> InScopeEnv -> Id -> [CoreExpr] -> Maybe CoreExpr
 type InScopeEnv = (InScopeSet, IdUnfoldingFun)
 
@@ -1176,6 +1246,17 @@ setRuleIdName nm ru = ru { ru_fn = nm }
 ************************************************************************
 
 The @Unfolding@ type is declared here to avoid numerous loops
+
+Note [Never put `OtherCon` unfoldings on lambda binders]
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+Based on #21496 we never attach unfoldings of any kind to lambda binders.
+It's just too easy for the call site to change and invalidate the unfolding.
+E.g. the caller of the lambda drops a seq (e.g. because the lambda is strict in it's binder)
+which in turn makes the OtherCon[] unfolding a lie.
+So unfoldings on lambda binders can never really be trusted when on lambda binders if there
+is the chance of the call site to change. So it's easiest to just never attach any
+to lambda binders to begin with, as well as stripping them off if we e.g. float out
+and expression while abstracting over some arguments.
 -}
 
 -- | Records the /unfolding/ of an identifier, which is approximately the form the
@@ -1458,6 +1539,22 @@ isStableUnfolding (CoreUnfolding { uf_src = src }) = isStableSource src
 isStableUnfolding (DFunUnfolding {})               = True
 isStableUnfolding _                                = False
 
+isInlineUnfolding :: Unfolding -> Bool
+-- ^ True of a /stable/ unfolding that is
+--   (a) always inlined; that is, with an `UnfWhen` guidance, or
+--   (b) a DFunUnfolding which never needs to be inlined
+isInlineUnfolding (CoreUnfolding { uf_src = src, uf_guidance = guidance })
+  | isStableSource src
+  , UnfWhen {} <- guidance
+  = True
+
+isInlineUnfolding (DFunUnfolding {})
+  = True
+
+-- Default case
+isInlineUnfolding _ = False
+
+
 -- | Only returns False if there is no unfolding information available at all
 hasSomeUnfolding :: Unfolding -> Bool
 hasSomeUnfolding NoUnfolding   = False
@@ -1576,9 +1673,7 @@ cmpAltCon (DataAlt _)  DEFAULT      = GT
 cmpAltCon (LitAlt  l1) (LitAlt  l2) = l1 `compare` l2
 cmpAltCon (LitAlt _)   DEFAULT      = GT
 
-cmpAltCon con1 con2 = WARN( True, text "Comparing incomparable AltCons" <+>
-                                  ppr con1 <+> ppr con2 )
-                      LT
+cmpAltCon con1 con2 = pprPanic "cmpAltCon" (ppr con1 $$ ppr con2)
 
 {-
 ************************************************************************
@@ -1804,7 +1899,7 @@ mkCoBind cv co      = NonRec cv (Coercion co)
 varToCoreExpr :: CoreBndr -> Expr b
 varToCoreExpr v | isTyVar v = Type (mkTyVarTy v)
                 | isCoVar v = Coercion (mkCoVarCo v)
-                | otherwise = ASSERT( isId v ) Var v
+                | otherwise = assert (isId v) $ Var v
 
 varsToCoreExprs :: [CoreBndr] -> [Expr b]
 varsToCoreExprs vs = map varToCoreExpr vs
@@ -1820,22 +1915,12 @@ These are defined here to avoid a module loop between GHC.Core.Utils and GHC.Cor
 
 -}
 
-applyTypeToArg :: Type -> CoreExpr -> Type
--- ^ Determines the type resulting from applying an expression with given type
--- to a given argument expression
-applyTypeToArg fun_ty arg = piResultTy fun_ty (exprToType arg)
-
 -- | If the expression is a 'Type', converts. Otherwise,
 -- panics. NB: This does /not/ convert 'Coercion' to 'CoercionTy'.
 exprToType :: CoreExpr -> Type
 exprToType (Type ty)     = ty
 exprToType _bad          = pprPanic "exprToType" empty
 
--- | If the expression is a 'Coercion', converts.
-exprToCoercion_maybe :: CoreExpr -> Maybe Coercion
-exprToCoercion_maybe (Coercion co) = Just co
-exprToCoercion_maybe _             = Nothing
-
 {-
 ************************************************************************
 *                                                                      *
@@ -1920,6 +2005,14 @@ collectArgs expr
     go (App f a) as = go f (a:as)
     go e         as = (e, as)
 
+-- | fmap on the body of a lambda.
+--   wrapLamBody f (\x -> body) == (\x -> f body)
+wrapLamBody :: (CoreExpr -> CoreExpr) -> CoreExpr -> CoreExpr
+wrapLamBody f expr = go expr
+  where
+  go (Lam v body) = Lam v $ go body
+  go expr = f expr
+
 -- | Attempt to remove the last N arguments of a function call.
 -- Strip off any ticks or coercions encountered along the way and any
 -- at the end.
