@@ -1,4 +1,3 @@
-{-# LANGUAGE CPP                 #-}
 {-# LANGUAGE DataKinds           #-}
 {-# LANGUAGE DeriveDataTypeable  #-}
 {-# LANGUAGE GADTs               #-}
@@ -47,6 +46,7 @@ import GHC.Types.Var
 import GHC.Utils.Misc
 import GHC.Utils.Binary
 import GHC.Utils.Panic
+import GHC.Utils.Panic.Plain
 import GHC.Data.Pair
 import GHC.Types.Basic
 import Data.Typeable ( Typeable )
@@ -55,8 +55,6 @@ import qualified Data.Data as Data
 import Data.Array
 import Data.List ( mapAccumL )
 
-#include "HsVersions.h"
-
 {-
 Note [Coercion axiom branches]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -143,7 +141,7 @@ newtype Branches (br :: BranchFlag)
 type role Branches nominal
 
 manyBranches :: [CoAxBranch] -> Branches Branched
-manyBranches brs = ASSERT( snd bnds >= fst bnds )
+manyBranches brs = assert (snd bnds >= fst bnds )
                    MkBranches (listArray bnds brs)
   where
     bnds = (0, length brs - 1)
@@ -155,7 +153,7 @@ toBranched :: Branches br -> Branches Branched
 toBranched = MkBranches . unMkBranches
 
 toUnbranched :: Branches br -> Branches Unbranched
-toUnbranched (MkBranches arr) = ASSERT( bounds arr == (0,0) )
+toUnbranched (MkBranches arr) = assert (bounds arr == (0,0) )
                                 MkBranches arr
 
 fromBranches :: Branches br -> [CoAxBranch]
@@ -325,7 +323,7 @@ isImplicitCoAxiom = co_ax_implicit
 coAxBranchIncomps :: CoAxBranch -> [CoAxBranch]
 coAxBranchIncomps = cab_incomps
 
--- See Note [Compatibility checking] in GHC.Core.FamInstEnv
+-- See Note [Compatibility] in GHC.Core.FamInstEnv
 placeHolderIncomps :: [CoAxBranch]
 placeHolderIncomps = panic "placeHolderIncomps"
 
@@ -457,6 +455,7 @@ See also:
   type; but it too is eta-reduced.
 * Note [Implementing eta reduction for data families] in "GHC.Tc.TyCl.Instance". This
   describes the implementation details of this eta reduction happen.
+* Note [RoughMap and rm_empty] for how this complicates the RoughMap implementation slightly.
 -}
 
 instance Eq (CoAxiom br) where
