@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP #-}
+
 
 {-
 (c) The University of Glasgow 2006
@@ -10,8 +10,6 @@ Matching guarded right-hand-sides (GRHSs)
 
 module GHC.HsToCore.GuardedRHSs ( dsGuarded, dsGRHSs, isTrueLHsExpr ) where
 
-#include "HsVersions.h"
-
 import GHC.Prelude
 
 import {-# SOURCE #-} GHC.HsToCore.Expr  ( dsLExpr, dsLocalBinds )
@@ -30,6 +28,7 @@ import GHC.Utils.Misc
 import GHC.Types.SrcLoc
 import GHC.Utils.Outputable
 import GHC.Utils.Panic
+import GHC.Utils.Panic.Plain
 import GHC.Core.Multiplicity
 import Control.Monad ( zipWithM )
 import Data.List.NonEmpty ( NonEmpty, toList )
@@ -50,7 +49,8 @@ necessary.  The type argument gives the type of the @ei@.
 dsGuarded :: GRHSs GhcTc (LHsExpr GhcTc) -> Type -> NonEmpty Nablas -> DsM CoreExpr
 dsGuarded grhss rhs_ty rhss_nablas = do
     match_result <- dsGRHSs PatBindRhs grhss rhs_ty rhss_nablas
-    error_expr <- mkErrorAppDs nON_EXHAUSTIVE_GUARDS_ERROR_ID rhs_ty empty
+    error_expr <- mkErrorAppDs nON_EXHAUSTIVE_GUARDS_ERROR_ID rhs_ty
+                               (text "pattern binding")
     extractMatchResult match_result error_expr
 
 -- In contrast, @dsGRHSs@ produces a @MatchResult CoreExpr@.
@@ -63,8 +63,8 @@ dsGRHSs :: HsMatchContext GhcRn
                                        --   one for each GRHS.
         -> DsM (MatchResult CoreExpr)
 dsGRHSs hs_ctx (GRHSs _ grhss binds) rhs_ty rhss_nablas
-  = ASSERT( notNull grhss )
-    do { match_results <- ASSERT( length grhss == length rhss_nablas )
+  = assert (notNull grhss) $
+    do { match_results <- assert (length grhss == length rhss_nablas) $
                           zipWithM (dsGRHS hs_ctx rhs_ty) (toList rhss_nablas) grhss
        ; nablas <- getPmNablas
        -- We need to remember the Nablas from the particular match context we
