@@ -155,6 +155,7 @@ type HieTypeFlat = HieType TypeIndex
 
 -- | Roughly isomorphic to the original core 'Type'.
 newtype HieTypeFix = Roll (HieType (HieTypeFix))
+  deriving Eq
 
 instance Binary (HieType TypeIndex) where
   put_ bh (HTyVarTy n) = do
@@ -305,7 +306,7 @@ data NodeAnnotation = NodeAnnotation
 
 instance Ord NodeAnnotation where
    compare (NodeAnnotation c0 t0) (NodeAnnotation c1 t1)
-      = mconcat [uniqCompareFS c0 c1, uniqCompareFS t0 t1]
+      = mconcat [lexicalCompareFS c0 c1, lexicalCompareFS t0 t1]
 
 instance Outputable NodeAnnotation where
    ppr (NodeAnnotation c t) = ppr (c,t)
@@ -780,5 +781,5 @@ toHieName name
   | isKnownKeyName name = KnownKeyName (nameUnique name)
   | isExternalName name = ExternalName (nameModule name)
                                        (nameOccName name)
-                                       (removeBufSpan $ nameSrcSpan name)
-  | otherwise = LocalName (nameOccName name) (removeBufSpan $ nameSrcSpan name)
+                                       (nameSrcSpan name)
+  | otherwise = LocalName (nameOccName name) (nameSrcSpan name)
