@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP #-}
+
 
 {-# OPTIONS_GHC -Wno-incomplete-record-updates #-}
 
@@ -14,8 +14,6 @@ module GHC.Iface.Rename (
     tcRnModExports,
     ) where
 
-#include "HsVersions.h"
-
 import GHC.Prelude
 
 import GHC.Driver.Env
@@ -27,10 +25,10 @@ import GHC.Iface.Env
 import {-# SOURCE #-} GHC.Iface.Load -- a bit vexing
 
 import GHC.Unit
-import GHC.Unit.State
 import GHC.Unit.Module.ModIface
 import GHC.Unit.Module.Deps
 
+import GHC.Tc.Errors.Types
 import GHC.Types.SrcLoc
 import GHC.Types.Unique.FM
 import GHC.Types.Avail
@@ -43,21 +41,20 @@ import GHC.Types.Name.Shape
 
 import GHC.Utils.Outputable
 import GHC.Utils.Misc
+import GHC.Utils.Error
 import GHC.Utils.Fingerprint
 import GHC.Utils.Panic
 
-import GHC.Data.Bag
-
 import qualified Data.Traversable as T
 
 import Data.IORef
 
-tcRnMsgMaybe :: IO (Either ErrorMessages a) -> TcM a
+tcRnMsgMaybe :: IO (Either (Messages TcRnMessage) a) -> TcM a
 tcRnMsgMaybe do_this = do
     r <- liftIO $ do_this
     case r of
-        Left errs -> do
-            addMessages (mkMessages errs)
+        Left msgs -> do
+            addMessages msgs
             failM
         Right x -> return x
 
@@ -71,12 +68,13 @@ tcRnModExports x y = do
     hsc_env <- getTopEnv
     tcRnMsgMaybe $ rnModExports hsc_env x y
 
-failWithRn :: SDoc -> ShIfM a
-failWithRn doc = do
+failWithRn :: TcRnMessage -> ShIfM a
+failWithRn tcRnMessage = do
     errs_var <- fmap sh_if_errs getGblEnv
     errs <- readTcRef errs_var
     -- TODO: maybe associate this with a source location?
-    writeTcRef errs_var (errs `snocBag` mkPlainMsgEnvelope noSrcSpan doc)
+    let msg = mkPlainErrorMsgEnvelope noSrcSpan tcRnMessage
+    writeTcRef errs_var (msg `addMessage` errs)
     failM
 
 -- | What we have is a generalized ModIface, which corresponds to
@@ -100,7 +98,7 @@ failWithRn doc = do
 -- should be Foo.T; then we'll also rename this (this is used
 -- when loading an interface to merge it into a requirement.)
 rnModIface :: HscEnv -> [(ModuleName, Module)] -> Maybe NameShape
-           -> ModIface -> IO (Either ErrorMessages ModIface)
+           -> ModIface -> IO (Either (Messages TcRnMessage) ModIface)
 rnModIface hsc_env insts nsubst iface =
     initRnIface hsc_env iface insts nsubst $ do
         mod <- rnModule (mi_module iface)
@@ -124,25 +122,24 @@ rnModIface hsc_env insts nsubst iface =
 
 -- | Rename just the exports of a 'ModIface'.  Useful when we're doing
 -- shaping prior to signature merging.
-rnModExports :: HscEnv -> [(ModuleName, Module)] -> ModIface -> IO (Either ErrorMessages [AvailInfo])
+rnModExports :: HscEnv -> [(ModuleName, Module)] -> ModIface -> IO (Either (Messages TcRnMessage) [AvailInfo])
 rnModExports hsc_env insts iface
     = initRnIface hsc_env iface insts Nothing
     $ mapM rnAvailInfo (mi_exports iface)
 
 rnDependencies :: Rename Dependencies
-rnDependencies deps = do
-    orphs  <- rnDepModules dep_orphs deps
-    finsts <- rnDepModules dep_finsts deps
-    return deps { dep_orphs = orphs, dep_finsts = finsts }
+rnDependencies deps0 = do
+    deps1  <- dep_orphs_update deps0 (rnDepModules dep_orphs)
+    dep_finsts_update deps1 (rnDepModules dep_finsts)
 
-rnDepModules :: (Dependencies -> [Module]) -> Dependencies -> ShIfM [Module]
-rnDepModules sel deps = do
+rnDepModules :: (Dependencies -> [Module]) -> [Module] -> ShIfM [Module]
+rnDepModules sel mods = do
     hsc_env <- getTopEnv
     hmap <- getHoleSubst
     -- NB: It's not necessary to test if we're doing signature renaming,
     -- because ModIface will never contain module reference for itself
     -- in these dependencies.
-    fmap (nubSort . concat) . T.forM (sel deps) $ \mod -> do
+    fmap (nubSort . concat) . T.forM mods $ \mod -> do
         -- For holes, its necessary to "see through" the instantiation
         -- of the hole to get accurate family instance dependencies.
         -- For example, if B imports <A>, and <A> is instantiated with
@@ -184,9 +181,9 @@ rnDepModules sel deps = do
 
 -- | Run a computation in the 'ShIfM' monad.
 initRnIface :: HscEnv -> ModIface -> [(ModuleName, Module)] -> Maybe NameShape
-            -> ShIfM a -> IO (Either ErrorMessages a)
+            -> ShIfM a -> IO (Either (Messages TcRnMessage) a)
 initRnIface hsc_env iface insts nsubst do_this = do
-    errs_var <- newIORef emptyBag
+    errs_var <- newIORef emptyMessages
     let hsubst = listToUFM insts
         rn_mod = renameHoleModule (hsc_units hsc_env) hsubst
         env = ShIfEnv {
@@ -200,9 +197,9 @@ initRnIface hsc_env iface insts nsubst do_this = do
     res <- initTcRnIf 'c' hsc_env env () $ tryM do_this
     msgs <- readIORef errs_var
     case res of
-        Left _                          -> return (Left msgs)
-        Right r | not (isEmptyBag msgs) -> return (Left msgs)
-                | otherwise             -> return (Right r)
+        Left _                               -> return (Left msgs)
+        Right r | not (isEmptyMessages msgs) -> return (Left msgs)
+                | otherwise                  -> return (Right r)
 
 -- | Environment for 'ShIfM' monads.
 data ShIfEnv = ShIfEnv {
@@ -220,8 +217,8 @@ data ShIfEnv = ShIfEnv {
         -- we just load the target interface and look at the export
         -- list to determine the renaming.
         sh_if_shape :: Maybe NameShape,
-        -- Mutable reference to keep track of errors (similar to 'tcl_errs')
-        sh_if_errs :: IORef ErrorMessages
+        -- Mutable reference to keep track of diagnostics (similar to 'tcl_errs')
+        sh_if_errs :: IORef (Messages TcRnMessage)
     }
 
 getHoleSubst :: ShIfM ShHoleSubst
@@ -248,7 +245,8 @@ rnAvailInfo (AvailTC n ns) = do
     ns' <- mapM rnGreName ns
     case ns' of
         [] -> panic "rnAvailInfoEmpty AvailInfo"
-        (rep:rest) -> ASSERT2( all ((== childModule rep) . childModule) rest, ppr rep $$ hcat (map ppr rest) ) do
+        (rep:rest) -> assertPpr (all ((== childModule rep) . childModule) rest)
+                                (ppr rep $$ hcat (map ppr rest)) $ do
                          n' <- setNameModule (Just (childModule rep)) n
                          return (AvailTC n' ns')
   where
@@ -328,11 +326,8 @@ rnIfaceGlobal n = do
                         -- TODO: This will give an unpleasant message if n'
                         -- is a constructor; then we'll suggest adding T
                         -- but it won't work.
-                        Nothing -> failWithRn $ vcat [
-                            text "The identifier" <+> ppr (occName n') <+>
-                                text "does not exist in the local signature.",
-                            parens (text "Try adding it to the export list of the hsig file.")
-                            ]
+                        Nothing ->
+                          failWithRn $ TcRnIdNotExportedFromLocalSig n'
                         Just n'' -> return n''
        -- Fastpath: we are renaming p[H=<H>]:A.T, in which case the
        -- export list is irrelevant.
@@ -355,12 +350,8 @@ rnIfaceGlobal n = do
                             $ loadSysInterface (text "rnIfaceGlobal") m''
             let nsubst = mkNameShape (moduleName m) (mi_exports iface)
             case maybeSubstNameShape nsubst n of
-                Nothing -> failWithRn $ vcat [
-                    text "The identifier" <+> ppr (occName n) <+>
-                        -- NB: report m' because it's more user-friendly
-                        text "does not exist in the signature for" <+> ppr m',
-                    parens (text "Try adding it to the export list in that hsig file.")
-                    ]
+                -- NB: report m' because it's more user-friendly
+                Nothing -> failWithRn $ TcRnIdNotExportedFromModuleSig n m'
                 Just n' -> return n'
 
 -- | Rename an implicit name, e.g., a DFun or coercion axiom.
@@ -373,7 +364,7 @@ rnIfaceNeverExported name = do
     iface_semantic_mod <- fmap sh_if_semantic_module getGblEnv
     let m = renameHoleModule unit_state hmap $ nameModule name
     -- Doublecheck that this DFun/coercion axiom was, indeed, locally defined.
-    MASSERT2( iface_semantic_mod == m, ppr iface_semantic_mod <+> ppr m )
+    massertPpr (iface_semantic_mod == m) (ppr iface_semantic_mod <+> ppr m)
     setNameModule (Just m) name
 
 -- Note [rnIfaceNeverExported]
