@@ -6,8 +6,9 @@
 Type checking of type signatures in interface files
 -}
 
-{-# LANGUAGE CPP #-}
+
 {-# LANGUAGE NondecreasingIndentation #-}
+{-# LANGUAGE FlexibleContexts #-}
 
 {-# OPTIONS_GHC -Wno-incomplete-record-updates #-}
 
@@ -24,8 +25,6 @@ module GHC.IfaceToCore (
         tcIfaceOneShot
  ) where
 
-#include "HsVersions.h"
-
 import GHC.Prelude
 
 import GHC.Driver.Env
@@ -39,7 +38,9 @@ import GHC.Iface.Load
 import GHC.Iface.Env
 
 import GHC.StgToCmm.Types
+import GHC.Runtime.Heap.Layout
 
+import GHC.Tc.Errors.Types
 import GHC.Tc.TyCl.Build
 import GHC.Tc.Utils.Monad
 import GHC.Tc.Utils.TcType
@@ -65,6 +66,7 @@ import GHC.Core.DataCon
 import GHC.Core.Opt.OccurAnal ( occurAnalyseExpr )
 import GHC.Core.Ppr
 
+import GHC.Unit.Env
 import GHC.Unit.External
 import GHC.Unit.Module
 import GHC.Unit.Module.ModDetails
@@ -74,6 +76,8 @@ import GHC.Unit.Home.ModInfo
 import GHC.Utils.Outputable
 import GHC.Utils.Misc
 import GHC.Utils.Panic
+import GHC.Utils.Panic.Plain
+import GHC.Utils.Constants (debugIsOn)
 import GHC.Utils.Logger
 
 import GHC.Data.Bag
@@ -102,12 +106,14 @@ import GHC.Types.Id.Make
 import GHC.Types.Id.Info
 import GHC.Types.Tickish
 import GHC.Types.TyThing
+import GHC.Types.Error
 
 import GHC.Fingerprint
 import qualified GHC.Data.BooleanFormula as BF
 
 import Control.Monad
 import GHC.Parser.Annotation
+import GHC.Driver.Env.KnotVars
 
 {-
 This module takes
@@ -156,7 +162,7 @@ a Name for another entity defined in A.hi.  How do we get the
     internal TyCons to MATCH the ones that we just constructed
     during typechecking: the knot is thus tied through if_rec_types.
 
-    2) retypecheckLoop in GHC.Driver.Make: We are retypechecking a
+    2) rehydrate in GHC.Driver.Make: We are rehydrating a
     mutually recursive cluster of hi files, in order to ensure
     that all of the references refer to each other correctly.
     In this case, the knot is tied through the HPT passed in,
@@ -215,7 +221,7 @@ typecheckIface iface
                          -- an example where this would cause non-termination.
                          text "Type envt:" <+> ppr (map fst names_w_things)])
         ; return $ ModDetails { md_types     = type_env
-                              , md_insts     = insts
+                              , md_insts     = mkInstEnv insts
                               , md_fam_insts = fam_insts
                               , md_rules     = rules
                               , md_anns      = anns
@@ -234,7 +240,7 @@ typecheckIface iface
 
 -- | Returns true if an 'IfaceDecl' is for @data T@ (an abstract data type)
 isAbstractIfaceDecl :: IfaceDecl -> Bool
-isAbstractIfaceDecl IfaceData{ ifCons = IfAbstractTyCon } = True
+isAbstractIfaceDecl IfaceData{ ifCons = IfAbstractTyCon {} } = True
 isAbstractIfaceDecl IfaceClass{ ifBody = IfAbstractClass } = True
 isAbstractIfaceDecl IfaceFamily{ ifFamFlav = IfaceAbstractClosedSynFamilyTyCon } = True
 isAbstractIfaceDecl _ = False
@@ -254,7 +260,7 @@ mergeIfaceDecl d1 d2
     | isAbstractIfaceDecl d2 = d1 `withRolesFrom` d2
     | IfaceClass{ ifBody = IfConcreteClass { ifSigs = ops1, ifMinDef = bf1 } } <- d1
     , IfaceClass{ ifBody = IfConcreteClass { ifSigs = ops2, ifMinDef = bf2 } } <- d2
-    = let ops = nameEnvElts $
+    = let ops = nonDetNameEnvElts $
                   plusNameEnv_C mergeIfaceClassOp
                     (mkNameEnv [ (n, op) | op@(IfaceClassOp n _ _) <- ops1 ])
                     (mkNameEnv [ (n, op) | op@(IfaceClassOp n _ _) <- ops2 ])
@@ -379,8 +385,8 @@ mergeIfaceDecls = plusOccEnv_C mergeIfaceDecl
 -- type synonym.  Perhaps this should be relaxed, where a type synonym
 -- in a signature is considered implemented by a data type declaration
 -- which matches the reference of the type synonym.
-typecheckIfacesForMerging :: Module -> [ModIface] -> IORef TypeEnv -> IfM lcl (TypeEnv, [ModDetails])
-typecheckIfacesForMerging mod ifaces tc_env_var =
+typecheckIfacesForMerging :: Module -> [ModIface] -> (KnotVars (IORef TypeEnv)) -> IfM lcl (TypeEnv, [ModDetails])
+typecheckIfacesForMerging mod ifaces tc_env_vars =
   -- cannot be boot (False)
   initIfaceLcl mod (text "typecheckIfacesForMerging") NotBoot $ do
     ignore_prags <- goptM Opt_IgnoreInterfacePragmas
@@ -400,9 +406,11 @@ typecheckIfacesForMerging mod ifaces tc_env_var =
                         ::  OccEnv IfaceDecl
     -- TODO: change tcIfaceDecls to accept w/o Fingerprint
     names_w_things <- tcIfaceDecls ignore_prags (map (\x -> (fingerprint0, x))
-                                                  (occEnvElts decl_env))
+                                                  (nonDetOccEnvElts decl_env))
     let global_type_env = mkNameEnv names_w_things
-    writeMutVar tc_env_var global_type_env
+    case lookupKnotVars tc_env_vars mod of
+      Just tc_env_var -> writeMutVar tc_env_var global_type_env
+      Nothing -> return ()
 
     -- OK, now typecheck each ModIface using this environment
     details <- forM ifaces $ \iface -> do
@@ -421,7 +429,7 @@ typecheckIfacesForMerging mod ifaces tc_env_var =
         exports   <- ifaceExportNames (mi_exports iface)
         complete_matches <- tcIfaceCompleteMatches (mi_complete_matches iface)
         return $ ModDetails { md_types     = type_env
-                            , md_insts     = insts
+                            , md_insts     = mkInstEnv insts
                             , md_fam_insts = fam_insts
                             , md_rules     = rules
                             , md_anns      = anns
@@ -460,7 +468,7 @@ typecheckIfaceForInstantiate nsubst iface =
     exports   <- ifaceExportNames (mi_exports iface)
     complete_matches <- tcIfaceCompleteMatches (mi_complete_matches iface)
     return $ ModDetails { md_types     = type_env
-                        , md_insts     = insts
+                        , md_insts     = mkInstEnv insts
                         , md_fam_insts = fam_insts
                         , md_rules     = rules
                         , md_anns      = anns
@@ -532,8 +540,8 @@ tcHiBootIface hsc_src mod
                 -- (it's been replaced by the mother module) so we can't check it.
                 -- And that's fine, because if M's ModInfo is in the HPT, then
                 -- it's been compiled once, and we don't need to check the boot iface
-          then do { hpt <- getHpt
-                 ; case lookupHpt hpt (moduleName mod) of
+          then do { (_, hug) <- getEpsAndHug
+                 ; case lookupHugByModule mod hug  of
                       Just info | mi_boot (hm_iface info) == IsBoot
                                 -> mkSelfBootInfo (hm_iface info) (hm_details info)
                       _ -> return NoSelfBoot }
@@ -543,7 +551,8 @@ tcHiBootIface hsc_src mod
         -- Re #9245, we always check if there is an hi-boot interface
         -- to check consistency against, rather than just when we notice
         -- that an hi-boot is necessary due to a circular import.
-        { read_result <- findAndReadIface
+        { hsc_env <- getTopEnv
+        ; read_result <- liftIO $ findAndReadIface hsc_env
                                 need (fst (getModuleInstantiation mod)) mod
                                 IsBoot  -- Hi-boot file
 
@@ -560,14 +569,14 @@ tcHiBootIface hsc_src mod
         -- a SOURCE import) or that our hi-boot file has mysteriously
         -- disappeared.
     do  { eps <- getEps
-        ; case lookupUFM (eps_is_boot eps) (moduleName mod) of
+        ; case lookupInstalledModuleEnv (eps_is_boot eps) (toUnitId <$> mod) of
             -- The typical case
             Nothing -> return NoSelfBoot
             -- error cases
             Just (GWIB { gwib_isBoot = is_boot }) -> case is_boot of
-              IsBoot -> failWithTc (elaborate err)
+              IsBoot -> failWithTc (TcRnUnknownMessage $ mkPlainError noHints (elaborate err))
               -- The hi-boot file has mysteriously disappeared.
-              NotBoot -> failWithTc moduleLoop
+              NotBoot -> failWithTc (TcRnUnknownMessage $ mkPlainError noHints moduleLoop)
               -- Someone below us imported us!
               -- This is a loop with no hi-boot in the way
     }}}}
@@ -594,7 +603,7 @@ mkSelfBootInfo iface mds
        return $ SelfBoot { sb_mds = mds
                          , sb_tcs = mkNameSet tcs }
   where
-    -- | Retuerns @True@ if, when you call 'tcIfaceDecl' on
+    -- Retuerns @True@ if, when you call 'tcIfaceDecl' on
     -- this 'IfaceDecl', an ATyCon would be returned.
     -- NB: This code assumes that a TyCon cannot be implicit.
     isIfaceTyCon IfaceId{}      = False
@@ -1021,11 +1030,17 @@ tc_ax_branch prev_branches
 tcIfaceDataCons :: Name -> TyCon -> [TyConBinder] -> IfaceConDecls -> IfL AlgTyConRhs
 tcIfaceDataCons tycon_name tycon tc_tybinders if_cons
   = case if_cons of
-        IfAbstractTyCon  -> return AbstractTyCon
-        IfDataTyCon cons -> do  { data_cons  <- mapM tc_con_decl cons
-                                ; return (mkDataTyConRhs data_cons) }
-        IfNewTyCon  con  -> do  { data_con  <- tc_con_decl con
-                                ; mkNewTyConRhs tycon_name tycon data_con }
+        IfAbstractTyCon
+          -> return AbstractTyCon
+        IfDataTyCon cons
+          -> do  { data_cons  <- mapM tc_con_decl cons
+                 ; return $
+                     mkLevPolyDataTyConRhs
+                       (isFixedRuntimeRepKind $ tyConResKind tycon)
+                       data_cons }
+        IfNewTyCon con
+          -> do  { data_con  <- tc_con_decl con
+                 ; mkNewTyConRhs tycon_name tycon data_con }
   where
     univ_tvs :: [TyVar]
     univ_tvs = binderVars tc_tybinders
@@ -1088,15 +1103,15 @@ tcIfaceDataCons tycon_name tycon tc_tybinders if_cons
 
         ; prom_rep_name <- newTyConRepName dc_name
 
+        ; let bang_opts = FixedBangOpts stricts
+            -- Pass the HsImplBangs (i.e. final decisions) to buildDataCon;
+            -- it'll use these to guide the construction of a worker.
+            -- See Note [Bangs on imported data constructors] in GHC.Types.Id.Make
+
         ; con <- buildDataCon (pprPanic "tcIfaceDataCons: FamInstEnvs" (ppr dc_name))
+                       bang_opts
                        dc_name is_infix prom_rep_name
                        (map src_strict if_src_stricts)
-                       (Just stricts)
-                       -- Pass the HsImplBangs (i.e. final
-                       -- decisions) to buildDataCon; it'll use
-                       -- these to guide the construction of a
-                       -- worker.
-                       -- See Note [Bangs on imported data constructors] in GHC.Types.Id.Make
                        lbl_names
                        univ_tvs ex_tvs user_tv_bndrs
                        eq_spec theta
@@ -1150,8 +1165,8 @@ look at it.
 -}
 
 tcRoughTyCon :: Maybe IfaceTyCon -> RoughMatchTc
-tcRoughTyCon (Just tc) = KnownTc (ifaceTyConName tc)
-tcRoughTyCon Nothing   = OtherTc
+tcRoughTyCon (Just tc) = RM_KnownTc (ifaceTyConName tc)
+tcRoughTyCon Nothing   = RM_WildCard
 
 tcIfaceInst :: IfaceClsInst -> IfL ClsInst
 tcIfaceInst (IfaceClsInst { ifDFun = dfun_name, ifOFlag = oflag
@@ -1213,7 +1228,7 @@ tcIfaceRule (IfaceRule {ifRuleName = name, ifActivation = act, ifRuleBndrs = bnd
                           Nothing   -> return ()
                           Just errs -> do
                             logger <- getLogger
-                            liftIO $ displayLintResults logger dflags False doc
+                            liftIO $ displayLintResults logger False doc
                                                (pprCoreExpr rhs')
                                                (emptyBag, errs) }
                    ; return (bndrs', args', rhs') }
@@ -1294,11 +1309,10 @@ tcIfaceCompleteMatch (IfaceCompleteMatch ms mtc) = forkM doc $ do -- See Note [P
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 We need to be lazy when type checking the interface, since these functions are
 called when the interface itself is being loaded, which means it is not in the
-PIT yet. If we are not lazy enough, in certain cases we might recursively try to
-load the same interface in an infinite loop.
-
-For this reason, the forkM should be around as much of the computation as
-possible.
+PIT yet. In particular, the `tcIfaceTCon` must be inside the forkM, otherwise
+we'll try to look it up the TyCon, find it's not there, and so initiate the
+process (again) of loading the (very same) interface file. Result: infinite
+loop. See #19744.
 -}
 
 {-
@@ -1458,8 +1472,7 @@ tcIfaceExpr (IfaceLit lit)
 tcIfaceExpr (IfaceFCall cc ty) = do
     ty' <- tcIfaceType ty
     u <- newUnique
-    dflags <- getDynFlags
-    return (Var (mkFCallId dflags u cc ty'))
+    return (Var (mkFCallId u cc ty'))
 
 tcIfaceExpr (IfaceTuple sort args)
   = do { args' <- mapM tcIfaceExpr args
@@ -1564,12 +1577,12 @@ tcIfaceAlt :: CoreExpr -> Mult -> (TyCon, [Type])
            -> IfaceAlt
            -> IfL CoreAlt
 tcIfaceAlt _ _ _ (IfaceAlt IfaceDefault names rhs)
-  = ASSERT( null names ) do
+  = assert (null names) $ do
     rhs' <- tcIfaceExpr rhs
     return (Alt DEFAULT [] rhs')
 
 tcIfaceAlt _ _ _ (IfaceAlt (IfaceLitAlt lit) names rhs)
-  = ASSERT( null names ) do
+  = assert (null names) $ do
     lit' <- tcIfaceLit lit
     rhs' <- tcIfaceExpr rhs
     return (Alt (LitAlt lit') [] rhs')
@@ -1606,6 +1619,7 @@ tcIfaceDataAlt mult con inst_tys arg_strs rhs
 
 tcIdDetails :: Type -> IfaceIdDetails -> IfL IdDetails
 tcIdDetails _  IfVanillaId = return VanillaId
+tcIdDetails _  (IfWorkerLikeId dmds) = return $ WorkerLikeId dmds
 tcIdDetails ty IfDFunId
   = return (DFunId (isNewTyCon (classTyCon cls)))
   where
@@ -1645,14 +1659,17 @@ tcIdInfo ignore_prags toplvl name ty info = do
     tcPrag :: IdInfo -> IfaceInfoItem -> IfL IdInfo
     tcPrag info HsNoCafRefs        = return (info `setCafInfo`   NoCafRefs)
     tcPrag info (HsArity arity)    = return (info `setArityInfo` arity)
-    tcPrag info (HsStrictness str) = return (info `setStrictnessInfo` str)
-    tcPrag info (HsCpr cpr)        = return (info `setCprInfo` cpr)
+    tcPrag info (HsDmdSig str)     = return (info `setDmdSigInfo` str)
+    tcPrag info (HsCprSig cpr)     = return (info `setCprSigInfo` cpr)
     tcPrag info (HsInline prag)    = return (info `setInlinePragInfo` prag)
-    tcPrag info HsLevity           = return (info `setNeverLevPoly` ty)
+    tcPrag info HsLevity           = return (info `setNeverRepPoly` ty)
     tcPrag info (HsLFInfo lf_info) = do
       lf_info <- tcLFInfo lf_info
       return (info `setLFInfo` lf_info)
 
+    tcPrag info (HsTagSig sig) = do
+      return (info `setTagSig` sig)
+
         -- The next two are lazy, so they don't transitively suck stuff in
     tcPrag info (HsUnfold lb if_unf)
       = do { unf <- tcUnfolding toplvl name ty info if_unf
@@ -1697,53 +1714,67 @@ tcLFInfo lfi = case lfi of
       return (LFUnknown fun_flag)
 
 tcUnfolding :: TopLevelFlag -> Name -> Type -> IdInfo -> IfaceUnfolding -> IfL Unfolding
+-- See Note [Lazily checking Unfoldings]
 tcUnfolding toplvl name _ info (IfCoreUnfold stable if_expr)
   = do  { uf_opts <- unfoldingOpts <$> getDynFlags
-        ; mb_expr <- tcPragExpr False toplvl name if_expr
+        ; expr <- tcUnfoldingRhs False toplvl name if_expr
         ; let unf_src | stable    = InlineStable
                       | otherwise = InlineRhs
-        ; return $ case mb_expr of
-            Nothing -> NoUnfolding
-            Just expr -> mkFinalUnfolding uf_opts unf_src strict_sig expr
-        }
+        ; return $ mkFinalUnfolding uf_opts unf_src strict_sig expr }
   where
     -- Strictness should occur before unfolding!
-    strict_sig = strictnessInfo info
+    strict_sig = dmdSigInfo info
 
 tcUnfolding toplvl name _ _ (IfCompulsory if_expr)
-  = do  { mb_expr <- tcPragExpr True toplvl name if_expr
-        ; return (case mb_expr of
-                    Nothing   -> NoUnfolding
-                    Just expr -> mkCompulsoryUnfolding' expr) }
+  = do  { expr <- tcUnfoldingRhs True toplvl name if_expr
+        ; return $ mkCompulsoryUnfolding' expr }
 
 tcUnfolding toplvl name _ _ (IfInlineRule arity unsat_ok boring_ok if_expr)
-  = do  { mb_expr <- tcPragExpr False toplvl name if_expr
-        ; return (case mb_expr of
-                    Nothing   -> NoUnfolding
-                    Just expr -> mkCoreUnfolding InlineStable True expr guidance )}
+  = do  { expr <- tcUnfoldingRhs False toplvl name if_expr
+        ; return $ mkCoreUnfolding InlineStable True expr guidance }
   where
     guidance = UnfWhen { ug_arity = arity, ug_unsat_ok = unsat_ok, ug_boring_ok = boring_ok }
 
 tcUnfolding _toplvl name dfun_ty _ (IfDFunUnfold bs ops)
   = bindIfaceBndrs bs $ \ bs' ->
-    do { mb_ops1 <- forkM_maybe doc $ mapM tcIfaceExpr ops
-       ; return (case mb_ops1 of
-                    Nothing   -> noUnfolding
-                    Just ops1 -> mkDFunUnfolding bs' (classDataCon cls) ops1) }
+    do { ops1 <- forkM doc $ mapM tcIfaceExpr ops
+       ; return $ mkDFunUnfolding bs' (classDataCon cls) ops1 }
   where
     doc = text "Class ops for dfun" <+> ppr name
     (_, _, cls, _) = tcSplitDFunTy dfun_ty
 
-{-
-For unfoldings we try to do the job lazily, so that we never type check
+{- Note [Lazily checking Unfoldings]
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+For unfoldings, we try to do the job lazily, so that we never typecheck
 an unfolding that isn't going to be looked at.
+
+The main idea is that if M.hi has a declaration
+   f :: Int -> Int
+   f = \x. ...A.g...   -- The unfolding for f
+
+then we don't even want to /read/ A.hi unless f's unfolding is actually used; say,
+if f is inlined. But we need to be careful. Even if we don't inline f, we might ask
+hasNoBinding of it (Core Lint does this in GHC.Core.Lint.checkCanEtaExpand),
+and hasNoBinding looks to see if f has a compulsory unfolding.
+So the root Unfolding constructor must be visible: we want to be able to read the 'uf_src'
+field which says whether it is a compulsory unfolding, without forcing the unfolding RHS
+which is stored in 'uf_tmpl'. This matters for efficiency, but not only: if g's unfolding
+mentions f, we must not look at the unfolding RHS for f, as this is precisely what we are
+in the middle of checking (so looking at it would cause a loop).
+
+Conclusion: `tcUnfolding` must return an `Unfolding` whose `uf_src` field is readable without
+forcing the `uf_tmpl` field. In particular, all the functions used at the end of
+`tcUnfolding` (such as `mkFinalUnfolding`, `mkCompulsoryUnfolding'`, `mkCoreUnfolding`) must be
+lazy in `expr`.
+
+Ticket #21139
 -}
 
-tcPragExpr :: Bool  -- Is this unfolding compulsory?
-                    -- See Note [Checking for levity polymorphism] in GHC.Core.Lint
-           -> TopLevelFlag -> Name -> IfaceExpr -> IfL (Maybe CoreExpr)
-tcPragExpr is_compulsory toplvl name expr
-  = forkM_maybe doc $ do
+tcUnfoldingRhs :: Bool -- ^ Is this unfolding compulsory?
+                       -- See Note [Checking for representation polymorphism] in GHC.Core.Lint
+               -> TopLevelFlag -> Name -> IfaceExpr -> IfL CoreExpr
+tcUnfoldingRhs is_compulsory toplvl name expr
+  = forkM doc $ do
     core_expr' <- tcIfaceExpr expr
 
     -- Check for type consistency in the unfolding
@@ -1756,7 +1787,7 @@ tcPragExpr is_compulsory toplvl name expr
         case lintUnfolding is_compulsory dflags noSrcLoc in_scope core_expr' of
           Nothing   -> return ()
           Just errs -> liftIO $
-            displayLintResults logger dflags False doc
+            displayLintResults logger False doc
                                (pprCoreExpr core_expr') (emptyBag, errs)
     return core_expr'
   where
@@ -1766,14 +1797,11 @@ tcPragExpr is_compulsory toplvl name expr
     get_in_scope :: IfL VarSet -- Totally disgusting; but just for linting
     get_in_scope
         = do { (gbl_env, lcl_env) <- getEnvs
-             ; rec_ids <- case if_rec_types gbl_env of
-                            Nothing -> return []
-                            Just (_, get_env) -> do
-                               { type_env <- setLclEnv () get_env
-                               ; return (typeEnvIds type_env) }
+             ; let type_envs = knotVarElems (if_rec_types gbl_env)
+             ; top_level_vars <- concat <$> mapM (fmap typeEnvIds . setLclEnv ())  type_envs
              ; return (bindingsVars (if_tv_env lcl_env) `unionVarSet`
                        bindingsVars (if_id_env lcl_env) `unionVarSet`
-                       mkVarSet rec_ids) }
+                       mkVarSet top_level_vars) }
 
     bindingsVars :: FastStringEnv Var -> VarSet
     bindingsVars ufm = mkVarSet $ nonDetEltsUFM ufm
@@ -1803,10 +1831,10 @@ tcIfaceGlobal name
 
   | otherwise
   = do  { env <- getGblEnv
-        ; case if_rec_types env of {    -- Note [Tying the knot]
-            Just (mod, get_type_env)
-                | nameIsLocalOrFrom mod name
-                -> do           -- It's defined in the module being compiled
+        ; cur_mod <- if_mod <$> getLclEnv
+        ; case lookupKnotVars (if_rec_types env) (fromMaybe cur_mod (nameModule_maybe name))  of     -- Note [Tying the knot]
+            Just get_type_env
+                -> do           -- It's defined in a module in the hs-boot loop
                 { type_env <- setLclEnv () get_type_env         -- yuk
                 ; case lookupNameEnv type_env name of
                     Just thing -> return thing
@@ -1814,7 +1842,7 @@ tcIfaceGlobal name
                     Nothing   -> via_external
                 }
 
-          ; _ -> via_external }}
+            _ -> via_external }
   where
     via_external =  do
         { hsc_env <- getTopEnv
@@ -1843,6 +1871,7 @@ tcIfaceGlobal name
 --      * Note [DFun knot-tying]
 --      * Note [hsc_type_env_var hack]
 --      * Note [Knot-tying fallback on boot]
+--      * Note [Hydrating Modules]
 --
 -- There is also a wiki page on the subject, see:
 --
@@ -1899,13 +1928,15 @@ tcIfaceCoAxiom name = do { thing <- tcIfaceImplicit name
 
 
 tcIfaceCoAxiomRule :: IfLclName -> IfL CoAxiomRule
--- Unlike CoAxioms, which arise form user 'type instance' declarations,
--- there are a fixed set of CoAxiomRules,
--- currently enumerated in typeNatCoAxiomRules
+-- Unlike CoAxioms, which arise from user 'type instance' declarations,
+-- there are a fixed set of CoAxiomRules:
+--   - axioms for type-level literals (Nat and Symbol),
+--     enumerated in typeNatCoAxiomRules
 tcIfaceCoAxiomRule n
-  = case lookupUFM typeNatCoAxiomRules n of
-        Just ax -> return ax
-        _  -> pprPanic "tcIfaceCoAxiomRule" (ppr n)
+  | Just ax <- lookupUFM typeNatCoAxiomRules n
+  = return ax
+  | otherwise
+  = pprPanic "tcIfaceCoAxiomRule" (ppr n)
 
 tcIfaceDataCon :: Name -> IfL DataCon
 tcIfaceDataCon name = do { thing <- tcIfaceGlobal name
