@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP #-}
+
 {-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 
@@ -7,8 +7,6 @@ module GHC.Settings.IO
  , initSettings
  ) where
 
-#include "HsVersions.h"
-
 import GHC.Prelude
 
 import GHC.Settings.Utils
@@ -37,12 +35,6 @@ initSettings
   => String -- ^ TopDir path
   -> ExceptT SettingsError m Settings
 initSettings top_dir = do
-  -- see Note [topdir: How GHC finds its files]
-  -- NB: top_dir is assumed to be in standard Unix
-  -- format, '/' separated
-  mtool_dir <- liftIO $ findToolDir top_dir
-        -- see Note [tooldir: How GHC finds mingw on Windows]
-
   let installed :: FilePath -> FilePath
       installed file = top_dir </> file
       libexec :: FilePath -> FilePath
@@ -60,25 +52,33 @@ initSettings top_dir = do
     Nothing -> throwE $ SettingsError_BadData $
       "Can't parse " ++ show settingsFile
   let mySettings = Map.fromList settingsList
+      getBooleanSetting :: String -> ExceptT SettingsError m Bool
+      getBooleanSetting key = either pgmError pure $
+        getRawBooleanSetting settingsFile mySettings key
+
+  -- On Windows, by mingw is often distributed with GHC,
+  -- so we look in TopDir/../mingw/bin,
+  -- as well as TopDir/../../mingw/bin for hadrian.
+  -- But we might be disabled, in which we we don't do that.
+  useInplaceMinGW <- getBooleanSetting "Use inplace MinGW toolchain"
+
+  -- see Note [topdir: How GHC finds its files]
+  -- NB: top_dir is assumed to be in standard Unix
+  -- format, '/' separated
+  mtool_dir <- liftIO $ findToolDir useInplaceMinGW top_dir
+        -- see Note [tooldir: How GHC finds mingw on Windows]
+
   -- See Note [Settings file] for a little more about this file. We're
   -- just partially applying those functions and throwing 'Left's; they're
   -- written in a very portable style to keep ghc-boot light.
   let getSetting key = either pgmError pure $
         getRawFilePathSetting top_dir settingsFile mySettings key
       getToolSetting :: String -> ExceptT SettingsError m String
-      getToolSetting key = expandToolDir mtool_dir <$> getSetting key
-      getBooleanSetting :: String -> ExceptT SettingsError m Bool
-      getBooleanSetting key = either pgmError pure $
-        getRawBooleanSetting settingsFile mySettings key
+      getToolSetting key = expandToolDir useInplaceMinGW mtool_dir <$> getSetting key
   targetPlatformString <- getSetting "target platform string"
   myExtraGccViaCFlags <- getSetting "GCC extra via C opts"
-  -- On Windows, mingw is distributed with GHC,
-  -- so we look in TopDir/../mingw/bin,
-  -- as well as TopDir/../../mingw/bin for hadrian.
-  -- It would perhaps be nice to be able to override this
-  -- with the settings file, but it would be a little fiddly
-  -- to make that possible, so for now you can't.
   cc_prog <- getToolSetting "C compiler command"
+  cxx_prog <- getToolSetting "C++ compiler command"
   cc_args_str <- getSetting "C compiler flags"
   cxx_args_str <- getSetting "C++ compiler flags"
   gccSupportsNoPie <- getBooleanSetting "C compiler supports -no-pie"
@@ -97,6 +97,7 @@ initSettings top_dir = do
   ldSupportsBuildId       <- getBooleanSetting "ld supports build-id"
   ldSupportsFilelist      <- getBooleanSetting "ld supports filelist"
   ldIsGnuLd               <- getBooleanSetting "ld is GNU ld"
+  arSupportsDashL         <- getBooleanSetting "ar supports -L"
 
   let globalpkgdb_path = installed "package.conf.d"
       ghc_usage_msg_path  = installed "ghc-usage.txt"
@@ -113,10 +114,6 @@ initSettings top_dir = do
   install_name_tool_path <- getToolSetting "install_name_tool command"
   ranlib_path <- getToolSetting "ranlib command"
 
-  -- TODO this side-effect doesn't belong here. Reading and parsing the settings
-  -- should be idempotent and accumulate no resources.
-  tmpdir <- liftIO $ getTemporaryDirectory
-
   touch_path <- getToolSetting "touch command"
 
   mkdll_prog <- getToolSetting "dllwrap command"
@@ -135,6 +132,9 @@ initSettings top_dir = do
         ld_args  = map Option (cc_args ++ words cc_link_args_str)
   ld_r_prog <- getToolSetting "Merge objects command"
   ld_r_args <- getSetting "Merge objects flags"
+  let ld_r
+        | null ld_r_prog = Nothing
+        | otherwise      = Just (ld_r_prog, map Option $ words ld_r_args)
 
   llvmTarget <- getSetting "LLVM target"
 
@@ -146,10 +146,7 @@ initSettings top_dir = do
   let iserv_prog = libexec "ghc-iserv"
 
   ghcWithInterpreter <- getBooleanSetting "Use interpreter"
-  ghcWithSMP <- getBooleanSetting "Support SMP"
-  ghcRTSWays <- getSetting "RTS ways"
   useLibFFI <- getBooleanSetting "Use LibFFI"
-  ghcRtsWithLibdw <- getBooleanSetting "RTS expects libdw"
 
   return $ Settings
     { sGhcNameVersion = GhcNameVersion
@@ -158,8 +155,7 @@ initSettings top_dir = do
       }
 
     , sFileSettings = FileSettings
-      { fileSettings_tmpDir         = normalise tmpdir
-      , fileSettings_ghcUsagePath   = ghc_usage_msg_path
+      { fileSettings_ghcUsagePath   = ghc_usage_msg_path
       , fileSettings_ghciUsagePath  = ghci_usage_msg_path
       , fileSettings_toolDir        = mtool_dir
       , fileSettings_topDir         = top_dir
@@ -172,14 +168,17 @@ initSettings top_dir = do
       , toolSettings_ldSupportsFilelist      = ldSupportsFilelist
       , toolSettings_ldIsGnuLd               = ldIsGnuLd
       , toolSettings_ccSupportsNoPie         = gccSupportsNoPie
+      , toolSettings_useInplaceMinGW         = useInplaceMinGW
+      , toolSettings_arSupportsDashL         = arSupportsDashL
 
       , toolSettings_pgm_L   = unlit_path
       , toolSettings_pgm_P   = (cpp_prog, cpp_args)
       , toolSettings_pgm_F   = ""
       , toolSettings_pgm_c   = cc_prog
+      , toolSettings_pgm_cxx = cxx_prog
       , toolSettings_pgm_a   = (as_prog, as_args)
       , toolSettings_pgm_l   = (ld_prog, ld_args)
-      , toolSettings_pgm_lm  = (ld_r_prog, map Option $ words ld_r_args)
+      , toolSettings_pgm_lm  = ld_r
       , toolSettings_pgm_dll = (mkdll_prog,mkdll_args)
       , toolSettings_pgm_T   = touch_path
       , toolSettings_pgm_windres = windres_path
@@ -214,10 +213,7 @@ initSettings top_dir = do
     , sPlatformMisc = PlatformMisc
       { platformMisc_targetPlatformString = targetPlatformString
       , platformMisc_ghcWithInterpreter = ghcWithInterpreter
-      , platformMisc_ghcWithSMP = ghcWithSMP
-      , platformMisc_ghcRTSWays = ghcRTSWays
       , platformMisc_libFFI = useLibFFI
-      , platformMisc_ghcRtsWithLibdw = ghcRtsWithLibdw
       , platformMisc_llvmTarget = llvmTarget
       }
 
@@ -242,6 +238,7 @@ getTargetPlatform settingsFile settings = do
   targetHasGnuNonexecStack <- getBooleanSetting "target has GNU nonexec stack"
   targetHasIdentDirective <- getBooleanSetting "target has .ident directive"
   targetHasSubsectionsViaSymbols <- getBooleanSetting "target has subsections via symbols"
+  targetHasLibm <- getBooleanSetting "target has libm"
   crossCompiling <- getBooleanSetting "cross compiling"
   tablesNextToCode <- getBooleanSetting "Tables next to code"
 
@@ -256,5 +253,6 @@ getTargetPlatform settingsFile settings = do
     , platformIsCrossCompiling = crossCompiling
     , platformLeadingUnderscore = targetLeadingUnderscore
     , platformTablesNextToCode  = tablesNextToCode
+    , platformHasLibm = targetHasLibm
     , platform_constants = Nothing -- will be filled later when loading (or building) the RTS unit
     }
