@@ -1,7 +1,8 @@
-{-# LANGUAGE CPP                        #-}
+
 {-# LANGUAGE DeriveFunctor              #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE RecordWildCards            #-}
+{-# LANGUAGE FlexibleContexts           #-}
 
 {-# OPTIONS_GHC -fprof-auto-top #-}
 {-# OPTIONS_GHC -Wno-incomplete-uni-patterns #-}
@@ -13,8 +14,6 @@
 -- | GHC.StgToByteCode: Generate bytecode from STG
 module GHC.StgToByteCode ( UnlinkedBCO, byteCodeGen) where
 
-#include "HsVersions.h"
-
 import GHC.Prelude
 
 import GHC.Driver.Session
@@ -38,12 +37,12 @@ import GHCi.RemoteTypes
 import GHC.Types.Basic
 import GHC.Utils.Outputable
 import GHC.Types.Name
-import GHC.Types.Id.Make
 import GHC.Types.Id
 import GHC.Types.ForeignCall
 import GHC.Core
 import GHC.Types.Literal
 import GHC.Builtin.PrimOps
+import GHC.Builtin.PrimOps.Ids (primOpId)
 import GHC.Core.Type
 import GHC.Types.RepType
 import GHC.Core.DataCon
@@ -51,15 +50,15 @@ import GHC.Core.TyCon
 import GHC.Utils.Misc
 import GHC.Utils.Logger
 import GHC.Types.Var.Set
-import GHC.Builtin.Types ( unboxedUnitTy )
 import GHC.Builtin.Types.Prim
 import GHC.Core.TyCo.Ppr ( pprType )
 import GHC.Utils.Error
 import GHC.Types.Unique
 import GHC.Builtin.Uniques
-import GHC.Builtin.Utils ( primOpId )
 import GHC.Data.FastString
 import GHC.Utils.Panic
+import GHC.Utils.Panic.Plain
+import GHC.Utils.Exception (evaluate)
 import GHC.StgToCmm.Closure ( NonVoid(..), fromNonVoid, nonVoidIds )
 import GHC.StgToCmm.Layout
 import GHC.Runtime.Heap.Layout hiding (WordOff, ByteOff, wordsToBytes)
@@ -75,10 +74,8 @@ import Foreign hiding (shiftL, shiftR)
 import Control.Monad
 import Data.Char
 
-import GHC.Types.Unique.Supply
 import GHC.Unit.Module
 
-import Control.Exception
 import Data.Array
 import Data.Coerce (coerce)
 import Data.ByteString (ByteString)
@@ -91,9 +88,7 @@ import Data.Ord
 import GHC.Stack.CCS
 import Data.Either ( partitionEithers )
 
-import qualified GHC.Types.CostCentre as CC
 import GHC.Stg.Syntax
-import GHC.Stg.FVs
 import qualified Data.IntSet as IntSet
 
 -- -----------------------------------------------------------------------------
@@ -101,12 +96,12 @@ import qualified Data.IntSet as IntSet
 
 byteCodeGen :: HscEnv
             -> Module
-            -> [StgTopBinding]
+            -> [CgStgTopBinding]
             -> [TyCon]
             -> Maybe ModBreaks
             -> IO CompiledByteCode
 byteCodeGen hsc_env this_mod binds tycs mb_modBreaks
-   = withTiming logger dflags
+   = withTiming logger
                 (text "GHC.StgToByteCode"<+>brackets (ppr this_mod))
                 (const ()) $ do
         -- Split top-level binds into strings and others.
@@ -120,18 +115,15 @@ byteCodeGen hsc_env this_mod binds tycs mb_modBreaks
             flattenBind (StgRec bs)     = bs
         stringPtrs <- allocateTopStrings interp strings
 
-        us <- mkSplitUniqSupply 'y'
         (BcM_State{..}, proto_bcos) <-
-           runBc hsc_env us this_mod mb_modBreaks (mkVarEnv stringPtrs) $ do
-             prepd_binds <- mapM bcPrepBind lifted_binds
-             let flattened_binds =
-                   concatMap (flattenBind . annBindingFreeVars) (reverse prepd_binds)
+           runBc hsc_env this_mod mb_modBreaks (mkVarEnv stringPtrs) $ do
+             let flattened_binds = concatMap flattenBind (reverse lifted_binds)
              mapM schemeTopBind flattened_binds
 
         when (notNull ffis)
              (panic "GHC.StgToByteCode.byteCodeGen: missing final emitBc?")
 
-        dumpIfSet_dyn logger dflags Opt_D_dump_BCOs
+        putDumpFileMaybe logger Opt_D_dump_BCOs
            "Proto-BCOs" FormatByteCode
            (vcat (intersperse (char ' ') (map ppr proto_bcos)))
 
@@ -150,8 +142,8 @@ byteCodeGen hsc_env this_mod binds tycs mb_modBreaks
 
         return cbc
 
-  where dflags = hsc_dflags hsc_env
-        logger = hsc_logger hsc_env
+  where dflags  = hsc_dflags hsc_env
+        logger  = hsc_logger hsc_env
         interp  = hscInterp hsc_env
         profile = targetProfile dflags
 
@@ -166,7 +158,7 @@ allocateTopStrings interp topStrings = do
 
 {-
 Note [generating code for top-level string literal bindings]
-
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Here is a summary on how the byte code generator deals with top-level string
 literals:
 
@@ -178,100 +170,6 @@ literals:
    BcM and used when generating code for variable references.
 -}
 
-{-
-  Prepare the STG for bytecode generation:
-
-   - Ensure that all breakpoints are directly under
-        a let-binding, introducing a new binding for
-        those that aren't already.
-
-   - Protect Not-necessarily lifted join points, see
-        Note [Not-necessarily-lifted join points]
-
- -}
-
-bcPrepRHS :: StgRhs -> BcM StgRhs
--- explicitly match all constructors so we get a warning if we miss any
-bcPrepRHS (StgRhsClosure fvs cc upd args (StgTick bp@Breakpoint{} expr)) = do
-  {- If we have a breakpoint directly under an StgRhsClosure we don't
-     need to introduce a new binding for it.
-   -}
-  expr' <- bcPrepExpr expr
-  pure (StgRhsClosure fvs cc upd args (StgTick bp expr'))
-bcPrepRHS (StgRhsClosure fvs cc upd args expr) =
-  StgRhsClosure fvs cc upd args <$> bcPrepExpr expr
-bcPrepRHS con@StgRhsCon{} = pure con
-
-bcPrepExpr :: StgExpr -> BcM StgExpr
--- explicitly match all constructors so we get a warning if we miss any
-bcPrepExpr (StgTick bp@(Breakpoint tick_ty _ _) rhs)
-  | isLiftedTypeKind (typeKind tick_ty) = do
-      id <- newId tick_ty
-      rhs' <- bcPrepExpr rhs
-      let expr' = StgTick bp rhs'
-          bnd = StgNonRec id (StgRhsClosure noExtFieldSilent
-                                            CC.dontCareCCS
-                                            ReEntrant
-                                            []
-                                            expr'
-                             )
-          letExp = StgLet noExtFieldSilent bnd (StgApp id [])
-      pure letExp
-  | otherwise = do
-      id <- newId (mkVisFunTyMany realWorldStatePrimTy tick_ty)
-      st <- newId realWorldStatePrimTy
-      rhs' <- bcPrepExpr rhs
-      let expr' = StgTick bp rhs'
-          bnd = StgNonRec id (StgRhsClosure noExtFieldSilent
-                                            CC.dontCareCCS
-                                            ReEntrant
-                                            [voidArgId]
-                                            expr'
-                             )
-      pure $ StgLet noExtFieldSilent bnd (StgApp id [StgVarArg st])
-bcPrepExpr (StgTick tick rhs) =
-  StgTick tick <$> bcPrepExpr rhs
-bcPrepExpr (StgLet xlet bnds expr) =
-  StgLet xlet <$> bcPrepBind bnds
-              <*> bcPrepExpr expr
-bcPrepExpr (StgLetNoEscape xlne bnds expr) =
-  StgLet xlne <$> bcPrepBind bnds
-              <*> bcPrepExpr expr
-bcPrepExpr (StgCase expr bndr alt_type alts) =
-  StgCase <$> bcPrepExpr expr
-          <*> pure bndr
-          <*> pure alt_type
-          <*> mapM bcPrepAlt alts
-bcPrepExpr lit@StgLit{} = pure lit
--- See Note [Not-necessarily-lifted join points], step 3.
-bcPrepExpr (StgApp x [])
-  | isNNLJoinPoint x = pure $
-      StgApp (protectNNLJoinPointId x) [StgVarArg voidPrimId]
-bcPrepExpr app@StgApp{} = pure app
-bcPrepExpr app@StgConApp{} = pure app
-bcPrepExpr app@StgOpApp{} = pure app
-
-bcPrepAlt :: StgAlt -> BcM StgAlt
-bcPrepAlt (ac, bndrs, expr) = (,,) ac bndrs <$> bcPrepExpr expr
-
-bcPrepBind :: StgBinding -> BcM StgBinding
--- explicitly match all constructors so we get a warning if we miss any
-bcPrepBind (StgNonRec bndr rhs) =
-  let (bndr', rhs') = bcPrepSingleBind (bndr, rhs)
-  in  StgNonRec bndr' <$> bcPrepRHS rhs'
-bcPrepBind (StgRec bnds) =
-  StgRec <$> mapM ((\(b,r) -> (,) b <$> bcPrepRHS r) . bcPrepSingleBind)
-                  bnds
-
-bcPrepSingleBind :: (Id, StgRhs) -> (Id, StgRhs)
--- If necessary, modify this Id and body to protect not-necessarily-lifted join points.
--- See Note [Not-necessarily-lifted join points], step 2.
-bcPrepSingleBind (x, StgRhsClosure ext cc upd_flag args body)
-  | isNNLJoinPoint x
-  = ( protectNNLJoinPointId x
-    , StgRhsClosure ext cc upd_flag (args ++ [voidArgId]) body)
-bcPrepSingleBind bnd = bnd
-
 -- -----------------------------------------------------------------------------
 -- Compilation schema for the bytecode generator
 
@@ -318,11 +216,11 @@ mkProtoBCO
    -> name
    -> BCInstrList
    -> Either  [CgStgAlt] (CgStgRhs)
-        -- ^ original expression; for debugging only
-   -> Int
-   -> Word16
-   -> [StgWord]
-   -> Bool      -- True <=> is a return point, rather than a function
+                -- ^ original expression; for debugging only
+   -> Int       -- ^ arity
+   -> Word16    -- ^ bitmap size
+   -> [StgWord] -- ^ bitmap
+   -> Bool      -- ^ True <=> is a return point, rather than a function
    -> [FFIInfo]
    -> ProtoBCO name
 mkProtoBCO platform nm instrs_ordlist origin arity bitmap_size bitmap is_ret ffis
@@ -399,10 +297,7 @@ schemeTopBind (id, rhs)
         -- by just re-using the single top-level definition.  So
         -- for the worker itself, we must allocate it directly.
     -- ioToBc (putStrLn $ "top level BCO")
-    let enter = if isUnliftedTypeKind (tyConResKind (dataConTyCon data_con))
-                then RETURN_UNLIFTED P
-                else ENTER
-    emitBc (mkProtoBCO platform (getName id) (toOL [PACK data_con 0, enter])
+    emitBc (mkProtoBCO platform (getName id) (toOL [PACK data_con 0, RETURN])
                        (Right rhs) 0 0 [{-no bitmap-}] False{-not alts-})
 
   | otherwise
@@ -593,7 +488,7 @@ returnUnboxedTuple d s p es = do
         (tuple_info, tuple_components) = layoutTuple profile d arg_ty es
         go _   pushes [] = return (reverse pushes)
         go !dd pushes ((a, off):cs) = do (push, szb) <- pushAtom dd p a
-                                         MASSERT(off == dd + szb)
+                                         massert (off == dd + szb)
                                          go (dd + szb) (push:pushes) cs
     pushes <- go d [] tuple_components
     ret <- returnUnliftedReps d
@@ -608,7 +503,7 @@ schemeE
     :: StackDepth -> Sequel -> BCEnv -> CgStgExpr -> BcM BCInstrList
 schemeE d s p (StgLit lit) = returnUnliftedAtom d s p (StgLitArg lit)
 schemeE d s p (StgApp x [])
-   | isUnliftedType (idType x) = returnUnliftedAtom d s p (StgVarArg x)
+   | not (usePlainReturn (idType x)) = returnUnliftedAtom d s p (StgVarArg x)
 -- Delegate tail-calls to schemeT.
 schemeE d s p e@(StgApp {}) = schemeT d s p e
 schemeE d s p e@(StgConApp {}) = schemeT d s p e
@@ -709,19 +604,6 @@ schemeE d s p (StgCase scrut _ _ []) = schemeE d s p scrut
 schemeE d s p (StgCase scrut bndr _ alts)
    = doCase d s p scrut bndr alts
 
--- Is this Id a not-necessarily-lifted join point?
--- See Note [Not-necessarily-lifted join points], step 1
-isNNLJoinPoint :: Id -> Bool
-isNNLJoinPoint x = isJoinId x &&
-                   Just True /= isLiftedType_maybe (idType x)
-
--- Update an Id's type to take a Void# argument.
--- Precondition: the Id is a not-necessarily-lifted join point.
--- See Note [Not-necessarily-lifted join points]
-protectNNLJoinPointId :: Id -> Id
-protectNNLJoinPointId x
-  = ASSERT( isNNLJoinPoint x )
-    updateIdTypeButNotMult (unboxedUnitTy `mkVisFunTyMany`) x
 
 {-
    Ticked Expressions
@@ -730,65 +612,6 @@ protectNNLJoinPointId x
   The idea is that the "breakpoint<n,fvs> E" is really just an annotation on
   the code. When we find such a thing, we pull out the useful information,
   and then compile the code as if it was just the expression E.
-
-Note [Not-necessarily-lifted join points]
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-A join point variable is essentially a goto-label: it is, for example,
-never used as an argument to another function, and it is called only
-in tail position. See Note [Join points] and Note [Invariants on join points],
-both in GHC.Core. Because join points do not compile to true, red-blooded
-variables (with, e.g., registers allocated to them), they are allowed
-to be levity-polymorphic. (See invariant #6 in Note [Invariants on join points]
-in GHC.Core.)
-
-However, in this byte-code generator, join points *are* treated just as
-ordinary variables. There is no check whether a binding is for a join point
-or not; they are all treated uniformly. (Perhaps there is a missed optimization
-opportunity here, but that is beyond the scope of my (Richard E's) Thursday.)
-
-We thus must have *some* strategy for dealing with levity-polymorphic and
-unlifted join points. Levity-polymorphic variables are generally not allowed
-(though levity-polymorphic join points *are*; see Note [Invariants on join points]
-in GHC.Core, point 6), and we don't wish to evaluate unlifted join points eagerly.
-The questionable join points are *not-necessarily-lifted join points*
-(NNLJPs). (Not having such a strategy led to #16509, which panicked in the
-isUnliftedType check in the AnnVar case of schemeE.) Here is the strategy:
-
-1. Detect NNLJPs. This is done in isNNLJoinPoint.
-
-2. When binding an NNLJP, add a `\ (_ :: (# #)) ->` to its RHS, and modify the
-   type to tack on a `(# #) ->`.
-   Note that functions are never levity-polymorphic, so this transformation
-   changes an NNLJP to a non-levity-polymorphic join point. This is done
-   in bcPrepSingleBind.
-
-3. At an occurrence of an NNLJP, add an application to void# (called voidPrimId),
-   being careful to note the new type of the NNLJP. This is done in the AnnVar
-   case of schemeE, with help from protectNNLJoinPointId.
-
-Here is an example. Suppose we have
-
-  f = \(r :: RuntimeRep) (a :: TYPE r) (x :: T).
-      join j :: a
-           j = error @r @a "bloop"
-      in case x of
-           A -> j
-           B -> j
-           C -> error @r @a "blurp"
-
-Our plan is to behave is if the code was
-
-  f = \(r :: RuntimeRep) (a :: TYPE r) (x :: T).
-      let j :: (Void# -> a)
-          j = \ _ -> error @r @a "bloop"
-      in case x of
-           A -> j void#
-           B -> j void#
-           C -> error @r @a "blurp"
-
-It's a bit hacky, but it works well in practice and is local. I suspect the
-Right Fix is to take advantage of join points as goto-labels.
-
 -}
 
 -- Compile code to do a tail call.  Specifically, push the fn,
@@ -836,7 +659,7 @@ schemeT _d _s _p (StgOpApp StgPrimCallOp{} _args _ty)
    = unsupportedCConvException
 
    -- Case 2: Unboxed tuple
-schemeT d s p (StgConApp con _ext args _tys)
+schemeT d s p (StgConApp con _cn args _tys)
    | isUnboxedTupleDataCon con || isUnboxedSumDataCon con
    = returnUnboxedTuple d s p args
 
@@ -845,10 +668,7 @@ schemeT d s p (StgConApp con _ext args _tys)
    = do alloc_con <- mkConAppCode d s p con args
         platform <- profilePlatform <$> getProfile
         return (alloc_con         `appOL`
-                mkSlideW 1 (bytesToWords platform $ d - s) `snocOL`
-                if isUnliftedTypeKind (tyConResKind (dataConTyCon con))
-                then RETURN_UNLIFTED P
-                else ENTER)
+                mkSlideW 1 (bytesToWords platform $ d - s) `snocOL` RETURN)
 
    -- Case 4: Tail call of function
 schemeT d s p (StgApp fn args)
@@ -911,15 +731,12 @@ doTailCall init_d s p fn args = do
    do_pushes init_d args (map (atomRep platform) args)
   where
   do_pushes !d [] reps = do
-        ASSERT( null reps ) return ()
+        assert (null reps) return ()
         (push_fn, sz) <- pushAtom d p (StgVarArg fn)
         platform <- profilePlatform <$> getProfile
-        ASSERT( sz == wordSize platform ) return ()
+        assert (sz == wordSize platform) return ()
         let slide = mkSlideB platform (d - init_d + wordSize platform) (init_d - s)
-            enter = if isUnliftedType (idType fn)
-                    then RETURN_UNLIFTED P
-                    else ENTER
-        return (push_fn `appOL` (slide `appOL` unitOL enter))
+        return (push_fn `appOL` (slide `appOL` unitOL ENTER))
   do_pushes !d args reps = do
       let (push_apply, n, rest_of_reps) = findPushSeq reps
           (these_args, rest_of_args) = splitAt n args
@@ -959,6 +776,9 @@ findPushSeq (D: rest)
   = (PUSH_APPLY_D, 1, rest)
 findPushSeq (L: rest)
   = (PUSH_APPLY_L, 1, rest)
+findPushSeq argReps
+  | any (`elem` [V16, V32, V64]) argReps
+  = sorry "SIMD vector operations are not available in GHCi"
 findPushSeq _
   = panic "GHC.StgToByteCode.findPushSeq"
 
@@ -992,6 +812,8 @@ doCase d s p scrut bndr alts
           (isUnboxedTupleType bndr_ty || isUnboxedSumType bndr_ty) &&
           length non_void_arg_reps > 1
 
+        ubx_frame = not ubx_tuple_frame && not (usePlainReturn bndr_ty)
+
         non_void_arg_reps = non_void (typeArgReps platform bndr_ty)
 
         profiling
@@ -1013,12 +835,11 @@ doCase d s p scrut bndr alts
                           not ubx_tuple_frame = 2 * wordSize platform
                         | otherwise = 0
 
-        -- An unlifted value gets an extra info table pushed on top
-        -- when it is returned.
+        -- The size of the return frame info table pointer if one exists
         unlifted_itbl_size_b :: StackDepth
-        unlifted_itbl_size_b | ubx_tuple_frame              = 3 * wordSize platform
-                             | not (isUnliftedType bndr_ty) = 0
-                             | otherwise                    = wordSize platform
+        unlifted_itbl_size_b | ubx_tuple_frame = wordSize platform
+                             | ubx_frame       = wordSize platform
+                             | otherwise       = 0
 
         (bndr_size, tuple_info, args_offsets)
            | ubx_tuple_frame =
@@ -1052,11 +873,12 @@ doCase d s p scrut bndr alts
         isAlgCase = isAlgType bndr_ty
 
         -- given an alt, return a discr and code for it.
-        codeAlt (DEFAULT, _, rhs)
+        codeAlt :: CgStgAlt -> BcM (Discr, BCInstrList)
+        codeAlt GenStgAlt{alt_con=DEFAULT,alt_bndrs=_,alt_rhs=rhs}
            = do rhs_code <- schemeE d_alts s p_alts rhs
                 return (NoDiscr, rhs_code)
 
-        codeAlt alt@(_, bndrs, rhs)
+        codeAlt alt@GenStgAlt{alt_con=_, alt_bndrs=bndrs, alt_rhs=rhs}
            -- primitive or nullary constructor alt: no need to UNPACK
            | null real_bndrs = do
                 rhs_code <- schemeE d_alts s p_alts rhs
@@ -1099,37 +921,33 @@ doCase d s p scrut bndr alts
                         | (NonVoid arg, offset) <- args_offsets ]
                         p_alts
 
-                 -- unlifted datatypes have an infotable word on top
-                 unpack = if isUnliftedType bndr_ty
-                          then PUSH_L 1 `consOL`
-                               UNPACK (trunc16W size) `consOL`
-                               unitOL (SLIDE (trunc16W size) 1)
-                          else unitOL (UNPACK (trunc16W size))
              in do
-             MASSERT(isAlgCase)
+             massert isAlgCase
              rhs_code <- schemeE stack_bot s p' rhs
-             return (my_discr alt, unpack `appOL` rhs_code)
+             return (my_discr alt,
+                     unitOL (UNPACK (trunc16W size)) `appOL` rhs_code)
            where
              real_bndrs = filterOut isTyVar bndrs
 
-        my_discr (DEFAULT, _, _) = NoDiscr {-shouldn't really happen-}
-        my_discr (DataAlt dc, _, _)
-           | isUnboxedTupleDataCon dc || isUnboxedSumDataCon dc
-           = NoDiscr
-           | otherwise
-           = DiscrP (fromIntegral (dataConTag dc - fIRST_TAG))
-        my_discr (LitAlt l, _, _)
-           = case l of LitNumber LitNumInt i  -> DiscrI (fromInteger i)
-                       LitNumber LitNumWord w -> DiscrW (fromInteger w)
-                       LitFloat r   -> DiscrF (fromRational r)
-                       LitDouble r  -> DiscrD (fromRational r)
-                       LitChar i    -> DiscrI (ord i)
-                       _ -> pprPanic "schemeE(StgCase).my_discr" (ppr l)
+        my_discr alt = case alt_con alt of
+            DEFAULT    -> NoDiscr {-shouldn't really happen-}
+            DataAlt dc
+              | isUnboxedTupleDataCon dc || isUnboxedSumDataCon dc
+              -> NoDiscr
+              | otherwise
+              -> DiscrP (fromIntegral (dataConTag dc - fIRST_TAG))
+            LitAlt l -> case l of
+              LitNumber LitNumInt i  -> DiscrI (fromInteger i)
+              LitNumber LitNumWord w -> DiscrW (fromInteger w)
+              LitFloat r             -> DiscrF (fromRational r)
+              LitDouble r            -> DiscrD (fromRational r)
+              LitChar i              -> DiscrI (ord i)
+              _ -> pprPanic "schemeE(StgCase).my_discr" (ppr l)
 
         maybe_ncons
            | not isAlgCase = Nothing
            | otherwise
-           = case [dc | (DataAlt dc, _, _) <- alts] of
+           = case [dc | DataAlt dc <- alt_con <$> alts] of
                 []     -> Nothing
                 (dc:_) -> Just (tyConFamilySize (dataConTyCon dc))
 
@@ -1178,7 +996,11 @@ doCase d s p scrut bndr alts
         bitmap = intsToReverseBitmap platform bitmap_size'{-size-} pointers
 
      alt_stuff <- mapM codeAlt alts
-     alt_final <- mkMultiBranch maybe_ncons alt_stuff
+     alt_final0 <- mkMultiBranch maybe_ncons alt_stuff
+
+     let alt_final
+           | ubx_tuple_frame    = mkSlideW 0 2 `mappend` alt_final0
+           | otherwise          = alt_final0
 
      let
          alt_bco_name = getName bndr
@@ -1197,7 +1019,7 @@ doCase d s p scrut bndr alts
               return (PUSH_ALTS_TUPLE alt_bco' tuple_info tuple_bco
                       `consOL` scrut_code)
        else let push_alts
-                  | not (isUnliftedType bndr_ty)
+                  | not ubx_frame
                   = PUSH_ALTS alt_bco'
                   | otherwise
                   = let unlifted_rep =
@@ -1275,9 +1097,21 @@ layoutTuple profile start_off arg_ty reps =
                   (orig_stk_params ++ map get_byte_off new_stk_params)
      )
 
+{-
+  We use the plain return convention (ENTER/PUSH_ALTS) for
+  lifted types and unlifted algebraic types.
+
+  Other types use PUSH_ALTS_UNLIFTED/PUSH_ALTS_TUPLE which expect
+  additional data on the stack.
+ -}
+usePlainReturn :: Type -> Bool
+usePlainReturn t
+  | isUnboxedTupleType t || isUnboxedSumType t = False
+  | otherwise = typePrimRep t == [LiftedRep] ||
+                (typePrimRep t == [UnliftedRep] && isAlgType t)
+
 {- Note [unboxed tuple bytecodes and tuple_BCO]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
   We have the bytecode instructions RETURN_TUPLE and PUSH_ALTS_TUPLE to
   return and receive arbitrary unboxed tuples, respectively. These
   instructions use the helper data tuple_BCO and tuple_info.
@@ -1617,7 +1451,9 @@ primRepToFFIType platform r
      AddrRep     -> FFIPointer
      FloatRep    -> FFIFloat
      DoubleRep   -> FFIDouble
-     _           -> panic "primRepToFFIType"
+     LiftedRep   -> FFIPointer
+     UnliftedRep -> FFIPointer
+     _           -> pprPanic "primRepToFFIType" (ppr r)
   where
     (signed_word, unsigned_word) = case platformWordSize platform of
        PW4 -> (FFISInt32, FFIUInt32)
@@ -1628,19 +1464,21 @@ primRepToFFIType platform r
 mkDummyLiteral :: Platform -> PrimRep -> Literal
 mkDummyLiteral platform pr
    = case pr of
-        IntRep    -> mkLitInt  platform 0
-        WordRep   -> mkLitWord platform 0
-        Int8Rep   -> mkLitInt8 0
-        Word8Rep  -> mkLitWord8 0
-        Int16Rep  -> mkLitInt16 0
-        Word16Rep -> mkLitWord16 0
-        Int32Rep  -> mkLitInt32 0
-        Word32Rep -> mkLitWord32 0
-        Int64Rep  -> mkLitInt64 0
-        Word64Rep -> mkLitWord64 0
-        AddrRep   -> LitNullAddr
-        DoubleRep -> LitDouble 0
-        FloatRep  -> LitFloat 0
+        IntRep      -> mkLitInt  platform 0
+        WordRep     -> mkLitWord platform 0
+        Int8Rep     -> mkLitInt8 0
+        Word8Rep    -> mkLitWord8 0
+        Int16Rep    -> mkLitInt16 0
+        Word16Rep   -> mkLitWord16 0
+        Int32Rep    -> mkLitInt32 0
+        Word32Rep   -> mkLitWord32 0
+        Int64Rep    -> mkLitInt64 0
+        Word64Rep   -> mkLitWord64 0
+        AddrRep     -> LitNullAddr
+        DoubleRep   -> LitDouble 0
+        FloatRep    -> LitFloat 0
+        LiftedRep   -> LitNullAddr
+        UnliftedRep -> LitNullAddr
         _         -> pprPanic "mkDummyLiteral" (ppr pr)
 
 
@@ -1671,9 +1509,7 @@ maybe_getCCallReturnRep fn_ty
        case r_reps of
          []            -> panic "empty typePrimRepArgs"
          [VoidRep]     -> Nothing
-         [rep]
-           | isGcPtrRep rep -> blargh
-           | otherwise      -> Just rep
+         [rep]         -> Just rep
 
                  -- if it was, it would be impossible to create a
                  -- valid return value placeholder on the stack
@@ -1742,7 +1578,7 @@ implement_tagToId
     -> BcM BCInstrList
 -- See Note [Implementing tagToEnum#]
 implement_tagToId d s p arg names
-  = ASSERT( notNull names )
+  = assert (notNull names) $
     do (push_arg, arg_bytes) <- pushAtom d p (StgVarArg arg)
        labels <- getLabelsBc (genericLength names)
        label_fail <- getLabelBc
@@ -1833,10 +1669,21 @@ pushAtom d p (StgVarArg var)
         case lookupVarEnv topStrings var of
             Just ptr -> pushAtom d p $ StgLitArg $ mkLitWord platform $
               fromIntegral $ ptrToWordPtr $ fromRemotePtr ptr
-            Nothing -> do
-                let sz = idSizeCon platform var
-                MASSERT( sz == wordSize platform )
-                return (unitOL (PUSH_G (getName var)), sz)
+            Nothing
+              -- PUSH_G doesn't tag constructors. So we use PACK here
+              -- if we are dealing with nullary constructor.
+              | Just con <- isDataConWorkId_maybe var
+              -> do
+                  massert (sz == wordSize platform)
+                  massert (isNullaryRepDataCon con)
+                  return (unitOL (PACK con 0), sz)
+              | otherwise
+              -> do
+                  let
+                  massert (sz == wordSize platform)
+                  return (unitOL (PUSH_G (getName var)), sz)
+              where
+                !sz = idSizeCon platform var
 
 
 pushAtom _ _ (StgLitArg lit) = pushLiteral True lit
@@ -1889,11 +1736,9 @@ pushLiteral padded lit =
           LitNumWord32  -> code Word32Rep
           LitNumInt64   -> code Int64Rep
           LitNumWord64  -> code Word64Rep
-          -- No LitInteger's or LitNatural's should be left by the time this is
-          -- called. CorePrep should have converted them all to a real core
-          -- representation.
-          LitNumInteger -> panic "pushAtom: LitInteger"
-          LitNumNatural -> panic "pushAtom: LitNatural"
+          -- No LitNumBigNat should be left by the time this is called. CorePrep
+          -- should have converted them all to a real core representation.
+          LitNumBigNat  -> panic "pushAtom: LitNumBigNat"
 
 -- | Push an atom for constructor (i.e., PACK instruction) onto the stack.
 -- This is slightly different to @pushAtom@ due to the fact that we allow
@@ -1957,7 +1802,9 @@ mkMultiBranch maybe_ncons raw_ways = do
             | otherwise
             = return (testEQ (fst val) lbl_default `consOL` snd val)
 
-            -- Note [CASEFAIL] It may be that this case has no default
+            -- Note [CASEFAIL]
+            -- ~~~~~~~~~~~~~~~
+            -- It may be that this case has no default
             -- branch, but the alternatives are not exhaustive - this
             -- happens for GADT cases for example, where the types
             -- prove that certain branches are impossible.  We could
@@ -2160,7 +2007,6 @@ typeArgReps platform = map (toArgRep platform) . typePrimRepArgs
 data BcM_State
    = BcM_State
         { bcm_hsc_env :: HscEnv
-        , uniqSupply  :: UniqSupply      -- for generating fresh variable names
         , thisModule  :: Module          -- current module (for breakpoints)
         , nextlabel   :: Word32          -- for generating local labels
         , ffis        :: [FFIInfo]       -- ffi info blocks, to free later
@@ -2178,12 +2024,12 @@ ioToBc io = BcM $ \st -> do
   x <- io
   return (st, x)
 
-runBc :: HscEnv -> UniqSupply -> Module -> Maybe ModBreaks
+runBc :: HscEnv -> Module -> Maybe ModBreaks
       -> IdEnv (RemotePtr ())
       -> BcM r
       -> IO (BcM_State, r)
-runBc hsc_env us this_mod modBreaks topStrings (BcM m)
-   = m (BcM_State hsc_env us this_mod 0 [] modBreaks IntMap.empty topStrings)
+runBc hsc_env this_mod modBreaks topStrings (BcM m)
+   = m (BcM_State hsc_env this_mod 0 [] modBreaks IntMap.empty topStrings)
 
 thenBc :: BcM a -> (a -> BcM b) -> BcM b
 thenBc (BcM expr) cont = BcM $ \st0 -> do
@@ -2249,22 +2095,11 @@ newBreakInfo :: BreakIndex -> CgBreakInfo -> BcM ()
 newBreakInfo ix info = BcM $ \st ->
   return (st{breakInfo = IntMap.insert ix info (breakInfo st)}, ())
 
-newUnique :: BcM Unique
-newUnique = BcM $
-   \st -> case takeUniqFromSupply (uniqSupply st) of
-             (uniq, us) -> let newState = st { uniqSupply = us }
-                           in  return (newState, uniq)
-
 getCurrentModule :: BcM Module
 getCurrentModule = BcM $ \st -> return (st, thisModule st)
 
 getTopStrings :: BcM (IdEnv (RemotePtr ()))
 getTopStrings = BcM $ \st -> return (st, topStrings st)
 
-newId :: Type -> BcM Id
-newId ty = do
-    uniq <- newUnique
-    return $ mkSysLocal tickFS uniq Many ty
-
 tickFS :: FastString
 tickFS = fsLit "ticked"
