@@ -4,7 +4,7 @@
 
 -}
 
-{-# LANGUAGE CPP, DeriveDataTypeable, ScopedTypeVariables #-}
+{-# LANGUAGE DeriveDataTypeable, ScopedTypeVariables #-}
 {-# LANGUAGE TypeApplications #-}
 {-# LANGUAGE MagicHash #-}
 {-# LANGUAGE AllowAmbiguousTypes #-}
@@ -31,13 +31,14 @@ module GHC.Types.Literal
         , mkLitWord64, mkLitWord64Wrap, mkLitWord64Unchecked
         , mkLitFloat, mkLitDouble
         , mkLitChar, mkLitString
-        , mkLitInteger, mkLitNatural
+        , mkLitBigNat
         , mkLitNumber, mkLitNumberWrap
 
         -- ** Operations on Literals
         , literalType
         , pprLiteral
         , litNumIsSigned
+        , litNumRange
         , litNumCheckRange
         , litNumWrap
         , litNumCoerce
@@ -63,12 +64,9 @@ module GHC.Types.Literal
         , nullAddrLit, floatToDoubleLit, doubleToFloatLit
         ) where
 
-#include "HsVersions.h"
-
 import GHC.Prelude
 
 import GHC.Builtin.Types.Prim
-import {-# SOURCE #-} GHC.Builtin.Types
 import GHC.Core.Type
 import GHC.Utils.Outputable
 import GHC.Data.FastString
@@ -76,8 +74,8 @@ import GHC.Types.Basic
 import GHC.Utils.Binary
 import GHC.Settings.Constants
 import GHC.Platform
-import GHC.Utils.Misc
 import GHC.Utils.Panic
+import GHC.Utils.Encoding
 
 import Data.ByteString (ByteString)
 import Data.Int
@@ -164,8 +162,7 @@ data Literal
 
 -- | Numeric literal type
 data LitNumType
-  = LitNumInteger -- ^ @Integer@ (see Note [BigNum literals])
-  | LitNumNatural -- ^ @Natural@ (see Note [BigNum literals])
+  = LitNumBigNat  -- ^ @Bignat@ (see Note [BigNum literals])
   | LitNumInt     -- ^ @Int#@ - according to target machine
   | LitNumInt8    -- ^ @Int8#@ - exactly 8 bits
   | LitNumInt16   -- ^ @Int16#@ - exactly 16 bits
@@ -181,8 +178,7 @@ data LitNumType
 -- | Indicate if a numeric literal type supports negative numbers
 litNumIsSigned :: LitNumType -> Bool
 litNumIsSigned nt = case nt of
-  LitNumInteger -> True
-  LitNumNatural -> False
+  LitNumBigNat  -> False
   LitNumInt     -> True
   LitNumInt8    -> True
   LitNumInt16   -> True
@@ -197,8 +193,7 @@ litNumIsSigned nt = case nt of
 -- | Number of bits
 litNumBitSize :: Platform -> LitNumType -> Maybe Word
 litNumBitSize platform nt = case nt of
-  LitNumInteger -> Nothing
-  LitNumNatural -> Nothing
+  LitNumBigNat  -> Nothing
   LitNumInt     -> Just (fromIntegral (platformWordSizeInBits platform))
   LitNumInt8    -> Just 8
   LitNumInt16   -> Just 16
@@ -219,16 +214,28 @@ instance Binary LitNumType where
 {-
 Note [BigNum literals]
 ~~~~~~~~~~~~~~~~~~~~~~
-GHC supports 2 kinds of arbitrary precision integers (a.k.a BigNum):
+GHC supports 2 kinds of arbitrary precision numbers (a.k.a BigNum):
 
-   * Natural: natural represented as a Word# or as a BigNat
+   * data Natural = NS Word# | NB BigNat#
 
-   * Integer: integer represented a an Int# or as a BigNat (Integer's
-   constructors indicate the sign)
+   * data Integer = IS Int# | IN BigNat# | IP BigNat#
 
-BigNum literal instances are removed from Core during the CorePrep phase. They
-are replaced with expression to build them at runtime from machine literals
-(Word#, Int#, etc.) or from a list of Word#s.
+In the past, we had Core constructors to represent Integer and Natural literals.
+These literals were then lowered into their real Core representation only in
+Core prep. The issue with this approach is that literals have two
+representations and we have to ensure that we handle them the same everywhere
+(in every optimisation, etc.).
+
+For example (0 :: Integer) was representable in Core with both:
+
+    Lit (LitNumber LitNumInteger 0)                          -- literal
+    App (Var integerISDataCon) (Lit (LitNumber LitNumInt 0)) -- real representation
+
+Nowadays we always use the real representation for Integer and Natural literals.
+However we still have two representations for BigNat# literals. BigNat# literals
+are still lowered in Core prep into a call to a constructor function (BigNat# is
+ByteArray# and we don't have ByteArray# literals yet so we have to build them at
+runtime).
 
 Note [String literals]
 ~~~~~~~~~~~~~~~~~~~~~~
@@ -342,9 +349,8 @@ mkLitNumberWrap platform nt i = case nt of
   LitNumWord16  -> wrap @Word16
   LitNumWord32  -> wrap @Word32
   LitNumWord64  -> wrap @Word64
-  LitNumInteger -> LitNumber nt i
-  LitNumNatural
-    | i < 0     -> panic "mkLitNumberWrap: trying to create a negative Natural"
+  LitNumBigNat
+    | i < 0     -> panic "mkLitNumberWrap: trying to create a negative BigNat"
     | otherwise -> LitNumber nt i
   where
     wrap :: forall a. (Integral a, Num a) => Literal
@@ -372,29 +378,40 @@ litNumNarrow _ _ l = pprPanic "litNumNarrow: invalid literal" (ppr l)
 
 -- | Check that a given number is in the range of a numeric literal
 litNumCheckRange :: Platform -> LitNumType -> Integer -> Bool
-litNumCheckRange platform nt i = case nt of
-     LitNumInt     -> platformInIntRange platform i
-     LitNumWord    -> platformInWordRange platform i
-     LitNumInt8    -> inBoundedRange @Int8 i
-     LitNumInt16   -> inBoundedRange @Int16 i
-     LitNumInt32   -> inBoundedRange @Int32 i
-     LitNumInt64   -> inBoundedRange @Int64 i
-     LitNumWord8   -> inBoundedRange @Word8 i
-     LitNumWord16  -> inBoundedRange @Word16 i
-     LitNumWord32  -> inBoundedRange @Word32 i
-     LitNumWord64  -> inBoundedRange @Word64 i
-     LitNumNatural -> i >= 0
-     LitNumInteger -> True
+litNumCheckRange platform nt i =
+    maybe True (i >=) m_lower &&
+    maybe True (i <=) m_upper
+  where
+    (m_lower, m_upper) = litNumRange platform nt
+
+-- | Get the literal range
+litNumRange :: Platform -> LitNumType -> (Maybe Integer, Maybe Integer)
+litNumRange platform nt = case nt of
+     LitNumInt     -> (Just (platformMinInt platform), Just (platformMaxInt platform))
+     LitNumWord    -> (Just 0, Just (platformMaxWord platform))
+     LitNumInt8    -> bounded_range @Int8
+     LitNumInt16   -> bounded_range @Int16
+     LitNumInt32   -> bounded_range @Int32
+     LitNumInt64   -> bounded_range @Int64
+     LitNumWord8   -> bounded_range @Word8
+     LitNumWord16  -> bounded_range @Word16
+     LitNumWord32  -> bounded_range @Word32
+     LitNumWord64  -> bounded_range @Word64
+     LitNumBigNat  -> (Just 0, Nothing)
+  where
+    bounded_range :: forall a . (Integral a, Bounded a) => (Maybe Integer,Maybe Integer)
+    bounded_range = case boundedRange @a of
+      (mi,ma) -> (Just mi, Just ma)
 
 -- | Create a numeric 'Literal' of the given type
 mkLitNumber :: Platform -> LitNumType -> Integer -> Literal
 mkLitNumber platform nt i =
-  ASSERT2(litNumCheckRange platform nt i, integer i)
+  assertPpr (litNumCheckRange platform nt i) (integer i)
   (LitNumber nt i)
 
 -- | Creates a 'Literal' of type @Int#@
 mkLitInt :: Platform -> Integer -> Literal
-mkLitInt platform x = ASSERT2( platformInIntRange platform x,  integer x )
+mkLitInt platform x = assertPpr (platformInIntRange platform x) (integer x)
                        (mkLitIntUnchecked x)
 
 -- | Creates a 'Literal' of type @Int#@.
@@ -418,7 +435,7 @@ mkLitIntWrapC platform i = (n, i /= i')
 
 -- | Creates a 'Literal' of type @Word#@
 mkLitWord :: Platform -> Integer -> Literal
-mkLitWord platform x = ASSERT2( platformInWordRange platform x, integer x )
+mkLitWord platform x = assertPpr (platformInWordRange platform x) (integer x)
                         (mkLitWordUnchecked x)
 
 -- | Creates a 'Literal' of type @Word#@.
@@ -442,7 +459,7 @@ mkLitWordWrapC platform i = (n, i /= i')
 
 -- | Creates a 'Literal' of type @Int8#@
 mkLitInt8 :: Integer -> Literal
-mkLitInt8  x = ASSERT2( inBoundedRange @Int8 x, integer x ) (mkLitInt8Unchecked x)
+mkLitInt8  x = assertPpr (inBoundedRange @Int8 x) (integer x) (mkLitInt8Unchecked x)
 
 -- | Creates a 'Literal' of type @Int8#@.
 --   If the argument is out of the range, it is wrapped.
@@ -455,7 +472,7 @@ mkLitInt8Unchecked i = LitNumber LitNumInt8 i
 
 -- | Creates a 'Literal' of type @Word8#@
 mkLitWord8 :: Integer -> Literal
-mkLitWord8 x = ASSERT2( inBoundedRange @Word8 x, integer x ) (mkLitWord8Unchecked x)
+mkLitWord8 x = assertPpr (inBoundedRange @Word8 x) (integer x) (mkLitWord8Unchecked x)
 
 -- | Creates a 'Literal' of type @Word8#@.
 --   If the argument is out of the range, it is wrapped.
@@ -468,7 +485,7 @@ mkLitWord8Unchecked i = LitNumber LitNumWord8 i
 
 -- | Creates a 'Literal' of type @Int16#@
 mkLitInt16 :: Integer -> Literal
-mkLitInt16  x = ASSERT2( inBoundedRange @Int16 x, integer x ) (mkLitInt16Unchecked x)
+mkLitInt16  x = assertPpr (inBoundedRange @Int16 x) (integer x) (mkLitInt16Unchecked x)
 
 -- | Creates a 'Literal' of type @Int16#@.
 --   If the argument is out of the range, it is wrapped.
@@ -481,7 +498,7 @@ mkLitInt16Unchecked i = LitNumber LitNumInt16 i
 
 -- | Creates a 'Literal' of type @Word16#@
 mkLitWord16 :: Integer -> Literal
-mkLitWord16 x = ASSERT2( inBoundedRange @Word16 x, integer x ) (mkLitWord16Unchecked x)
+mkLitWord16 x = assertPpr (inBoundedRange @Word16 x) (integer x) (mkLitWord16Unchecked x)
 
 -- | Creates a 'Literal' of type @Word16#@.
 --   If the argument is out of the range, it is wrapped.
@@ -494,7 +511,7 @@ mkLitWord16Unchecked i = LitNumber LitNumWord16 i
 
 -- | Creates a 'Literal' of type @Int32#@
 mkLitInt32 :: Integer -> Literal
-mkLitInt32  x = ASSERT2( inBoundedRange @Int32 x, integer x ) (mkLitInt32Unchecked x)
+mkLitInt32  x = assertPpr (inBoundedRange @Int32 x) (integer x) (mkLitInt32Unchecked x)
 
 -- | Creates a 'Literal' of type @Int32#@.
 --   If the argument is out of the range, it is wrapped.
@@ -507,7 +524,7 @@ mkLitInt32Unchecked i = LitNumber LitNumInt32 i
 
 -- | Creates a 'Literal' of type @Word32#@
 mkLitWord32 :: Integer -> Literal
-mkLitWord32 x = ASSERT2( inBoundedRange @Word32 x, integer x ) (mkLitWord32Unchecked x)
+mkLitWord32 x = assertPpr (inBoundedRange @Word32 x) (integer x) (mkLitWord32Unchecked x)
 
 -- | Creates a 'Literal' of type @Word32#@.
 --   If the argument is out of the range, it is wrapped.
@@ -520,7 +537,7 @@ mkLitWord32Unchecked i = LitNumber LitNumWord32 i
 
 -- | Creates a 'Literal' of type @Int64#@
 mkLitInt64 :: Integer -> Literal
-mkLitInt64  x = ASSERT2( inBoundedRange @Int64 x, integer x ) (mkLitInt64Unchecked x)
+mkLitInt64  x = assertPpr (inBoundedRange @Int64 x) (integer x) (mkLitInt64Unchecked x)
 
 -- | Creates a 'Literal' of type @Int64#@.
 --   If the argument is out of the range, it is wrapped.
@@ -533,7 +550,7 @@ mkLitInt64Unchecked i = LitNumber LitNumInt64 i
 
 -- | Creates a 'Literal' of type @Word64#@
 mkLitWord64 :: Integer -> Literal
-mkLitWord64 x = ASSERT2( inBoundedRange @Word64 x, integer x ) (mkLitWord64Unchecked x)
+mkLitWord64 x = assertPpr (inBoundedRange @Word64 x) (integer x) (mkLitWord64Unchecked x)
 
 -- | Creates a 'Literal' of type @Word64#@.
 --   If the argument is out of the range, it is wrapped.
@@ -560,26 +577,24 @@ mkLitChar = LitChar
 -- e.g. some of the \"error\" functions in GHC.Err such as @GHC.Err.runtimeError@
 mkLitString :: String -> Literal
 -- stored UTF-8 encoded
-mkLitString s = LitString (bytesFS $ mkFastString s)
+mkLitString [] = LitString mempty
+mkLitString s  = LitString (utf8EncodeString s)
 
-mkLitInteger :: Integer -> Literal
-mkLitInteger x = LitNumber LitNumInteger x
-
-mkLitNatural :: Integer -> Literal
-mkLitNatural x = ASSERT2( inNaturalRange x,  integer x )
-                    (LitNumber LitNumNatural x)
+mkLitBigNat :: Integer -> Literal
+mkLitBigNat x = assertPpr (x >= 0) (integer x)
+                    (LitNumber LitNumBigNat x)
 
 isLitRubbish :: Literal -> Bool
 isLitRubbish (LitRubbish {}) = True
 isLitRubbish _               = False
 
-inNaturalRange :: Integer -> Bool
-inNaturalRange x = x >= 0
-
 inBoundedRange :: forall a. (Bounded a, Integral a) => Integer -> Bool
 inBoundedRange x  = x >= toInteger (minBound :: a) &&
                     x <= toInteger (maxBound :: a)
 
+boundedRange :: forall a. (Bounded a, Integral a) => (Integer,Integer)
+boundedRange = (toInteger (minBound :: a), toInteger (maxBound :: a))
+
 isMinBound :: Platform -> Literal -> Bool
 isMinBound _        (LitChar c)        = c == minBound
 isMinBound platform (LitNumber nt i)   = case nt of
@@ -593,8 +608,7 @@ isMinBound platform (LitNumber nt i)   = case nt of
    LitNumWord16  -> i == 0
    LitNumWord32  -> i == 0
    LitNumWord64  -> i == 0
-   LitNumNatural -> i == 0
-   LitNumInteger -> False
+   LitNumBigNat  -> i == 0
 isMinBound _        _                  = False
 
 isMaxBound :: Platform -> Literal -> Bool
@@ -610,8 +624,7 @@ isMaxBound platform (LitNumber nt i)   = case nt of
    LitNumWord16  -> i == toInteger (maxBound :: Word16)
    LitNumWord32  -> i == toInteger (maxBound :: Word32)
    LitNumWord64  -> i == toInteger (maxBound :: Word64)
-   LitNumNatural -> False
-   LitNumInteger -> False
+   LitNumBigNat  -> False
 isMaxBound _        _                  = False
 
 inCharRange :: Char -> Bool
@@ -632,7 +645,7 @@ isOneLit (LitDouble 1)   = True
 isOneLit _               = False
 
 -- | Returns the 'Integer' contained in the 'Literal', for when that makes
--- sense, i.e. for 'Char', 'Int', 'Word', 'LitInteger' and 'LitNatural'.
+-- sense, i.e. for 'Char' and numbers.
 litValue  :: Literal -> Integer
 litValue l = case isLitValue_maybe l of
    Just x  -> x
@@ -757,8 +770,7 @@ litIsTrivial :: Literal -> Bool
 --      c.f. GHC.Core.Utils.exprIsTrivial
 litIsTrivial (LitString _)    = False
 litIsTrivial (LitNumber nt _) = case nt of
-  LitNumInteger -> False
-  LitNumNatural -> False
+  LitNumBigNat  -> False
   LitNumInt     -> True
   LitNumInt8    -> True
   LitNumInt16   -> True
@@ -775,9 +787,8 @@ litIsTrivial _                  = True
 litIsDupable :: Platform -> Literal -> Bool
 --      c.f. GHC.Core.Utils.exprIsDupable
 litIsDupable platform x = case x of
-   (LitNumber nt i) -> case nt of
-      LitNumInteger -> platformInIntRange platform i
-      LitNumNatural -> platformInWordRange platform i
+   LitNumber nt i -> case nt of
+      LitNumBigNat  -> i <= platformMaxWord platform * 8 -- arbitrary, reasonable
       LitNumInt     -> True
       LitNumInt8    -> True
       LitNumInt16   -> True
@@ -788,8 +799,8 @@ litIsDupable platform x = case x of
       LitNumWord16  -> True
       LitNumWord32  -> True
       LitNumWord64  -> True
-   (LitString _) -> False
-   _             -> True
+   LitString _ -> False
+   _           -> True
 
 litFitsInChar :: Literal -> Bool
 litFitsInChar (LitNumber _ i) = i >= toInteger (ord minBound)
@@ -798,8 +809,7 @@ litFitsInChar _               = False
 
 litIsLifted :: Literal -> Bool
 litIsLifted (LitNumber nt _) = case nt of
-  LitNumInteger -> True
-  LitNumNatural -> True
+  LitNumBigNat  -> True
   LitNumInt     -> False
   LitNumInt8    -> False
   LitNumInt16   -> False
@@ -827,8 +837,7 @@ literalType (LitFloat _)      = floatPrimTy
 literalType (LitDouble _)     = doublePrimTy
 literalType (LitLabel _ _ _)  = addrPrimTy
 literalType (LitNumber lt _)  = case lt of
-   LitNumInteger -> integerTy
-   LitNumNatural -> naturalTy
+   LitNumBigNat  -> byteArrayPrimTy
    LitNumInt     -> intPrimTy
    LitNumInt8    -> int8PrimTy
    LitNumInt16   -> int16PrimTy
@@ -844,7 +853,7 @@ literalType (LitNumber lt _)  = case lt of
 literalType (LitRubbish rep)
   = mkForAllTy a Inferred (mkTyVarTy a)
   where
-    a = mkTemplateKindVar (tYPE rep)
+    a = mkTemplateKindVar (mkTYPEapp rep)
 
 {-
         Comparison
@@ -877,10 +886,9 @@ pprLiteral _       (LitString s)   = pprHsBytes s
 pprLiteral _       (LitNullAddr)   = text "__NULL"
 pprLiteral _       (LitFloat f)    = float (fromRat f) <> primFloatSuffix
 pprLiteral _       (LitDouble d)   = double (fromRat d) <> primDoubleSuffix
-pprLiteral add_par (LitNumber nt i)
+pprLiteral _       (LitNumber nt i)
    = case nt of
-       LitNumInteger -> pprIntegerVal add_par i
-       LitNumNatural -> pprIntegerVal add_par i
+       LitNumBigNat  -> integer i
        LitNumInt     -> pprPrimInt i
        LitNumInt8    -> pprPrimInt8 i
        LitNumInt16   -> pprPrimInt16 i
@@ -899,17 +907,12 @@ pprLiteral add_par (LitLabel l mb fod) =
 pprLiteral _       (LitRubbish rep)
   = text "RUBBISH" <> parens (ppr rep)
 
-pprIntegerVal :: (SDoc -> SDoc) -> Integer -> SDoc
--- See Note [Printing of literals in Core].
-pprIntegerVal add_par i | i < 0     = add_par (integer i)
-                        | otherwise = integer i
-
 {-
 Note [Printing of literals in Core]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-The function `add_par` is used to wrap parenthesis around negative integers
-(`LitInteger`) and labels (`LitLabel`), if they occur in a context requiring
-an atomic thing (for example function application).
+The function `add_par` is used to wrap parenthesis around labels (`LitLabel`),
+if they occur in a context requiring an atomic thing (for example function
+application).
 
 Although not all Core literals would be valid Haskell, we are trying to stay
 as close as possible to Haskell syntax in the printing of Core, to make it
@@ -937,7 +940,7 @@ LitWord          1##
 LitWordN         1##N
 LitFloat        -1.0#
 LitDouble       -1.0##
-LitInteger      -1                 (-1)
+LitBigNat       1
 LitLabel        "__label" ...      ("__label" ...)
 LitRubbish      "RUBBISH[...]"
 
@@ -1030,7 +1033,7 @@ c) Literal is in Ord, because (and only because) we use Ord on AltCon when
    Moreover, rubbish literals should not appear in patterns anyway.
 
 d) Why not lower LitRubbish in CoreToStg? Because it enables us to use
-   RubbishLit when unarising unboxed sums in the future, and it allows
+   LitRubbish when unarising unboxed sums in the future, and it allows
    rubbish values of e.g.  VecRep, for which we can't cough up dummy
    values in STG.
 
