@@ -1,4 +1,5 @@
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE MultiWayIf #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module     : Hadrian.Haskell.Cabal.Parse
@@ -154,12 +155,12 @@ configurePackage context@Context {..} = do
     trackArgsHash (target context (Cabal Flags stage) [] [])
     trackArgsHash (target context (Cabal Setup stage) [] [])
     verbosity   <- getVerbosity
-    when (verbosity >= Loud) $
-        putProgressInfo $ "| Package " ++ quote (pkgName package) ++ " configuration flags: " ++ unwords argList
-    let v = if verbosity >= Loud then "-v3" else "-v0"
+    let v = if verbosity >= Diagnostic then "-v3" else "-v0"
+        argList' = argList ++ ["--flags=" ++ unwords flagList, v]
+    when (verbosity >= Verbose) $
+        putProgressInfo $ "| Package " ++ quote (pkgName package) ++ " configuration flags: " ++ unwords argList'
     traced "cabal-configure" $
-        C.defaultMainWithHooksNoReadArgs hooks gpd
-            (argList ++ ["--flags=" ++ unwords flagList, v])
+        C.defaultMainWithHooksNoReadArgs hooks gpd argList'
 
     dir <- Context.buildPath context
     files <- liftIO $ getDirectoryFilesIO "." [ dir -/- "include" -/- "**"
@@ -177,7 +178,7 @@ copyPackage context@Context {..} = do
     ctxPath   <- Context.contextPath context
     pkgDbPath <- packageDbPath stage
     verbosity <- getVerbosity
-    let v = if verbosity >= Loud then "-v3" else "-v0"
+    let v = if verbosity >= Diagnostic then "-v3" else "-v0"
     traced "cabal-copy" $
         C.defaultMainWithHooksNoReadArgs C.autoconfUserHooks gpd
             [ "copy", "--builddir", ctxPath, "--target-package-db", pkgDbPath, v ]
@@ -189,7 +190,7 @@ registerPackage context@Context {..} = do
     ctxPath <- Context.contextPath context
     gpd <- pkgGenericDescription package
     verbosity <- getVerbosity
-    let v = if verbosity >= Loud then "-v3" else "-v0"
+    let v = if verbosity >= Diagnostic then "-v3" else "-v0"
     traced "cabal-register" $
         C.defaultMainWithHooksNoReadArgs C.autoconfUserHooks gpd
             [ "register", "--builddir", ctxPath, v ]
