@@ -5,7 +5,7 @@ module Settings.Builders.Common (
     module Oracles.Setting,
     module Settings,
     module UserSettings,
-    cIncludeArgs, ldArgs, cArgs, cWarnings,
+    cIncludeArgs, ldArgs, cArgs, cppArgs, cWarnings,
     packageDatabaseArgs, bootPackageDatabaseArgs
     ) where
 
@@ -15,7 +15,6 @@ import Base
 import Expression
 import Oracles.Flag
 import Oracles.Setting
-import Packages
 import Settings
 import UserSettings
 
@@ -25,10 +24,7 @@ cIncludeArgs = do
     path    <- getBuildPath
     incDirs <- getContextData includeDirs
     depDirs <- getContextData depIncludeDirs
-    stage <- getStage
-    libPath <- expr $ stageLibPath stage
-    mconcat [ notStage0 ||^ package compiler ? arg "-Iincludes"
-            , arg $ "-I" ++ libPath
+    mconcat [ notStage0 ? arg "-Irts/include"
             , arg $ "-I" ++ path
             -- Add @incDirs@ in the build directory, since some files generated
             -- with @autoconf@ may end up in the build directory.
@@ -44,6 +40,9 @@ ldArgs = mempty
 cArgs :: Args
 cArgs = mempty
 
+cppArgs :: Args
+cppArgs = mempty
+
 -- TODO: should be in a different file
 cWarnings :: Args
 cWarnings = mconcat
