@@ -31,12 +31,14 @@ data SPDXLicenseListVersion
     | SPDXLicenseListVersion_3_6
     | SPDXLicenseListVersion_3_9
     | SPDXLicenseListVersion_3_10
+    | SPDXLicenseListVersion_3_16 
   deriving (Eq, Ord, Show, Enum, Bounded)
 
 allVers :: Set.Set SPDXLicenseListVersion
 allVers =  Set.fromList [minBound .. maxBound]
 
 prettyVer :: SPDXLicenseListVersion -> Text
+prettyVer SPDXLicenseListVersion_3_16 = "SPDX License List 3.16"
 prettyVer SPDXLicenseListVersion_3_10 = "SPDX License List 3.10"
 prettyVer SPDXLicenseListVersion_3_9  = "SPDX License List 3.9"
 prettyVer SPDXLicenseListVersion_3_6  = "SPDX License List 3.6"
@@ -44,6 +46,7 @@ prettyVer SPDXLicenseListVersion_3_2  = "SPDX License List 3.2"
 prettyVer SPDXLicenseListVersion_3_0  = "SPDX License List 3.0"
 
 suffixVer :: SPDXLicenseListVersion -> String
+suffixVer SPDXLicenseListVersion_3_16 = "_3_16"
 suffixVer SPDXLicenseListVersion_3_10 = "_3_10"
 suffixVer SPDXLicenseListVersion_3_9  = "_3_9"
 suffixVer SPDXLicenseListVersion_3_6  = "_3_6"
@@ -54,7 +57,7 @@ suffixVer SPDXLicenseListVersion_3_0  = "_3_0"
 -- Per version
 -------------------------------------------------------------------------------
 
-data PerV a = PerV a a a a a
+data PerV a = PerV a a a a a a
   deriving (Show, Functor, Foldable, Traversable)
 
 class Functor f => Representable i f | f -> i where
@@ -62,11 +65,12 @@ class Functor f => Representable i f | f -> i where
     tabulate :: (i -> a) -> f a
 
 instance Representable SPDXLicenseListVersion PerV where
-    index SPDXLicenseListVersion_3_0  (PerV x _ _ _ _) = x
-    index SPDXLicenseListVersion_3_2  (PerV _ x _ _ _) = x
-    index SPDXLicenseListVersion_3_6  (PerV _ _ x _ _) = x
-    index SPDXLicenseListVersion_3_9  (PerV _ _ _ x _) = x
-    index SPDXLicenseListVersion_3_10 (PerV _ _ _ _ x) = x
+    index SPDXLicenseListVersion_3_0  (PerV x _ _ _ _ _) = x
+    index SPDXLicenseListVersion_3_2  (PerV _ x _ _ _ _) = x
+    index SPDXLicenseListVersion_3_6  (PerV _ _ x _ _ _) = x
+    index SPDXLicenseListVersion_3_9  (PerV _ _ _ x _ _) = x
+    index SPDXLicenseListVersion_3_10 (PerV _ _ _ _ x _) = x
+    index SPDXLicenseListVersion_3_16 (PerV _ _ _ _ _ x) = x
 
     tabulate f = PerV
         (f SPDXLicenseListVersion_3_0)
@@ -74,6 +78,7 @@ instance Representable SPDXLicenseListVersion PerV where
         (f SPDXLicenseListVersion_3_6)
         (f SPDXLicenseListVersion_3_9)
         (f SPDXLicenseListVersion_3_10)
+        (f SPDXLicenseListVersion_3_16)
 
 -------------------------------------------------------------------------------
 -- Sorting
@@ -89,7 +94,7 @@ instance Ord OrdT where
         | otherwise          = compare a b
 
 -------------------------------------------------------------------------------
--- Commmons
+-- Commons
 -------------------------------------------------------------------------------
 
 header :: String
