@@ -1,15 +1,15 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE PatternGuards #-}
-#if __GLASGOW_HASKELL__
-{-# LANGUAGE MagicHash, DeriveDataTypeable, StandaloneDeriving #-}
+#ifdef __GLASGOW_HASKELL__
+{-# LANGUAGE DeriveLift #-}
+{-# LANGUAGE MagicHash #-}
+{-# LANGUAGE StandaloneDeriving #-}
+{-# LANGUAGE TypeFamilies #-}
 #endif
 #if !defined(TESTING) && defined(__GLASGOW_HASKELL__)
 {-# LANGUAGE Trustworthy #-}
 #endif
-#if __GLASGOW_HASKELL__ >= 708
-{-# LANGUAGE TypeFamilies #-}
-#endif
 
 {-# OPTIONS_HADDOCK not-home #-}
 
@@ -65,9 +65,9 @@
 -- it is likely that many values lie close to each other. The asymptotics are
 -- not affected by this optimization.
 --
--- Many operations have a worst-case complexity of /O(min(n,W))/.
+-- Many operations have a worst-case complexity of \(O(\min(n,W))\).
 -- This means that the operation can become linear in the number of
--- elements with a maximum of /W/ -- the number of bits in an 'Int'
+-- elements with a maximum of \(W\) -- the number of bits in an 'Int'
 -- (32 or 64).
 --
 -- @since 0.5.9
@@ -193,20 +193,11 @@ import Control.DeepSeq (NFData(rnf))
 import Data.Bits
 import qualified Data.List as List
 import Data.Maybe (fromMaybe)
-#if !MIN_VERSION_base(4,8,0)
-import Data.Monoid (Monoid(..))
-import Data.Word (Word)
-#endif
-#if MIN_VERSION_base(4,9,0)
 import Data.Semigroup (Semigroup(stimes))
-#endif
-#if !(MIN_VERSION_base(4,11,0)) && MIN_VERSION_base(4,9,0)
+#if !(MIN_VERSION_base(4,11,0))
 import Data.Semigroup (Semigroup((<>)))
 #endif
-#if MIN_VERSION_base(4,9,0)
 import Data.Semigroup (stimesIdempotentMonoid)
-#endif
-import Data.Typeable
 import Prelude hiding (filter, foldr, foldl, null, map)
 
 import Utils.Containers.Internal.BitUtil
@@ -220,17 +211,14 @@ import Text.Read
 
 #if __GLASGOW_HASKELL__
 import qualified GHC.Exts
-#if !(MIN_VERSION_base(4,8,0) && (WORD_SIZE_IN_BITS==64))
+#  if !(WORD_SIZE_IN_BITS==64)
 import qualified GHC.Int
-#endif
+#  endif
+import Language.Haskell.TH.Syntax (Lift)
 #endif
 
 import qualified Data.Foldable as Foldable
-#if MIN_VERSION_base(4,8,0)
 import Data.Functor.Identity (Identity(..))
-#else
-import Data.Foldable (Foldable())
-#endif
 
 infixl 9 \\{-This comment teaches CPP correct behaviour -}
 
@@ -248,7 +236,7 @@ intFromNat w = fromIntegral w
 {--------------------------------------------------------------------
   Operators
 --------------------------------------------------------------------}
--- | /O(n+m)/. See 'difference'.
+-- | \(O(n+m)\). See 'difference'.
 (\\) :: IntSet -> IntSet -> IntSet
 m1 \\ m2 = difference m1 m2
 
@@ -283,19 +271,20 @@ type Mask   = Int
 type BitMap = Word
 type Key    = Int
 
+#ifdef __GLASGOW_HASKELL__
+-- | @since FIXME
+deriving instance Lift IntSet
+#endif
+
 instance Monoid IntSet where
     mempty  = empty
     mconcat = unions
-#if !(MIN_VERSION_base(4,9,0))
-    mappend = union
-#else
     mappend = (<>)
 
 -- | @since 0.5.7
 instance Semigroup IntSet where
     (<>)    = union
     stimes  = stimesIdempotentMonoid
-#endif
 
 #if __GLASGOW_HASKELL__
 
@@ -325,13 +314,13 @@ intSetDataType = mkDataType "Data.IntSet.Internal.IntSet" [fromListConstr]
 {--------------------------------------------------------------------
   Query
 --------------------------------------------------------------------}
--- | /O(1)/. Is the set empty?
+-- | \(O(1)\). Is the set empty?
 null :: IntSet -> Bool
 null Nil = True
 null _   = False
 {-# INLINE null #-}
 
--- | /O(n)/. Cardinality of the set.
+-- | \(O(n)\). Cardinality of the set.
 size :: IntSet -> Int
 size = go 0
   where
@@ -339,7 +328,7 @@ size = go 0
     go acc (Tip _ bm) = acc + bitcount 0 bm
     go acc Nil = acc
 
--- | /O(min(n,W))/. Is the value a member of the set?
+-- | \(O(\min(n,W))\). Is the value a member of the set?
 
 -- See Note: Local 'go' functions and capturing.
 member :: Key -> IntSet -> Bool
@@ -352,11 +341,11 @@ member !x = go
     go (Tip y bm) = prefixOf x == y && bitmapOf x .&. bm /= 0
     go Nil = False
 
--- | /O(min(n,W))/. Is the element not in the set?
+-- | \(O(\min(n,W))\). Is the element not in the set?
 notMember :: Key -> IntSet -> Bool
 notMember k = not . member k
 
--- | /O(log n)/. Find largest element smaller than the given one.
+-- | \(O(\log n)\). Find largest element smaller than the given one.
 --
 -- > lookupLT 3 (fromList [3, 5]) == Nothing
 -- > lookupLT 5 (fromList [3, 5]) == Just 3
@@ -377,7 +366,7 @@ lookupLT !x t = case t of
     go def Nil = unsafeFindMax def
 
 
--- | /O(log n)/. Find smallest element greater than the given one.
+-- | \(O(\log n)\). Find smallest element greater than the given one.
 --
 -- > lookupGT 4 (fromList [3, 5]) == Just 5
 -- > lookupGT 5 (fromList [3, 5]) == Nothing
@@ -398,7 +387,7 @@ lookupGT !x t = case t of
     go def Nil = unsafeFindMin def
 
 
--- | /O(log n)/. Find largest element smaller or equal to the given one.
+-- | \(O(\log n)\). Find largest element smaller or equal to the given one.
 --
 -- > lookupLE 2 (fromList [3, 5]) == Nothing
 -- > lookupLE 4 (fromList [3, 5]) == Just 3
@@ -420,7 +409,7 @@ lookupLE !x t = case t of
     go def Nil = unsafeFindMax def
 
 
--- | /O(log n)/. Find smallest element greater or equal to the given one.
+-- | \(O(\log n)\). Find smallest element greater or equal to the given one.
 --
 -- > lookupGE 3 (fromList [3, 5]) == Just 3
 -- > lookupGE 4 (fromList [3, 5]) == Just 5
@@ -460,13 +449,13 @@ unsafeFindMax (Bin _ _ _ r) = unsafeFindMax r
 {--------------------------------------------------------------------
   Construction
 --------------------------------------------------------------------}
--- | /O(1)/. The empty set.
+-- | \(O(1)\). The empty set.
 empty :: IntSet
 empty
   = Nil
 {-# INLINE empty #-}
 
--- | /O(1)/. A set of one element.
+-- | \(O(1)\). A set of one element.
 singleton :: Key -> IntSet
 singleton x
   = Tip (prefixOf x) (bitmapOf x)
@@ -475,7 +464,7 @@ singleton x
 {--------------------------------------------------------------------
   Insert
 --------------------------------------------------------------------}
--- | /O(min(n,W))/. Add a value to the set. There is no left- or right bias for
+-- | \(O(\min(n,W))\). Add a value to the set. There is no left- or right bias for
 -- IntSets.
 insert :: Key -> IntSet -> IntSet
 insert !x = insertBM (prefixOf x) (bitmapOf x)
@@ -491,7 +480,7 @@ insertBM kx bm t@(Tip kx' bm')
   | otherwise = link kx (Tip kx bm) kx' t
 insertBM kx bm Nil = Tip kx bm
 
--- | /O(min(n,W))/. Delete a value in the set. Returns the
+-- | \(O(\min(n,W))\). Delete a value in the set. Returns the
 -- original set when the value was not present.
 delete :: Key -> IntSet -> IntSet
 delete !x = deleteBM (prefixOf x) (bitmapOf x)
@@ -508,7 +497,7 @@ deleteBM kx bm t@(Tip kx' bm')
   | otherwise = t
 deleteBM _ _ Nil = Nil
 
--- | /O(min(n,W))/. @('alterF' f x s)@ can delete or insert @x@ in @s@ depending
+-- | \(O(\min(n,W))\). @('alterF' f x s)@ can delete or insert @x@ in @s@ depending
 -- on whether it is already present in @s@.
 --
 -- In short:
@@ -541,9 +530,7 @@ alterF f k s = fmap choose (f member_)
  #-}
 #endif
 
-#if MIN_VERSION_base(4,8,0)
 {-# SPECIALIZE alterF :: (Bool -> Identity Bool) -> Key -> IntSet -> Identity IntSet #-}
-#endif
 
 {--------------------------------------------------------------------
   Union
@@ -554,7 +541,7 @@ unions xs
   = Foldable.foldl' union empty xs
 
 
--- | /O(n+m)/. The union of two sets.
+-- | \(O(n+m)\). The union of two sets.
 union :: IntSet -> IntSet -> IntSet
 union t1@(Bin p1 m1 l1 r1) t2@(Bin p2 m2 l2 r2)
   | shorter m1 m2  = union1
@@ -579,7 +566,7 @@ union Nil t = t
 {--------------------------------------------------------------------
   Difference
 --------------------------------------------------------------------}
--- | /O(n+m)/. Difference between two sets.
+-- | \(O(n+m)\). Difference between two sets.
 difference :: IntSet -> IntSet -> IntSet
 difference t1@(Bin p1 m1 l1 r1) t2@(Bin p2 m2 l2 r2)
   | shorter m1 m2  = difference1
@@ -613,7 +600,7 @@ difference Nil _     = Nil
 {--------------------------------------------------------------------
   Intersection
 --------------------------------------------------------------------}
--- | /O(n+m)/. The intersection of two sets.
+-- | \(O(n+m)\). The intersection of two sets.
 intersection :: IntSet -> IntSet -> IntSet
 intersection t1@(Bin p1 m1 l1 r1) t2@(Bin p2 m2 l2 r2)
   | shorter m1 m2  = intersection1
@@ -652,7 +639,7 @@ intersection Nil _ = Nil
 {--------------------------------------------------------------------
   Subset
 --------------------------------------------------------------------}
--- | /O(n+m)/. Is this a proper subset? (ie. a subset but not equal).
+-- | \(O(n+m)\). Is this a proper subset? (ie. a subset but not equal).
 isProperSubsetOf :: IntSet -> IntSet -> Bool
 isProperSubsetOf t1 t2
   = case subsetCmp t1 t2 of
@@ -691,7 +678,7 @@ subsetCmp (Tip _ _) Nil = GT -- disjoint
 subsetCmp Nil Nil = EQ
 subsetCmp Nil _   = LT
 
--- | /O(n+m)/. Is this a subset?
+-- | \(O(n+m)\). Is this a subset?
 -- @(s1 \`isSubsetOf\` s2)@ tells whether @s1@ is a subset of @s2@.
 
 isSubsetOf :: IntSet -> IntSet -> Bool
@@ -713,7 +700,7 @@ isSubsetOf Nil _         = True
 {--------------------------------------------------------------------
   Disjoint
 --------------------------------------------------------------------}
--- | /O(n+m)/. Check whether two sets are disjoint (i.e. their intersection
+-- | \(O(n+m)\). Check whether two sets are disjoint (i.e. their intersection
 --   is empty).
 --
 -- > disjoint (fromList [2,4,6])   (fromList [1,3])     == True
@@ -761,7 +748,7 @@ disjoint Nil _ = True
 {--------------------------------------------------------------------
   Filter
 --------------------------------------------------------------------}
--- | /O(n)/. Filter all elements that satisfy some predicate.
+-- | \(O(n)\). Filter all elements that satisfy some predicate.
 filter :: (Key -> Bool) -> IntSet -> IntSet
 filter predicate t
   = case t of
@@ -774,7 +761,7 @@ filter predicate t
                          | otherwise           = bm
         {-# INLINE bitPred #-}
 
--- | /O(n)/. partition the set according to some predicate.
+-- | \(O(n)\). partition the set according to some predicate.
 partition :: (Key -> Bool) -> IntSet -> (IntSet,IntSet)
 partition predicate0 t0 = toPair $ go predicate0 t0
   where
@@ -793,7 +780,7 @@ partition predicate0 t0 = toPair $ go predicate0 t0
             {-# INLINE bitPred #-}
 
 
--- | /O(min(n,W))/. The expression (@'split' x set@) is a pair @(set1,set2)@
+-- | \(O(\min(n,W))\). The expression (@'split' x set@) is a pair @(set1,set2)@
 -- where @set1@ comprises the elements of @set@ less than @x@ and @set2@
 -- comprises the elements of @set@ greater than @x@.
 --
@@ -827,7 +814,7 @@ split x t =
                   higherBitmap = complement (lowerBitmap + bitmapOf x')
     go _ Nil = (Nil :*: Nil)
 
--- | /O(min(n,W))/. Performs a 'split' but also returns whether the pivot
+-- | \(O(\min(n,W))\). Performs a 'split' but also returns whether the pivot
 -- element was found in the original set.
 splitMember :: Key -> IntSet -> (IntSet,Bool,IntSet)
 splitMember x t =
@@ -865,7 +852,7 @@ splitMember x t =
   Min/Max
 ----------------------------------------------------------------------}
 
--- | /O(min(n,W))/. Retrieves the maximal key of the set, and the set
+-- | \(O(\min(n,W))\). Retrieves the maximal key of the set, and the set
 -- stripped of that element, or 'Nothing' if passed an empty set.
 maxView :: IntSet -> Maybe (Key, IntSet)
 maxView t =
@@ -877,7 +864,7 @@ maxView t =
     go (Tip kx bm) = case highestBitSet bm of bi -> (kx + bi, tip kx (bm .&. complement (bitmapOfSuffix bi)))
     go Nil = error "maxView Nil"
 
--- | /O(min(n,W))/. Retrieves the minimal key of the set, and the set
+-- | \(O(\min(n,W))\). Retrieves the minimal key of the set, and the set
 -- stripped of that element, or 'Nothing' if passed an empty set.
 minView :: IntSet -> Maybe (Key, IntSet)
 minView t =
@@ -889,20 +876,20 @@ minView t =
     go (Tip kx bm) = case lowestBitSet bm of bi -> (kx + bi, tip kx (bm .&. complement (bitmapOfSuffix bi)))
     go Nil = error "minView Nil"
 
--- | /O(min(n,W))/. Delete and find the minimal element.
+-- | \(O(\min(n,W))\). Delete and find the minimal element.
 --
 -- > deleteFindMin set = (findMin set, deleteMin set)
 deleteFindMin :: IntSet -> (Key, IntSet)
 deleteFindMin = fromMaybe (error "deleteFindMin: empty set has no minimal element") . minView
 
--- | /O(min(n,W))/. Delete and find the maximal element.
+-- | \(O(\min(n,W))\). Delete and find the maximal element.
 --
 -- > deleteFindMax set = (findMax set, deleteMax set)
 deleteFindMax :: IntSet -> (Key, IntSet)
 deleteFindMax = fromMaybe (error "deleteFindMax: empty set has no maximal element") . maxView
 
 
--- | /O(min(n,W))/. The minimal element of the set.
+-- | \(O(\min(n,W))\). The minimal element of the set.
 findMin :: IntSet -> Key
 findMin Nil = error "findMin: empty set has no minimal element"
 findMin (Tip kx bm) = kx + lowestBitSet bm
@@ -913,7 +900,7 @@ findMin (Bin _ m l r)
           find (Bin _ _ l' _) = find l'
           find Nil            = error "findMin Nil"
 
--- | /O(min(n,W))/. The maximal element of a set.
+-- | \(O(\min(n,W))\). The maximal element of a set.
 findMax :: IntSet -> Key
 findMax Nil = error "findMax: empty set has no maximal element"
 findMax (Tip kx bm) = kx + highestBitSet bm
@@ -925,14 +912,14 @@ findMax (Bin _ m l r)
           find Nil            = error "findMax Nil"
 
 
--- | /O(min(n,W))/. Delete the minimal element. Returns an empty set if the set is empty.
+-- | \(O(\min(n,W))\). Delete the minimal element. Returns an empty set if the set is empty.
 --
 -- Note that this is a change of behaviour for consistency with 'Data.Set.Set' &#8211;
 -- versions prior to 0.5 threw an error if the 'IntSet' was already empty.
 deleteMin :: IntSet -> IntSet
 deleteMin = maybe Nil snd . minView
 
--- | /O(min(n,W))/. Delete the maximal element. Returns an empty set if the set is empty.
+-- | \(O(\min(n,W))\). Delete the maximal element. Returns an empty set if the set is empty.
 --
 -- Note that this is a change of behaviour for consistency with 'Data.Set.Set' &#8211;
 -- versions prior to 0.5 threw an error if the 'IntSet' was already empty.
@@ -943,7 +930,7 @@ deleteMax = maybe Nil snd . maxView
   Map
 ----------------------------------------------------------------------}
 
--- | /O(n*min(n,W))/.
+-- | \(O(n \min(n,W))\).
 -- @'map' f s@ is the set obtained by applying @f@ to each element of @s@.
 --
 -- It's worth noting that the size of the result may be smaller if,
@@ -952,7 +939,7 @@ deleteMax = maybe Nil snd . maxView
 map :: (Key -> Key) -> IntSet -> IntSet
 map f = fromList . List.map f . toList
 
--- | /O(n)/. The
+-- | \(O(n)\). The
 --
 -- @'mapMonotonic' f s == 'map' f s@, but works only when @f@ is strictly increasing.
 -- /The precondition is not checked./
@@ -972,7 +959,7 @@ mapMonotonic f = fromDistinctAscList . List.map f . toAscList
 {--------------------------------------------------------------------
   Fold
 --------------------------------------------------------------------}
--- | /O(n)/. Fold the elements in the set using the given right-associative
+-- | \(O(n)\). Fold the elements in the set using the given right-associative
 -- binary operator. This function is an equivalent of 'foldr' and is present
 -- for compatibility only.
 --
@@ -981,7 +968,7 @@ fold :: (Key -> b -> b) -> b -> IntSet -> b
 fold = foldr
 {-# INLINE fold #-}
 
--- | /O(n)/. Fold the elements in the set using the given right-associative
+-- | \(O(n)\). Fold the elements in the set using the given right-associative
 -- binary operator, such that @'foldr' f z == 'Prelude.foldr' f z . 'toAscList'@.
 --
 -- For example,
@@ -998,7 +985,7 @@ foldr f z = \t ->      -- Use lambda t to be inlinable with two arguments only.
     go z' (Bin _ _ l r) = go (go z' r) l
 {-# INLINE foldr #-}
 
--- | /O(n)/. A strict version of 'foldr'. Each application of the operator is
+-- | \(O(n)\). A strict version of 'foldr'. Each application of the operator is
 -- evaluated before using the result in the next application. This
 -- function is strict in the starting value.
 foldr' :: (Key -> b -> b) -> b -> IntSet -> b
@@ -1012,7 +999,7 @@ foldr' f z = \t ->      -- Use lambda t to be inlinable with two arguments only.
     go z' (Bin _ _ l r) = go (go z' r) l
 {-# INLINE foldr' #-}
 
--- | /O(n)/. Fold the elements in the set using the given left-associative
+-- | \(O(n)\). Fold the elements in the set using the given left-associative
 -- binary operator, such that @'foldl' f z == 'Prelude.foldl' f z . 'toAscList'@.
 --
 -- For example,
@@ -1029,7 +1016,7 @@ foldl f z = \t ->      -- Use lambda t to be inlinable with two arguments only.
     go z' (Bin _ _ l r) = go (go z' l) r
 {-# INLINE foldl #-}
 
--- | /O(n)/. A strict version of 'foldl'. Each application of the operator is
+-- | \(O(n)\). A strict version of 'foldl'. Each application of the operator is
 -- evaluated before using the result in the next application. This
 -- function is strict in the starting value.
 foldl' :: (a -> Key -> a) -> a -> IntSet -> a
@@ -1046,7 +1033,7 @@ foldl' f z = \t ->      -- Use lambda t to be inlinable with two arguments only.
 {--------------------------------------------------------------------
   List variations
 --------------------------------------------------------------------}
--- | /O(n)/. An alias of 'toAscList'. The elements of a set in ascending order.
+-- | \(O(n)\). An alias of 'toAscList'. The elements of a set in ascending order.
 -- Subject to list fusion.
 elems :: IntSet -> [Key]
 elems
@@ -1055,7 +1042,8 @@ elems
 {--------------------------------------------------------------------
   Lists
 --------------------------------------------------------------------}
-#if __GLASGOW_HASKELL__ >= 708
+
+#ifdef __GLASGOW_HASKELL__
 -- | @since 0.5.6.2
 instance GHC.Exts.IsList IntSet where
   type Item IntSet = Key
@@ -1063,17 +1051,17 @@ instance GHC.Exts.IsList IntSet where
   toList   = toList
 #endif
 
--- | /O(n)/. Convert the set to a list of elements. Subject to list fusion.
+-- | \(O(n)\). Convert the set to a list of elements. Subject to list fusion.
 toList :: IntSet -> [Key]
 toList
   = toAscList
 
--- | /O(n)/. Convert the set to an ascending list of elements. Subject to list
+-- | \(O(n)\). Convert the set to an ascending list of elements. Subject to list
 -- fusion.
 toAscList :: IntSet -> [Key]
 toAscList = foldr (:) []
 
--- | /O(n)/. Convert the set to a descending list of elements. Subject to list
+-- | \(O(n)\). Convert the set to a descending list of elements. Subject to list
 -- fusion.
 toDescList :: IntSet -> [Key]
 toDescList = foldl (flip (:)) []
@@ -1108,26 +1096,26 @@ foldlFB = foldl
 #endif
 
 
--- | /O(n*min(n,W))/. Create a set from a list of integers.
+-- | \(O(n \min(n,W))\). Create a set from a list of integers.
 fromList :: [Key] -> IntSet
 fromList xs
   = Foldable.foldl' ins empty xs
   where
     ins t x  = insert x t
 
--- | /O(n)/. Build a set from an ascending list of elements.
+-- | \(O(n)\). Build a set from an ascending list of elements.
 -- /The precondition (input list is ascending) is not checked./
 fromAscList :: [Key] -> IntSet
 fromAscList = fromMonoList
 {-# NOINLINE fromAscList #-}
 
--- | /O(n)/. Build a set from an ascending list of distinct elements.
+-- | \(O(n)\). Build a set from an ascending list of distinct elements.
 -- /The precondition (input list is strictly ascending) is not checked./
 fromDistinctAscList :: [Key] -> IntSet
 fromDistinctAscList = fromAscList
 {-# INLINE fromDistinctAscList #-}
 
--- | /O(n)/. Build a set from a monotonic list of elements.
+-- | \(O(n)\). Build a set from a monotonic list of elements.
 --
 -- The precise conditions under which this function works are subtle:
 -- For any branch mask, keys with the same prefix w.r.t. the branch
@@ -1243,12 +1231,6 @@ instance Read IntSet where
     return (fromList xs,t)
 #endif
 
-{--------------------------------------------------------------------
-  Typeable
---------------------------------------------------------------------}
-
-INSTANCE_TYPEABLE0(IntSet)
-
 {--------------------------------------------------------------------
   NFData
 --------------------------------------------------------------------}
@@ -1261,14 +1243,14 @@ instance NFData IntSet where rnf x = seq x ()
 {--------------------------------------------------------------------
   Debugging
 --------------------------------------------------------------------}
--- | /O(n)/. Show the tree that implements the set. The tree is shown
+-- | \(O(n)\). Show the tree that implements the set. The tree is shown
 -- in a compressed, hanging format.
 showTree :: IntSet -> String
 showTree s
   = showTreeWith True False s
 
 
-{- | /O(n)/. The expression (@'showTreeWith' hang wide map@) shows
+{- | \(O(n)\). The expression (@'showTreeWith' hang wide map@) shows
  the tree that implements the set. If @hang@ is
  'True', a /hanging/ tree is shown otherwise a rotated tree is shown. If
  @wide@ is 'True', an extra wide version is shown.
@@ -1375,11 +1357,7 @@ tip kx bm = Tip kx bm
 ----------------------------------------------------------------------}
 
 suffixBitMask :: Int
-#if MIN_VERSION_base(4,7,0)
 suffixBitMask = finiteBitSize (undefined::Word) - 1
-#else
-suffixBitMask = bitSize (undefined::Word) - 1
-#endif
 {-# INLINE suffixBitMask #-}
 
 prefixBitMask :: Int
@@ -1474,7 +1452,7 @@ foldr'Bits :: Int -> (Int -> a -> a) -> a -> Nat -> a
 #if defined(__GLASGOW_HASKELL__) && (WORD_SIZE_IN_BITS==32 || WORD_SIZE_IN_BITS==64)
 indexOfTheOnlyBit :: Nat -> Int
 {-# INLINE indexOfTheOnlyBit #-}
-#if MIN_VERSION_base(4,8,0) && (WORD_SIZE_IN_BITS==64)
+#if WORD_SIZE_IN_BITS==64
 indexOfTheOnlyBit bitmask = countTrailingZeros bitmask
 
 lowestBitSet x = countTrailingZeros x
@@ -1625,7 +1603,7 @@ foldr'Bits prefix f z bm = let lb = lowestBitSet bm
   Utilities
 --------------------------------------------------------------------}
 
--- | /O(1)/.  Decompose a set into pieces based on the structure of the underlying
+-- | \(O(1)\).  Decompose a set into pieces based on the structure of the underlying
 -- tree.  This function is useful for consuming a set in parallel.
 --
 -- No guarantee is made as to the sizes of the pieces; an internal, but
