@@ -1,7 +1,10 @@
 {-# LANGUAGE MagicHash, NoImplicitPrelude, TypeFamilies, UnboxedTuples,
              MultiParamTypeClasses, RoleAnnotations, CPP, TypeOperators,
-             PolyKinds, NegativeLiterals, DataKinds #-}
+             PolyKinds, NegativeLiterals, DataKinds, ScopedTypeVariables,
+             TypeApplications, StandaloneKindSignatures,
+             FlexibleInstances, UndecidableInstances #-}
 -- NegativeLiterals: see Note [Fixity of (->)]
+{-# OPTIONS_HADDOCK print-explicit-runtime-reps #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  GHC.Types
@@ -25,6 +28,7 @@ module GHC.Types (
         --    Lists:          []( [], (:) )
         --    Type equality:  (~)( Eq# )
 
+        -- * Built-in types
         Bool(..), Char(..), Int(..), Word(..),
         Float(..), Double(..),
         Ordering(..), IO(..),
@@ -32,22 +36,27 @@ module GHC.Types (
         SPEC(..),
         Symbol,
         Any,
-        type (~~), Coercible,
+
+        -- * Type equality
+        type (~), type (~~), Coercible,
+
+        -- * Representation polymorphism
         TYPE, Levity(..), RuntimeRep(..),
         LiftedRep, UnliftedRep,
         Type, UnliftedType, Constraint,
           -- The historical type * should ideally be written as
           -- `type *`, without the parentheses. But that's a true
           -- pain to parse, and for little gain.
+        ZeroBitRep, ZeroBitType,
         VecCount(..), VecElem(..),
         Void#,
 
+        -- * Multiplicity types
+        Multiplicity(..), MultMul,
+
         -- * Runtime type representation
         Module(..), TrName(..), TyCon(..), TypeLitSort(..),
         KindRep(..), KindBndr,
-
-        -- * Multiplicity Types
-        Multiplicity(..), MultMul
     ) where
 
 import GHC.Prim
@@ -76,7 +85,7 @@ This declaration is important for :info (->) command (issue #10145)
 
 -- | The regular function type
 type (->) = FUN 'Many
--- See Note [Linear Types] in Multiplicity
+-- See Note [Linear types] in Multiplicity
 
 {- *********************************************************************
 *                                                                      *
@@ -93,6 +102,11 @@ type LiftedRep = 'BoxedRep 'Lifted
 -- | The runtime representation of unlifted types.
 type UnliftedRep = 'BoxedRep 'Unlifted
 
+-- | The runtime representation of a zero-width tuple,
+--   represented by no bits at all
+type ZeroBitRep = 'TupleRep '[]
+
+-------------------------
 -- | The kind of types with lifted values. For example @Int :: Type@.
 type Type = TYPE LiftedRep
 
@@ -100,6 +114,10 @@ type Type = TYPE LiftedRep
 -- unlifted data type, using @-XUnliftedDataTypes@.
 type UnliftedType = TYPE UnliftedRep
 
+-- | The kind of the empty unboxed tuple type (# #)
+type ZeroBitType = TYPE ZeroBitRep
+
+-------------------------
 data Multiplicity = Many | One
 
 type family MultMul (a :: Multiplicity) (b :: Multiplicity) :: Multiplicity where
@@ -237,14 +255,12 @@ newtype IO a = IO (State# RealWorld -> (# State# RealWorld, a #))
 *                                                                      *
                     (~) and Coercible
 
-   NB: (~) is built-in syntax, and hence not explicitly exported
 *                                                                      *
 ********************************************************************* -}
 
 {-
 Note [Kind-changing of (~) and Coercible]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
 (~) and Coercible are tricky to define. To the user, they must appear as
 constraints, but we cannot define them as such in Haskell. But we also cannot
 just define them only in GHC.Prim (like (->)), because we need a real module
@@ -444,12 +460,12 @@ data RuntimeRep = VecRep VecCount VecElem   -- ^ a SIMD vector type
                 | Int8Rep         -- ^ signed,  8-bit value
                 | Int16Rep        -- ^ signed, 16-bit value
                 | Int32Rep        -- ^ signed, 32-bit value
-                | Int64Rep        -- ^ signed, 64-bit value (on 32-bit only)
+                | Int64Rep        -- ^ signed, 64-bit value
                 | WordRep         -- ^ unsigned, word-sized value
                 | Word8Rep        -- ^ unsigned,  8-bit value
                 | Word16Rep       -- ^ unsigned, 16-bit value
                 | Word32Rep       -- ^ unsigned, 32-bit value
-                | Word64Rep       -- ^ unsigned, 64-bit value (on 32-bit only)
+                | Word64Rep       -- ^ unsigned, 64-bit value
                 | AddrRep         -- ^ A pointer, but /not/ to a Haskell value
                 | FloatRep        -- ^ a 32-bit floating point number
                 | DoubleRep       -- ^ a 64-bit floating point number
@@ -520,12 +536,6 @@ data TrName
 -- | A de Bruijn index for a binder within a 'KindRep'.
 type KindBndr = Int
 
-#if WORD_SIZE_IN_BITS < 64
-#define WORD64_TY Word64#
-#else
-#define WORD64_TY Word#
-#endif
-
 -- | The representation produced by GHC for conjuring up the kind of a
 -- 'Data.Typeable.TypeRep'.
 
@@ -543,8 +553,8 @@ data TypeLitSort = TypeLitSymbol
                  | TypeLitChar
 
 -- Show instance for TyCon found in GHC.Show
-data TyCon = TyCon WORD64_TY  -- ^ Fingerprint (high)
-                   WORD64_TY  -- ^ Fingerprint (low)
+data TyCon = TyCon Word64#    -- ^ Fingerprint (high)
+                   Word64#    -- ^ Fingerprint (low)
                    Module     -- ^ Module in which this is defined
                    TrName     -- ^ Type constructor name
                    Int#       -- ^ How many kind variables do we accept?
