@@ -27,17 +27,6 @@
 
 #include "BeginPrivate.h"
 
-// We never want a Capability to overlap a cache line with
-// anything else, so round it up to a cache line size:
-#if defined(s390x_HOST_ARCH)
-#define CAPABILITY_ALIGNMENT 256
-#elif defined(mingw32_HOST_OS)
-// N.B. it's quite unclear why this special case exists
-#define CAPABILITY_ALIGNMENT 1
-#else
-#define CAPABILITY_ALIGNMENT 64
-#endif
-
 /* N.B. This must be consistent with CapabilityPublic in RtsAPI.h */
 struct Capability_ {
     // State required by the STG virtual machine when running Haskell
@@ -99,9 +88,6 @@ struct Capability_ {
 
     // The update remembered set for the non-moving collector
     UpdRemSet upd_rem_set;
-    // Array of current segments for the non-moving collector.
-    // Of length NONMOVING_ALLOCA_CNT.
-    struct NonmovingSegment **current_segments;
 
     // block for allocating pinned objects into
     bdescr *pinned_object_block;
@@ -183,12 +169,14 @@ struct Capability_ {
     StgTRecHeader *free_trec_headers;
     uint32_t transaction_tokens;
 } // typedef Capability is defined in RtsAPI.h
-  ATTRIBUTE_ALIGNED(CAPABILITY_ALIGNMENT)
-;
-
-// We allocate arrays of Capabilities therefore we must ensure that the size is
-// a multiple of the claimed alignment
-GHC_STATIC_ASSERT(sizeof(struct Capability_) % CAPABILITY_ALIGNMENT == 0, "Capability size does not match cache size");
+  // We never want a Capability to overlap a cache line with anything
+  // else, so round it up to a cache line size:
+#if defined(s390x_HOST_ARCH)
+  ATTRIBUTE_ALIGNED(256)
+#elif !defined(mingw32_HOST_OS)
+  ATTRIBUTE_ALIGNED(64)
+#endif
+  ;
 
 #if defined(THREADED_RTS)
 #define ASSERT_TASK_ID(task) ASSERT(task->id == osThreadId())
@@ -265,10 +253,10 @@ INLINE_HEADER void releaseCapability_ (Capability* cap STG_UNUSED,
                                        bool always_wakeup STG_UNUSED) {};
 #endif
 
-// declared in includes/rts/Threads.h:
+// declared in rts/include/rts/Threads.h:
 // extern Capability MainCapability;
 
-// declared in includes/rts/Threads.h:
+// declared in rts/include/rts/Threads.h:
 // extern uint32_t n_capabilities;
 // extern uint32_t enabled_capabilities;
 
