@@ -6,7 +6,7 @@
  *
  * --------------------------------------------------------------------------*/
 
-#include "PosixSource.h"
+#include "rts/PosixSource.h"
 #include "Rts.h"
 #include "RtsAPI.h"
 #include "HsFFI.h"
@@ -752,7 +752,7 @@ PauseToken *rts_pause (void)
     // Now we own all capabilities so we own rts_pausing_task and may set it.
     rts_pausing_task = task;
 
-    PauseToken *token = malloc(sizeof(PauseToken));
+    PauseToken *token = stgMallocBytes(sizeof(PauseToken), "rts_pause");
     token->capability = task->cap;
     return token;
 }
@@ -772,9 +772,9 @@ void rts_resume (PauseToken *pauseToken)
 
     // releaseAllCapabilities will not block because the current task owns all
     // capabilities.
-    releaseAllCapabilities(getNumCapabilities(), NULL, task);
+    releaseAllCapabilities(n_capabilities, NULL, task);
     exitMyTask();
-    free(pauseToken);
+    stgFree(pauseToken);
 }
 
 // See RtsAPI.h
@@ -810,7 +810,7 @@ static void assert_isPausedOnMyTask(const char *functionName)
     }
 
     // Check that we own all capabilities.
-    for (unsigned int i = 0; i < getNumCapabilities(); i++)
+    for (unsigned int i = 0; i < n_capabilities; i++)
     {
         Capability *cap = capabilities[i];
         if (cap->running_task != task)
