@@ -24,6 +24,7 @@
 #include "Ticky.h"
 #include "StgRun.h"
 #include "Prelude.h"            /* fixupRTStoPreludeRefs */
+#include "Adjustor.h"           /* initAdjustors */
 #include "ThreadLabels.h"
 #include "sm/BlockAlloc.h"
 #include "Trace.h"
@@ -214,7 +215,11 @@ static void initBuiltinGcRoots(void)
     getStablePtr((StgPtr)processRemoteCompletion_closure);
 #endif
 
-    /* See Note [Wired-in exceptions are not CAFfy] in GHC.Core.Make. */
+    /*
+     * See Note [Wired-in exceptions are not CAFfy] in GHC.Core.Make.
+     * These are precisely the functions for which we construct `Id`s using
+     * GHC.Core.Make.mkExceptionId.
+     */
     getStablePtr((StgPtr)absentSumFieldError_closure);
     getStablePtr((StgPtr)raiseUnderflowException_closure);
     getStablePtr((StgPtr)raiseOverflowException_closure);
@@ -321,6 +326,9 @@ hs_init_ghc(int *argc, char **argv[], RtsConfig rts_config)
       initConsoleCP();
 #endif
 
+    /* Initialise the adjustors subsystem */
+    initAdjustors();
+
     /* Initialise the stats department, phase 1 */
     initStats1();
 
@@ -382,6 +390,7 @@ hs_init_ghc(int *argc, char **argv[], RtsConfig rts_config)
 #if defined(PROFILING)
     initProfiling();
 #endif
+    initIpeMapLock();
     traceInitEvent(dumpIPEToEventLog);
     initHeapProfiling();
 
@@ -466,13 +475,12 @@ hs_exit_(bool wait_foreign)
     exitScheduler(wait_foreign);
 
     /* run C finalizers for all active weak pointers */
-    for (i = 0; i < getNumCapabilities(); i++) {
+    for (i = 0; i < n_capabilities; i++) {
         runAllCFinalizers(capabilities[i]->weak_ptr_list_hd);
     }
     for (g = 0; g < RtsFlags.GcFlags.generations; g++) {
         runAllCFinalizers(generations[g].weak_ptr_list);
     }
-    runAllCFinalizers(nonmoving_weak_ptr_list);
 
 #if defined(RTS_USER_SIGNALS)
     if (RtsFlags.MiscFlags.install_signal_handlers) {
@@ -609,6 +617,8 @@ hs_exit_(bool wait_foreign)
 
     // Free threading resources
     freeThreadingResources();
+
+    closeIpeMapLock();
 }
 
 // Flush stdout and stderr.  We do this during shutdown so that it
