@@ -8,7 +8,7 @@
  *
  * -------------------------------------------------------------------------*/
 
-#include "PosixSource.h"
+#include "rts/PosixSource.h"
 #include "Rts.h"
 
 #include "RtsUtils.h"
@@ -388,8 +388,7 @@ discardTasksExcept (Task *keep)
 void
 workerTaskStop (Task *task)
 {
-    DEBUG_ONLY( OSThreadId id );
-    DEBUG_ONLY( id = osThreadId() );
+    OSThreadId id = osThreadId();
     ASSERT(task->id == id);
     ASSERT(myTask() == task);
 
@@ -530,7 +529,7 @@ void rts_setInCallCapability (
 #if defined(THREADED_RTS)
     if (affinity) {
         if (RtsFlags.ParFlags.setAffinity) {
-            setThreadAffinity(preferred_capability, getNumCapabilities());
+            setThreadAffinity(preferred_capability, n_capabilities);
         }
     }
 #endif
@@ -566,8 +565,8 @@ printAllTasks(void)
                 debugBelch("on capability %d, ", task->cap->no);
             }
             if (task->incall->tso) {
-              debugBelch("bound to thread %lu",
-                         (unsigned long)task->incall->tso->id);
+              debugBelch("bound to thread %" FMT_StgThreadID,
+                         task->incall->tso->id);
             } else {
                 debugBelch("worker");
             }
