@@ -29,13 +29,13 @@
 #endif
 
 // events
-int TRACE_sched;
-int TRACE_gc;
-int TRACE_nonmoving_gc;
-int TRACE_spark_sampled;
-int TRACE_spark_full;
-int TRACE_user;
-int TRACE_cap;
+uint8_t TRACE_sched;
+uint8_t TRACE_gc;
+uint8_t TRACE_nonmoving_gc;
+uint8_t TRACE_spark_sampled;
+uint8_t TRACE_spark_full;
+uint8_t TRACE_user;
+uint8_t TRACE_cap;
 
 #if defined(THREADED_RTS)
 static Mutex trace_utx;
@@ -49,12 +49,12 @@ static void traceCap_stderr(Capability *cap, char *msg, ...);
    Starting up / shutting down the tracing facilities
  --------------------------------------------------------------------------- */
 
-void initTracing (void)
+/*
+ * Update the TRACE_* globals. Must be called whenever RtsFlags.TraceFlags is
+ * modified.
+ */
+static void updateTraceFlagCache (void)
 {
-#if defined(THREADED_RTS)
-    initMutex(&trace_utx);
-#endif
-
     // -Ds turns on scheduler tracing too
     TRACE_sched =
         RtsFlags.TraceFlags.scheduler ||
@@ -65,9 +65,6 @@ void initTracing (void)
         RtsFlags.TraceFlags.gc ||
         RtsFlags.DebugFlags.gc ||
         RtsFlags.DebugFlags.scheduler;
-    if (TRACE_gc && RtsFlags.GcFlags.giveStats == NO_GC_STATS) {
-        RtsFlags.GcFlags.giveStats = COLLECT_GC_STATS;
-    }
 
     TRACE_nonmoving_gc =
         RtsFlags.TraceFlags.nonmoving_gc;
@@ -90,6 +87,19 @@ void initTracing (void)
         TRACE_spark_sampled ||
         TRACE_spark_full ||
         TRACE_user;
+}
+
+void initTracing (void)
+{
+#if defined(THREADED_RTS)
+    initMutex(&trace_utx);
+#endif
+
+    updateTraceFlagCache();
+
+    if (TRACE_gc && RtsFlags.GcFlags.giveStats == NO_GC_STATS) {
+        RtsFlags.GcFlags.giveStats = COLLECT_GC_STATS;
+    }
 
     /* Note: we can have any of the TRACE_* flags turned on even when
        eventlog_enabled is off. In the DEBUG way we may be tracing to stderr.
@@ -671,6 +681,17 @@ void traceIPE(StgInfoTable * info,
               const char *module,
               const char *srcloc )
 {
+#if defined(DEBUG)
+    if (RtsFlags.TraceFlags.tracing == TRACE_STDERR) {
+        ACQUIRE_LOCK(&trace_utx);
+
+        tracePreface();
+        debugBelch("IPE: table_name %s, closure_desc %s, ty_desc %s, label %s, module %s, srcloc %s\n",
+                   table_name, closure_desc, ty_desc, label, module, srcloc);
+
+        RELEASE_LOCK(&trace_utx);
+    } else
+#endif
     if (eventlog_enabled) {
         postIPE((W_) INFO_PTR_TO_STRUCT(info), table_name, closure_desc, ty_desc, label, module, srcloc);
     }
