@@ -10,7 +10,7 @@
 #if defined(PROFILING)
 
 #include <string.h>
-#include "PosixSource.h"
+#include "rts/PosixSource.h"
 #include "Rts.h"
 #include "sm/Storage.h"
 
@@ -1036,7 +1036,7 @@ traversePushStack(traverseState *ts, StgClosure *cp, stackElement *sep,
 /**
  * Call traversePushClosure for each of the children of a PAP/AP
  */
-STATIC_INLINE StgPtr
+static StgPtr
 traversePAP (traverseState *ts,
                     StgClosure *pap,    /* NOT tagged */
                     stackElement *sep,
@@ -1095,7 +1095,7 @@ resetMutableObjects(traverseState* ts)
         // Traversing through mut_list is necessary
         // because we can find MUT_VAR objects which have not been
         // visited during heap traversal.
-        for (n = 0; n < getNumCapabilities(); n++) {
+        for (n = 0; n < n_capabilities; n++) {
           for (bd = capabilities[n]->mut_lists[g]; bd != NULL; bd = bd->link) {
             for (ml = bd->start; ml < bd->free; ml++) {
                 traverseMaybeInitClosureData(ts, (StgClosure *)*ml);
