@@ -6,7 +6,7 @@
  *
  * ---------------------------------------------------------------------------*/
 
-#include "PosixSource.h"
+#include "rts/PosixSource.h"
 #include "Rts.h"
 
 #if defined(TRACING)
@@ -95,7 +95,7 @@ static eventlog_init_func_t *eventlog_header_funcs = NULL;
 
 #define EVENT_LOG_SIZE 2 * (1024 * 1024) // 2MB
 
-static int flushCount;
+static int flushCount = 0;
 
 // Struct for record keeping of buffer to store event types and events.
 //
@@ -108,100 +108,21 @@ typedef struct _EventsBuf {
   EventCapNo capno; // which capability this buffer belongs to, or -1
 } EventsBuf;
 
-EventsBuf *capEventBuf; // one EventsBuf for each Capability
+static EventsBuf *capEventBuf; // one EventsBuf for each Capability
 
-EventsBuf eventBuf; // an EventsBuf not associated with any Capability
+static EventsBuf eventBuf; // an EventsBuf not associated with any Capability
 #if defined(THREADED_RTS)
-Mutex eventBufMutex; // protected by this mutex
+static Mutex eventBufMutex; // protected by this mutex
 #endif
 
-char *EventDesc[] = {
-  [EVENT_CREATE_THREAD]       = "Create thread",
-  [EVENT_RUN_THREAD]          = "Run thread",
-  [EVENT_STOP_THREAD]         = "Stop thread",
-  [EVENT_THREAD_RUNNABLE]     = "Thread runnable",
-  [EVENT_MIGRATE_THREAD]      = "Migrate thread",
-  [EVENT_THREAD_WAKEUP]       = "Wakeup thread",
-  [EVENT_THREAD_LABEL]        = "Thread label",
-  [EVENT_CAP_CREATE]          = "Create capability",
-  [EVENT_CAP_DELETE]          = "Delete capability",
-  [EVENT_CAP_DISABLE]         = "Disable capability",
-  [EVENT_CAP_ENABLE]          = "Enable capability",
-  [EVENT_GC_START]            = "Starting GC",
-  [EVENT_GC_END]              = "Finished GC",
-  [EVENT_REQUEST_SEQ_GC]      = "Request sequential GC",
-  [EVENT_REQUEST_PAR_GC]      = "Request parallel GC",
-  [EVENT_GC_GLOBAL_SYNC]      = "Synchronise stop-the-world GC",
-  [EVENT_GC_STATS_GHC]        = "GC statistics",
-  [EVENT_MEM_RETURN]          = "Memory return statistics",
-  [EVENT_HEAP_INFO_GHC]       = "Heap static parameters",
-  [EVENT_HEAP_ALLOCATED]      = "Total heap mem ever allocated",
-  [EVENT_HEAP_SIZE]           = "Current heap size (size of allocated mblocks)",
-  [EVENT_BLOCKS_SIZE]         = "Current heap size (size of allocated blocks)",
-  [EVENT_HEAP_LIVE]           = "Current heap live data",
-  [EVENT_CREATE_SPARK_THREAD] = "Create spark thread",
-  [EVENT_LOG_MSG]             = "Log message",
-  [EVENT_USER_MSG]            = "User message",
-  [EVENT_USER_MARKER]         = "User marker",
-  [EVENT_GC_IDLE]             = "GC idle",
-  [EVENT_GC_WORK]             = "GC working",
-  [EVENT_GC_DONE]             = "GC done",
-  [EVENT_BLOCK_MARKER]        = "Block marker",
-  [EVENT_CAPSET_CREATE]       = "Create capability set",
-  [EVENT_CAPSET_DELETE]       = "Delete capability set",
-  [EVENT_CAPSET_ASSIGN_CAP]   = "Add capability to capability set",
-  [EVENT_CAPSET_REMOVE_CAP]   = "Remove capability from capability set",
-  [EVENT_RTS_IDENTIFIER]      = "RTS name and version",
-  [EVENT_PROGRAM_ARGS]        = "Program arguments",
-  [EVENT_PROGRAM_ENV]         = "Program environment variables",
-  [EVENT_OSPROCESS_PID]       = "Process ID",
-  [EVENT_OSPROCESS_PPID]      = "Parent process ID",
-  [EVENT_WALL_CLOCK_TIME]     = "Wall clock time",
-  [EVENT_SPARK_COUNTERS]      = "Spark counters",
-  [EVENT_SPARK_CREATE]        = "Spark create",
-  [EVENT_SPARK_DUD]           = "Spark dud",
-  [EVENT_SPARK_OVERFLOW]      = "Spark overflow",
-  [EVENT_SPARK_RUN]           = "Spark run",
-  [EVENT_SPARK_STEAL]         = "Spark steal",
-  [EVENT_SPARK_FIZZLE]        = "Spark fizzle",
-  [EVENT_SPARK_GC]            = "Spark GC",
-  [EVENT_TASK_CREATE]         = "Task create",
-  [EVENT_TASK_MIGRATE]        = "Task migrate",
-  [EVENT_TASK_DELETE]         = "Task delete",
-  [EVENT_HACK_BUG_T9003]      = "Empty event for bug #9003",
-  [EVENT_HEAP_PROF_BEGIN]     = "Start of heap profile",
-  [EVENT_HEAP_PROF_COST_CENTRE]   = "Cost center definition",
-  [EVENT_IPE]                     = "Info Table Source Position",
-  [EVENT_HEAP_PROF_SAMPLE_BEGIN]  = "Start of heap profile sample",
-  [EVENT_HEAP_BIO_PROF_SAMPLE_BEGIN]  = "Start of heap profile (biographical) sample",
-  [EVENT_HEAP_PROF_SAMPLE_END]    = "End of heap profile sample",
-  [EVENT_HEAP_PROF_SAMPLE_STRING] = "Heap profile string sample",
-  [EVENT_HEAP_PROF_SAMPLE_COST_CENTRE] = "Heap profile cost-centre sample",
-  [EVENT_PROF_SAMPLE_COST_CENTRE] = "Time profile cost-centre stack",
-  [EVENT_PROF_BEGIN] = "Start of a time profile",
-  [EVENT_USER_BINARY_MSG]     = "User binary message",
-  [EVENT_CONC_MARK_BEGIN]        = "Begin concurrent mark phase",
-  [EVENT_CONC_MARK_END]          = "End concurrent mark phase",
-  [EVENT_CONC_SYNC_BEGIN]        = "Begin concurrent GC synchronisation",
-  [EVENT_CONC_SYNC_END]          = "End concurrent GC synchronisation",
-  [EVENT_CONC_SWEEP_BEGIN]       = "Begin concurrent sweep",
-  [EVENT_CONC_SWEEP_END]         = "End concurrent sweep",
-  [EVENT_CONC_UPD_REM_SET_FLUSH] = "Update remembered set flushed",
-  [EVENT_NONMOVING_HEAP_CENSUS]  = "Nonmoving heap census",
-  [EVENT_TICKY_COUNTER_DEF]    = "Ticky-ticky entry counter definition",
-  [EVENT_TICKY_COUNTER_BEGIN_SAMPLE] = "Ticky-ticky entry counter begin sample",
-  [EVENT_TICKY_COUNTER_SAMPLE] = "Ticky-ticky entry counter sample",
-};
-
-// Event type.
-
+// Event type
 typedef struct _EventType {
   EventTypeNum etNum;  // Event Type number.
   uint32_t   size;     // size of the payload in bytes
   char *desc;     // Description
 } EventType;
 
-EventType eventTypes[NUM_GHC_EVENT_TAGS];
+#include "rts/EventTypes.h"
 
 static void initEventsBuf(EventsBuf* eb, StgWord64 size, EventCapNo capno);
 static void resetEventsBuf(EventsBuf* eb);
@@ -352,241 +273,6 @@ flushEventLogWriter(void)
     }
 }
 
-static void
-init_event_types(void)
-{
-    for (int t = 0; t < NUM_GHC_EVENT_TAGS; ++t) {
-        eventTypes[t].etNum = t;
-        eventTypes[t].desc = EventDesc[t];
-
-        switch (t) {
-        case EVENT_CREATE_THREAD:   // (cap, thread)
-        case EVENT_RUN_THREAD:      // (cap, thread)
-        case EVENT_THREAD_RUNNABLE: // (cap, thread)
-        case EVENT_CREATE_SPARK_THREAD: // (cap, spark_thread)
-            eventTypes[t].size = sizeof(EventThreadID);
-            break;
-
-        case EVENT_MIGRATE_THREAD:  // (cap, thread, new_cap)
-        case EVENT_THREAD_WAKEUP:   // (cap, thread, other_cap)
-            eventTypes[t].size =
-                sizeof(EventThreadID) + sizeof(EventCapNo);
-            break;
-
-        case EVENT_STOP_THREAD:     // (cap, thread, status)
-            eventTypes[t].size = sizeof(EventThreadID)
-                               + sizeof(StgWord16)
-                               + sizeof(EventThreadID);
-            break;
-
-        case EVENT_CAP_CREATE:      // (cap)
-        case EVENT_CAP_DELETE:      // (cap)
-        case EVENT_CAP_ENABLE:      // (cap)
-        case EVENT_CAP_DISABLE:     // (cap)
-            eventTypes[t].size = sizeof(EventCapNo);
-            break;
-
-        case EVENT_CAPSET_CREATE:   // (capset, capset_type)
-            eventTypes[t].size =
-                sizeof(EventCapsetID) + sizeof(EventCapsetType);
-            break;
-
-        case EVENT_CAPSET_DELETE:   // (capset)
-            eventTypes[t].size = sizeof(EventCapsetID);
-            break;
-
-        case EVENT_CAPSET_ASSIGN_CAP:  // (capset, cap)
-        case EVENT_CAPSET_REMOVE_CAP:
-            eventTypes[t].size =
-                sizeof(EventCapsetID) + sizeof(EventCapNo);
-            break;
-
-        case EVENT_OSPROCESS_PID:   // (cap, pid)
-        case EVENT_OSPROCESS_PPID:
-            eventTypes[t].size =
-                sizeof(EventCapsetID) + sizeof(StgWord32);
-            break;
-
-        case EVENT_WALL_CLOCK_TIME: // (capset, unix_epoch_seconds, nanoseconds)
-            eventTypes[t].size =
-                sizeof(EventCapsetID) + sizeof(StgWord64) + sizeof(StgWord32);
-            break;
-
-        case EVENT_SPARK_STEAL:     // (cap, victim_cap)
-            eventTypes[t].size =
-                sizeof(EventCapNo);
-            break;
-
-        case EVENT_REQUEST_SEQ_GC:  // (cap)
-        case EVENT_REQUEST_PAR_GC:  // (cap)
-        case EVENT_GC_START:        // (cap)
-        case EVENT_GC_END:          // (cap)
-        case EVENT_GC_IDLE:
-        case EVENT_GC_WORK:
-        case EVENT_GC_DONE:
-        case EVENT_GC_GLOBAL_SYNC:  // (cap)
-        case EVENT_SPARK_CREATE:    // (cap)
-        case EVENT_SPARK_DUD:       // (cap)
-        case EVENT_SPARK_OVERFLOW:  // (cap)
-        case EVENT_SPARK_RUN:       // (cap)
-        case EVENT_SPARK_FIZZLE:    // (cap)
-        case EVENT_SPARK_GC:        // (cap)
-            eventTypes[t].size = 0;
-            break;
-
-        case EVENT_LOG_MSG:          // (msg)
-        case EVENT_USER_MSG:         // (msg)
-        case EVENT_USER_MARKER:      // (markername)
-        case EVENT_RTS_IDENTIFIER:   // (capset, str)
-        case EVENT_PROGRAM_ARGS:     // (capset, strvec)
-        case EVENT_PROGRAM_ENV:      // (capset, strvec)
-        case EVENT_THREAD_LABEL:     // (thread, str)
-            eventTypes[t].size = 0xffff;
-            break;
-
-        case EVENT_SPARK_COUNTERS:   // (cap, 7*counter)
-            eventTypes[t].size = 7 * sizeof(StgWord64);
-            break;
-
-        case EVENT_HEAP_ALLOCATED:    // (heap_capset, alloc_bytes)
-        case EVENT_HEAP_SIZE:         // (heap_capset, size_bytes)
-        case EVENT_BLOCKS_SIZE:       // (heap_capset, size_bytes)
-        case EVENT_HEAP_LIVE:         // (heap_capset, live_bytes)
-            eventTypes[t].size = sizeof(EventCapsetID) + sizeof(StgWord64);
-            break;
-
-        case EVENT_HEAP_INFO_GHC:     // (heap_capset, n_generations,
-                                      //  max_heap_size, alloc_area_size,
-                                      //  mblock_size, block_size)
-            eventTypes[t].size = sizeof(EventCapsetID)
-                               + sizeof(StgWord16)
-                               + sizeof(StgWord64) * 4;
-            break;
-
-        case EVENT_GC_STATS_GHC:      // (heap_capset, generation,
-                                      //  copied_bytes, slop_bytes, frag_bytes,
-                                      //  par_n_threads,
-                                      //  par_max_copied, par_tot_copied,
-                                      //  par_balanced_copied
-                                      //  )
-            eventTypes[t].size = sizeof(EventCapsetID)
-                               + sizeof(StgWord16)
-                               + sizeof(StgWord64) * 3
-                               + sizeof(StgWord32)
-                               + sizeof(StgWord64) * 3;
-            break;
-        case EVENT_MEM_RETURN:        // (heap_capset, current_mblocks
-                                      // , needed_mblocks, returned_mblocks)
-            eventTypes[t].size = sizeof(EventCapsetID)
-                               + sizeof(StgWord32) * 3;
-            break;
-
-        case EVENT_TASK_CREATE:   // (taskId, cap, tid)
-            eventTypes[t].size = sizeof(EventTaskId)
-                               + sizeof(EventCapNo)
-                               + sizeof(EventKernelThreadId);
-            break;
-
-        case EVENT_TASK_MIGRATE:   // (taskId, cap, new_cap)
-            eventTypes[t].size =
-                sizeof(EventTaskId) + sizeof(EventCapNo) + sizeof(EventCapNo);
-            break;
-
-        case EVENT_TASK_DELETE:   // (taskId)
-            eventTypes[t].size = sizeof(EventTaskId);
-            break;
-
-        case EVENT_BLOCK_MARKER:
-            eventTypes[t].size = sizeof(StgWord32) + sizeof(EventTimestamp) +
-                sizeof(EventCapNo);
-            break;
-
-        case EVENT_HACK_BUG_T9003:
-            eventTypes[t].size = 0;
-            break;
-
-        case EVENT_HEAP_PROF_BEGIN:
-            eventTypes[t].size = EVENT_SIZE_DYNAMIC;
-            break;
-
-        case EVENT_HEAP_PROF_COST_CENTRE:
-            eventTypes[t].size = EVENT_SIZE_DYNAMIC;
-            break;
-        case EVENT_IPE:
-            eventTypes[t].size = EVENT_SIZE_DYNAMIC;
-            break;
-
-        case EVENT_HEAP_PROF_SAMPLE_BEGIN:
-            eventTypes[t].size = 8;
-            break;
-
-        case EVENT_HEAP_BIO_PROF_SAMPLE_BEGIN:
-            eventTypes[t].size = 16;
-            break;
-
-        case EVENT_HEAP_PROF_SAMPLE_END:
-            eventTypes[t].size = 8;
-            break;
-
-        case EVENT_HEAP_PROF_SAMPLE_STRING:
-            eventTypes[t].size = EVENT_SIZE_DYNAMIC;
-            break;
-
-        case EVENT_HEAP_PROF_SAMPLE_COST_CENTRE:
-            eventTypes[t].size = EVENT_SIZE_DYNAMIC;
-            break;
-
-        case EVENT_PROF_SAMPLE_COST_CENTRE:
-            eventTypes[t].size = EVENT_SIZE_DYNAMIC;
-            break;
-
-        case EVENT_PROF_BEGIN:
-            eventTypes[t].size = 8;
-            break;
-
-        case EVENT_USER_BINARY_MSG:
-            eventTypes[t].size = EVENT_SIZE_DYNAMIC;
-            break;
-
-        case EVENT_CONC_MARK_BEGIN:
-        case EVENT_CONC_SYNC_BEGIN:
-        case EVENT_CONC_SYNC_END:
-        case EVENT_CONC_SWEEP_BEGIN:
-        case EVENT_CONC_SWEEP_END:
-            eventTypes[t].size = 0;
-            break;
-
-        case EVENT_CONC_MARK_END:
-            eventTypes[t].size = 4;
-            break;
-
-        case EVENT_CONC_UPD_REM_SET_FLUSH: // (cap)
-            eventTypes[t].size =
-                sizeof(EventCapNo);
-            break;
-
-        case EVENT_NONMOVING_HEAP_CENSUS: // (cap, blk_size, active_segs, filled_segs, live_blks)
-            eventTypes[t].size = 13;
-            break;
-
-        case EVENT_TICKY_COUNTER_DEF: // (counter_id, arity, arg_kinds, name)
-            eventTypes[t].size = EVENT_SIZE_DYNAMIC;
-            break;
-
-        case EVENT_TICKY_COUNTER_BEGIN_SAMPLE:
-            eventTypes[t].size = 0;
-            break;
-
-        case EVENT_TICKY_COUNTER_SAMPLE: // (counter_id, entry_count, allocs, allocd)
-            eventTypes[t].size = 8*4;
-            break;
-
-        default:
-            continue; /* ignore deprecated events */
-        }
-    }
-}
-
 static void
 postHeaderEvents(void)
 {
@@ -665,8 +351,7 @@ get_n_capabilities(void)
 {
 #if defined(THREADED_RTS)
     // XXX n_capabilities may not have been initialized yet
-    unsigned int n = getNumCapabilities();
-    return (n != 0) ? n : RtsFlags.ParFlags.nCapabilities;
+    return (n_capabilities != 0) ? n_capabilities : RtsFlags.ParFlags.nCapabilities;
 #else
     return 1;
 #endif
@@ -675,14 +360,6 @@ get_n_capabilities(void)
 void
 initEventLogging()
 {
-    init_event_types();
-
-    int num_descs = sizeof(EventDesc) / sizeof(char*);
-    if (num_descs != NUM_GHC_EVENT_TAGS) {
-        barf("EventDesc array has the wrong number of elements (%d, NUM_GHC_EVENT_TAGS=%d)",
-             num_descs, NUM_GHC_EVENT_TAGS);
-    }
-
     /*
      * Allocate buffer(s) to store events.
      * Create buffer large enough for the header begin marker, all event
@@ -773,10 +450,11 @@ finishCapEventLogging(void)
     if (eventlog_enabled) {
         // Flush all events remaining in the capabilities' buffers and free them.
         // N.B. at this point we hold all capabilities.
-        for (uint32_t c = 0; c < getNumCapabilities(); ++c) {
+        for (uint32_t c = 0; c < n_capabilities; ++c) {
             if (capEventBuf[c].begin != NULL) {
                 printAndClearEventBuf(&capEventBuf[c]);
                 stgFree(capEventBuf[c].begin);
+                capEventBuf[c].begin = NULL;
             }
         }
     }
@@ -848,6 +526,7 @@ freeEventLoggingBuffer(void)
 {
     if (capEventBuf != NULL)  {
         stgFree(capEventBuf);
+        capEventBuf = NULL;
     }
 }
 
@@ -1075,16 +754,7 @@ void postCapsetVecEvent (EventTypeNum tag,
 
     for (int i = 0; i < argc; i++) {
         // 1 + strlen to account for the trailing \0, used as separator
-        int increment = 1 + strlen(argv[i]);
-        if (size + increment > EVENT_PAYLOAD_SIZE_MAX) {
-            errorBelch("Event size exceeds EVENT_PAYLOAD_SIZE_MAX, record only %d out of %d args",
-                       i,
-                       argc);
-            argc = i;
-            break;
-        } else {
-            size += increment;
-        }
+        size += 1 + strlen(argv[i]);
     }
 
     ACQUIRE_LOCK(&eventBufMutex);
@@ -1687,7 +1357,7 @@ void postProfBegin(void)
 #if defined(TICKY_TICKY)
 static void postTickyCounterDef(EventsBuf *eb, StgEntCounter *p)
 {
-    StgWord len = 8 + 2 + strlen(p->arg_kinds)+1 + strlen(p->str)+1;
+    StgWord len = 8 + 2 + strlen(p->arg_kinds)+1 + strlen(p->str)+1 + 8 + strlen(p->ticky_json)+1;
     ensureRoomForVariableEvent(eb, len);
     postEventHeader(eb, EVENT_TICKY_COUNTER_DEF);
     postPayloadSize(eb, len);
@@ -1696,6 +1366,9 @@ static void postTickyCounterDef(EventsBuf *eb, StgEntCounter *p)
     postWord16(eb, (uint16_t) p->arity);
     postString(eb, p->arg_kinds);
     postString(eb, p->str);
+    postWord64(eb, (W_) (INFO_PTR_TO_STRUCT(p->info)));
+    postString(eb, p->ticky_json);
+
 }
 
 void postTickyCounterDefs(StgEntCounter *counters)
@@ -1873,11 +1546,15 @@ void flushLocalEventsBuf(Capability *cap)
 // Used during forkProcess.
 void flushAllCapsEventsBufs()
 {
+    if (!event_log_writer) {
+        return;
+    }
+
     ACQUIRE_LOCK(&eventBufMutex);
     printAndClearEventBuf(&eventBuf);
     RELEASE_LOCK(&eventBufMutex);
 
-    for (unsigned int i=0; i < getNumCapabilities(); i++) {
+    for (unsigned int i=0; i < n_capabilities; i++) {
         flushLocalEventsBuf(capabilities[i]);
     }
     flushEventLogWriter();
@@ -1885,6 +1562,10 @@ void flushAllCapsEventsBufs()
 
 void flushEventLog(Capability **cap USED_IF_THREADS)
 {
+    if (!event_log_writer) {
+        return;
+    }
+
     ACQUIRE_LOCK(&eventBufMutex);
     printAndClearEventBuf(&eventBuf);
     RELEASE_LOCK(&eventBufMutex);
@@ -1893,7 +1574,7 @@ void flushEventLog(Capability **cap USED_IF_THREADS)
     Task *task = getMyTask();
     stopAllCapabilitiesWith(cap, task, SYNC_FLUSH_EVENT_LOG);
     flushAllCapsEventsBufs();
-    releaseAllCapabilities(getNumCapabilities(), cap ? *cap : NULL, task);
+    releaseAllCapabilities(n_capabilities, cap ? *cap : NULL, task);
 #else
     flushLocalEventsBuf(capabilities[0]);
 #endif
