@@ -241,7 +241,7 @@ resolveImports(
             addr = (SymbolAddr*) (symbol->nlist->n_value);
             IF_DEBUG(linker, debugBelch("resolveImports: undefined external %s has value %p\n", symbol->name, addr));
         } else {
-            addr = lookupDependentSymbol(symbol->name, oc);
+            addr = lookupDependentSymbol(symbol->name, oc, NULL);
             IF_DEBUG(linker, debugBelch("resolveImports: looking up %s, %p\n", symbol->name, addr));
         }
 
@@ -506,7 +506,7 @@ relocateSectionAarch64(ObjectCode * oc, Section * section)
                      * or asking the system, if not found
                      * in the symbol hashmap
                      */
-                    value = (uint64_t)lookupDependentSymbol((char*)symbol->name, oc);
+                    value = (uint64_t)lookupDependentSymbol((char*)symbol->name, oc, NULL);
                     if(!value)
                         barf("Could not lookup symbol: %s!", symbol->name);
                 } else {
@@ -546,7 +546,7 @@ relocateSectionAarch64(ObjectCode * oc, Section * section)
                 uint64_t pc = (uint64_t)section->start + ri->r_address;
                 uint64_t value = 0;
                 if(symbol->nlist->n_type & N_EXT) {
-                    value = (uint64_t)lookupDependentSymbol((char*)symbol->name, oc);
+                    value = (uint64_t)lookupDependentSymbol((char*)symbol->name, oc, NULL);
                     if(!value)
                         barf("Could not lookup symbol: %s!", symbol->name);
                 } else {
@@ -664,14 +664,14 @@ relocateSection(ObjectCode* oc, int curSection)
         int relocLenBytes;
         int nextInstrAdj = 0;
 
-        IF_DEBUG(linker, debugBelch("relocateSection: relocation %d\n", i));
-        IF_DEBUG(linker, debugBelch("               : type      = %d\n", reloc->r_type));
-        IF_DEBUG(linker, debugBelch("               : address   = %d\n", reloc->r_address));
-        IF_DEBUG(linker, debugBelch("               : symbolnum = %u\n", reloc->r_symbolnum));
-        IF_DEBUG(linker, debugBelch("               : pcrel     = %d\n", reloc->r_pcrel));
-        IF_DEBUG(linker, debugBelch("               : length    = %d\n", reloc->r_length));
-        IF_DEBUG(linker, debugBelch("               : extern    = %d\n", reloc->r_extern));
-        IF_DEBUG(linker, debugBelch("               : type      = %d\n", reloc->r_type));
+        IF_DEBUG(linker_verbose, debugBelch("relocateSection: relocation %d\n", i));
+        IF_DEBUG(linker_verbose, debugBelch("               : type      = %d\n", reloc->r_type));
+        IF_DEBUG(linker_verbose, debugBelch("               : address   = %d\n", reloc->r_address));
+        IF_DEBUG(linker_verbose, debugBelch("               : symbolnum = %u\n", reloc->r_symbolnum));
+        IF_DEBUG(linker_verbose, debugBelch("               : pcrel     = %d\n", reloc->r_pcrel));
+        IF_DEBUG(linker_verbose, debugBelch("               : length    = %d\n", reloc->r_length));
+        IF_DEBUG(linker_verbose, debugBelch("               : extern    = %d\n", reloc->r_extern));
+        IF_DEBUG(linker_verbose, debugBelch("               : type      = %d\n", reloc->r_type));
 
         switch(reloc->r_length)
         {
@@ -715,7 +715,7 @@ relocateSection(ObjectCode* oc, int curSection)
 
 
 
-        IF_DEBUG(linker,
+        IF_DEBUG(linker_verbose,
                  debugBelch("relocateSection: length = %d, thing = %" PRId64 ", baseValue = %p\n",
                             reloc->r_length, thing, (char *)baseValue));
 
@@ -726,7 +726,7 @@ relocateSection(ObjectCode* oc, int curSection)
             SymbolName* nm = symbol->name;
             SymbolAddr* addr = NULL;
 
-            IF_DEBUG(linker, debugBelch("relocateSection: making jump island for %s, extern = %d, X86_64_RELOC_GOT\n",
+            IF_DEBUG(linker_verbose, debugBelch("relocateSection: making jump island for %s, extern = %d, X86_64_RELOC_GOT\n",
                                         nm, reloc->r_extern));
 
             if (reloc->r_extern == 0) {
@@ -739,10 +739,11 @@ relocateSection(ObjectCode* oc, int curSection)
                     // symtab, or it is undefined, meaning dlsym must be used
                     // to resolve it.
 
-                    addr = lookupDependentSymbol(nm, oc);
-                    IF_DEBUG(linker, debugBelch("relocateSection: looked up %s, "
-                                                "external X86_64_RELOC_GOT or X86_64_RELOC_GOT_LOAD\n"
-                                                "               : addr = %p\n", nm, addr));
+                    addr = lookupDependentSymbol(nm, oc, NULL);
+                    IF_DEBUG(linker_verbose,
+                             debugBelch("relocateSection: looked up %s, "
+                                        "external X86_64_RELOC_GOT or X86_64_RELOC_GOT_LOAD\n"
+                                        "               : addr = %p\n", nm, addr));
 
                     if (addr == NULL) {
                             errorBelch("\nlookupSymbol failed in relocateSection (RELOC_GOT)\n"
@@ -750,7 +751,7 @@ relocateSection(ObjectCode* oc, int curSection)
                             return 0;
                     }
             } else {
-                    IF_DEBUG(linker, debugBelch("relocateSection: %s is not an exported symbol\n", nm));
+                    IF_DEBUG(linker_verbose, debugBelch("relocateSection: %s is not an exported symbol\n", nm));
 
                     // The symbol is not exported, or defined in another
                     // module, so it must be in the current object module,
@@ -765,9 +766,11 @@ relocateSection(ObjectCode* oc, int curSection)
 
                         addr = symbol->addr;
 
-                        IF_DEBUG(linker, debugBelch("relocateSection: calculated relocation of "
-                                                    "non-external X86_64_RELOC_GOT or X86_64_RELOC_GOT_LOAD\n"));
-                        IF_DEBUG(linker, debugBelch("               : addr = %p\n", addr));
+                        IF_DEBUG(linker_verbose,
+                                 debugBelch("relocateSection: calculated relocation of "
+                                            "non-external X86_64_RELOC_GOT or X86_64_RELOC_GOT_LOAD\n"));
+                        IF_DEBUG(linker_verbose,
+                                 debugBelch("               : addr = %p\n", addr));
                     } else {
                         errorBelch("\nrelocateSection: %s is not exported,"
                                    " and should be defined in a section, but isn't!\n", nm);
@@ -787,20 +790,21 @@ relocateSection(ObjectCode* oc, int curSection)
             SymbolName* nm = symbol->name;
             SymbolAddr* addr = NULL;
 
-            IF_DEBUG(linker, debugBelch("relocateSection: looking up external symbol %s\n", nm));
-            IF_DEBUG(linker, debugBelch("               : type  = %d\n", symbol->nlist->n_type));
-            IF_DEBUG(linker, debugBelch("               : sect  = %d\n", symbol->nlist->n_sect));
-            IF_DEBUG(linker, debugBelch("               : desc  = %d\n", symbol->nlist->n_desc));
-            IF_DEBUG(linker, debugBelch("               : value = %p\n", (void *)symbol->nlist->n_value));
+            IF_DEBUG(linker_verbose, debugBelch("relocateSection: looking up external symbol %s\n", nm));
+            IF_DEBUG(linker_verbose, debugBelch("               : type  = %d\n", symbol->nlist->n_type));
+            IF_DEBUG(linker_verbose, debugBelch("               : sect  = %d\n", symbol->nlist->n_sect));
+            IF_DEBUG(linker_verbose, debugBelch("               : desc  = %d\n", symbol->nlist->n_desc));
+            IF_DEBUG(linker_verbose, debugBelch("               : value = %p\n", (void *)symbol->nlist->n_value));
 
             if ((symbol->nlist->n_type & N_TYPE) == N_SECT) {
                 CHECK(symbol->addr != NULL);
                 value = (uint64_t) symbol->addr;
-                IF_DEBUG(linker, debugBelch("relocateSection, defined external symbol %s, relocated address %p\n",
-                                            nm, (void *)value));
+                IF_DEBUG(linker_verbose,
+                         debugBelch("relocateSection, defined external symbol %s, relocated address %p\n",
+                                    nm, (void *)value));
             }
             else {
-                addr = lookupDependentSymbol(nm, oc);
+                addr = lookupDependentSymbol(nm, oc, NULL);
                 if (addr == NULL)
                 {
                      errorBelch("\nlookupSymbol failed in relocateSection (relocate external)\n"
@@ -809,7 +813,9 @@ relocateSection(ObjectCode* oc, int curSection)
                 }
 
                 value = (uint64_t) addr;
-                IF_DEBUG(linker, debugBelch("relocateSection: external symbol %s, address %p\n", nm, (void *)value));
+                IF_DEBUG(linker_verbose,
+                         debugBelch("relocateSection: external symbol %s, address %p\n",
+                                    nm, (void *)value));
             }
         }
         else
@@ -833,7 +839,7 @@ relocateSection(ObjectCode* oc, int curSection)
             Section * targetSec = &oc->sections[targetSecNum];
             MachOSection * targetMacho = targetSec->info->macho_section;
 
-            IF_DEBUG(linker,
+            IF_DEBUG(linker_verbose,
                      debugBelch("relocateSection: internal relocation relative to section %d (%s, %s)\n",
                                 targetSecNum, targetMacho->segname, targetMacho->sectname));
 
@@ -844,13 +850,15 @@ relocateSection(ObjectCode* oc, int curSection)
                        thing, (uint64_t) targetMacho->addr);
 
                 uint64_t thingRelativeOffset = thing - targetMacho->addr;
-                IF_DEBUG(linker, debugBelch("                 "
-                                            "unsigned displacement %" PRIx64 " with section relative offset %" PRIx64 "\n",
+                IF_DEBUG(linker_verbose,
+                         debugBelch("                 "
+                                    "unsigned displacement %" PRIx64 " with section relative offset %" PRIx64 "\n",
                                             thing, thingRelativeOffset));
 
                 thing = (uint64_t) targetSec->start + thingRelativeOffset;
-                IF_DEBUG(linker, debugBelch("                 "
-                                            "relocated address is %p\n", (void *) thing));
+                IF_DEBUG(linker_verbose,
+                         debugBelch("                 "
+                                    "relocated address is %p\n", (void *) thing));
 
                 /* Compared to external relocation we don't need to adjust value
                  * any further since thing already has absolute address.
@@ -870,7 +878,7 @@ relocateSection(ObjectCode* oc, int curSection)
                        (void *) imThingLoc, (void *) targetMacho->addr);
 
                 int64_t thingRelativeOffset = imThingLoc - targetMacho->addr;
-                IF_DEBUG(linker,
+                IF_DEBUG(linker_verbose,
                      debugBelch("                 "
                                 "original displacement %" PRId64 " to %p with section relative offset %" PRIu64 "\n",
                                 thing, (void *) imThingLoc, thingRelativeOffset));
@@ -878,7 +886,7 @@ relocateSection(ObjectCode* oc, int curSection)
                 thing = (int64_t) ((uint64_t) targetSec->start + thingRelativeOffset)
                                 - ((uint64_t) sect->start + baseValueOffset);
                 value = baseValue; // so that it further cancels out with baseValue
-                IF_DEBUG(linker,
+                IF_DEBUG(linker_verbose,
                          debugBelch("                 "
                                     "relocated displacement %" PRId64 " to %p\n",
                                     (int64_t) thing, (void *) (baseValue + thing)));
@@ -890,7 +898,7 @@ relocateSection(ObjectCode* oc, int curSection)
             }
         }
 
-        IF_DEBUG(linker, debugBelch("relocateSection: value = %p\n", (void *) value));
+        IF_DEBUG(linker_verbose, debugBelch("relocateSection: value = %p\n", (void *) value));
 
         if (type == X86_64_RELOC_BRANCH)
         {
@@ -925,7 +933,7 @@ relocateSection(ObjectCode* oc, int curSection)
                 barf("unknown relocation");
         }
 
-        IF_DEBUG(linker, debugBelch("relocateSection: thing = %p\n", (void *) thing));
+        IF_DEBUG(linker_verbose, debugBelch("relocateSection: thing = %p\n", (void *) thing));
 
         /* Thing points to memory within one of the relocated sections. We can
          * probe the first byte to sanity check internal relocations.
@@ -963,22 +971,18 @@ relocateSection(ObjectCode* oc, int curSection)
 SectionKind
 getSectionKind_MachO(MachOSection *section)
 {
-    SectionKind kind;
-
-    /* todo: Use section flags instead */
-    if (0==strcmp(section->sectname,"__text")) {
-        kind = SECTIONKIND_CODE_OR_RODATA;
-    } else if (0==strcmp(section->sectname,"__const") ||
-               0==strcmp(section->sectname,"__data") ||
-               0==strcmp(section->sectname,"__bss") ||
-               0==strcmp(section->sectname,"__common") ||
-               0==strcmp(section->sectname,"__mod_init_func")) {
-        kind = SECTIONKIND_RWDATA;
+    uint8_t s_type = section->flags & SECTION_TYPE;
+    if (s_type == S_MOD_INIT_FUNC_POINTERS) {
+        return SECTIONKIND_INIT_ARRAY;
+    } else if (s_type == S_MOD_TERM_FUNC_POINTERS) {
+        return SECTIONKIND_FINI_ARRAY;
+    } else if (0==strcmp(section->segname,"__TEXT")) {
+        return SECTIONKIND_CODE_OR_RODATA;
+    } else if (0==strcmp(section->segname,"__DATA")) {
+        return SECTIONKIND_RWDATA;
     } else {
-        kind = SECTIONKIND_OTHER;
+        return SECTIONKIND_OTHER;
     }
-
-    return kind;
 }
 
 /* Calculate the # of active segments and their sizes based on section
@@ -1327,42 +1331,45 @@ ocGetNames_MachO(ObjectCode* oc)
             SymbolName* nm = oc->info->macho_symbols[i].name;
             if (oc->info->nlist[i].n_type & N_STAB)
             {
-                IF_DEBUG(linker, debugBelch("ocGetNames_MachO: Skip STAB: %s\n", nm));
+                IF_DEBUG(linker_verbose, debugBelch("ocGetNames_MachO: Skip STAB: %s\n", nm));
             }
             else if ((oc->info->nlist[i].n_type & N_TYPE) == N_SECT)
             {
                 if (oc->info->nlist[i].n_type & N_EXT)
                 {
                     if (   (oc->info->nlist[i].n_desc & N_WEAK_DEF)
-                        && lookupDependentSymbol(nm, oc)) {
+                        && lookupDependentSymbol(nm, oc, NULL)) {
                         // weak definition, and we already have a definition
-                        IF_DEBUG(linker, debugBelch("    weak: %s\n", nm));
+                        IF_DEBUG(linker_verbose, debugBelch("    weak: %s\n", nm));
                     }
                     else
                     {
-                            IF_DEBUG(linker, debugBelch("ocGetNames_MachO: inserting %s\n", nm));
+                            IF_DEBUG(linker_verbose, debugBelch("ocGetNames_MachO: inserting %s\n", nm));
                             SymbolAddr* addr = oc->info->macho_symbols[i].addr;
-
+                            // TODO: Make figure out how to determine this from the object file
+                            SymType sym_type = SYM_TYPE_CODE;
                             ghciInsertSymbolTable( oc->fileName
                                                  , symhash
                                                  , nm
                                                  , addr
                                                  , HS_BOOL_FALSE
+                                                 , sym_type
                                                  , oc);
 
                             oc->symbols[curSymbol].name = nm;
                             oc->symbols[curSymbol].addr = addr;
+                            oc->symbols[curSymbol].type = sym_type;
                             curSymbol++;
                     }
                 }
                 else
                 {
-                    IF_DEBUG(linker, debugBelch("ocGetNames_MachO: \t...not external, skipping %s\n", nm));
+                    IF_DEBUG(linker_verbose, debugBelch("ocGetNames_MachO: \t...not external, skipping %s\n", nm));
                 }
             }
             else
             {
-                IF_DEBUG(linker, debugBelch("ocGetNames_MachO: \t...not defined in this section, skipping %s\n", nm));
+                IF_DEBUG(linker_verbose, debugBelch("ocGetNames_MachO: \t...not defined in this section, skipping %s\n", nm));
             }
         }
     }
@@ -1384,10 +1391,12 @@ ocGetNames_MachO(ObjectCode* oc)
 
                 /* also set the final address to the macho_symbol */
                 oc->info->macho_symbols[i].addr = (void*)commonCounter;
+                /* TODO: Figure out how to determine this from object */
+                SymType sym_type = SYM_TYPE_CODE;
 
-                IF_DEBUG(linker, debugBelch("ocGetNames_MachO: inserting common symbol: %s\n", nm));
+                IF_DEBUG(linker_verbose, debugBelch("ocGetNames_MachO: inserting common symbol: %s\n", nm));
                 ghciInsertSymbolTable(oc->fileName, symhash, nm,
-                                       (void*)commonCounter, HS_BOOL_FALSE, oc);
+                                       (void*)commonCounter, HS_BOOL_FALSE, sym_type, oc);
                 oc->symbols[curSymbol].name = nm;
                 oc->symbols[curSymbol].addr = oc->info->macho_symbols[i].addr;
                 curSymbol++;
@@ -1509,7 +1518,7 @@ ocResolve_MachO(ObjectCode* oc)
                  * have the address.
                  */
                 if(NULL == symbol->addr) {
-                    symbol->addr = lookupDependentSymbol((char*)symbol->name, oc);
+                    symbol->addr = lookupDependentSymbol((char*)symbol->name, oc, NULL);
                     if(NULL == symbol->addr) {
                         errorBelch("Failed to lookup symbol: %s", symbol->name);
                         return 0;
@@ -1568,12 +1577,7 @@ ocRunInit_MachO ( ObjectCode *oc )
     for (int i = 0; i < oc->n_sections; i++) {
         IF_DEBUG(linker, debugBelch("ocRunInit_MachO: checking section %d\n", i));
 
-        // ToDo: replace this with a proper check for the S_MOD_INIT_FUNC_POINTERS
-        // flag.  We should do this elsewhere in the Mach-O linker code
-        // too.  Note that the system linker will *refuse* to honor
-        // sections which don't have this flag, so this could cause
-        // weird behavior divergence (albeit reproducible).
-        if (0 == strcmp(oc->info->macho_sections[i].sectname, "__mod_init_func")) {
+        if (oc->sections[i].kind == SECTIONKIND_INIT_ARRAY) {
             IF_DEBUG(linker, debugBelch("ocRunInit_MachO:     running mod init functions\n"));
 
             void *init_startC = oc->sections[i].start;
@@ -1593,6 +1597,35 @@ ocRunInit_MachO ( ObjectCode *oc )
     return 1;
 }
 
+int
+ocRunFini_MachO ( ObjectCode *oc )
+{
+    if (NULL == oc->info->segCmd) {
+        barf("ocRunInit_MachO: no segment load command");
+    }
+
+    for (int i = 0; i < oc->n_sections; i++) {
+        IF_DEBUG(linker, debugBelch("ocRunFini_MachO: checking section %d\n", i));
+
+        if (oc->sections[i].kind == SECTIONKIND_FINI_ARRAY) {
+            IF_DEBUG(linker, debugBelch("ocRunFini_MachO:     running mod fini functions\n"));
+
+            void *fini_startC = oc->sections[i].start;
+            fini_t *fini = (fini_t*)fini_startC;
+            fini_t *fini_end = (fini_t*)((uint8_t*)fini_startC
+                             + oc->sections[i].info->macho_section->size);
+
+            for (int pn = 0; fini < fini_end; fini++, pn++) {
+                IF_DEBUG(linker, debugBelch("ocRunFini_MachO:     function pointer %d at %p to %p\n",
+                                            pn, (void *) fini, (void *) *fini));
+                (*fini)();
+            }
+        }
+    }
+
+    return 1;
+}
+
 /*
  * Figure out by how much to shift the entire Mach-O file in memory
  * when loading so that its single segment ends up 16-byte-aligned
