@@ -6,7 +6,7 @@
  *
  * ---------------------------------------------------------------------------*/
 
-#include "PosixSource.h"
+#include "rts/PosixSource.h"
 #include "Rts.h"
 
 #include "Schedule.h"
@@ -202,7 +202,7 @@ ioManagerDie (void)
 
     {
         // Shut down IO managers
-        for (i=0; i < getNumCapabilities(); i++) {
+        for (i=0; i < n_capabilities; i++) {
             const int fd = RELAXED_LOAD(&capabilities[i]->io_manager_control_wr_fd);
             if (0 <= fd) {
                 r = write(fd, &byte, 1);
