@@ -11,7 +11,7 @@
  *
  * ---------------------------------------------------------------------------*/
 
-#include "PosixSource.h"
+#include "rts/PosixSource.h"
 #include "Rts.h"
 #include "HsFFI.h"
 
@@ -158,6 +158,7 @@ StgWord8 the_gc_thread[sizeof(gc_thread) + 64 * sizeof(gen_workspace)]
 #endif // THREADED_RTS
 
 /* Note [n_gc_threads]
+   ~~~~~~~~~~~~~~~~~~~
 This is a global variable that originally tracked the number of threads
 participating in the current gc. It's meaning has diverged from this somewhat,
 as it does not distinguish betweeen idle and non-idle threads. An idle thread
@@ -291,7 +292,6 @@ GarbageCollect (uint32_t collect_gen,
       any_work, scav_find_work, max_n_todo_overflow;
 #if defined(THREADED_RTS)
   gc_thread *saved_gct;
-  bool gc_sparks_all_caps;
 #endif
   uint32_t g, n;
   // The time we should report our heap census as occurring at, if necessary.
@@ -309,7 +309,7 @@ GarbageCollect (uint32_t collect_gen,
 #endif
 
 #if defined(PROFILING)
-  CostCentreStack *save_CCS[getNumCapabilities()];
+  CostCentreStack *save_CCS[n_capabilities];
 #endif
 
   ACQUIRE_SM_LOCK;
@@ -340,7 +340,7 @@ GarbageCollect (uint32_t collect_gen,
 
   // attribute any costs to CCS_GC
 #if defined(PROFILING)
-  for (n = 0; n < getNumCapabilities(); n++) {
+  for (n = 0; n < n_capabilities; n++) {
       save_CCS[n] = capabilities[n]->r.rCCCS;
       capabilities[n]->r.rCCCS = CCS_GC;
   }
@@ -375,8 +375,7 @@ GarbageCollect (uint32_t collect_gen,
           static_flag == STATIC_FLAG_A ? STATIC_FLAG_B : STATIC_FLAG_A;
   }
 
-  /* N.B. We currently don't unload code with the non-moving collector. */
-  if (major_gc && !RtsFlags.GcFlags.useNonmoving) {
+  if (major_gc) {
       unload_mark_needed = prepareUnloadCheck();
   } else {
       unload_mark_needed = false;
@@ -397,9 +396,9 @@ GarbageCollect (uint32_t collect_gen,
    * here
   */
   if (gc_type == SYNC_GC_PAR) {
-      n_gc_threads = getNumCapabilities();
+      n_gc_threads = n_capabilities;
       n_gc_idle_threads = 0;
-      for (uint32_t i = 0; i < getNumCapabilities(); ++i) {
+      for (uint32_t i = 0; i < n_capabilities; ++i) {
           if (idle_cap[i]) {
               ASSERT(i != gct->thread_index);
               ++n_gc_idle_threads;
@@ -407,7 +406,7 @@ GarbageCollect (uint32_t collect_gen,
       }
   } else {
       n_gc_threads = 1;
-      n_gc_idle_threads = getNumCapabilities() - 1;
+      n_gc_idle_threads = n_capabilities - 1;
   }
   work_stealing = RtsFlags.ParFlags.parGcLoadBalancingEnabled &&
       N >= RtsFlags.ParFlags.parGcLoadBalancingGen &&
@@ -430,15 +429,15 @@ GarbageCollect (uint32_t collect_gen,
   SEQ_CST_STORE(&gc_running_threads, 0);
 
   ASSERT(n_gc_threads > 0);
-  ASSERT(n_gc_threads <= getNumCapabilities());
-  ASSERT(n_gc_idle_threads < getNumCapabilities());
+  ASSERT(n_gc_threads <= n_capabilities);
+  ASSERT(n_gc_idle_threads < n_capabilities);
   // If we are work stealing, there better be another(i.e. not us) non-idle gc
   // thread
   ASSERT(!work_stealing || n_gc_threads - 1 > n_gc_idle_threads);
 
 
   debugTrace(DEBUG_gc, "GC (gen %d, using %d thread(s), %s work stealing)",
-             N, (int)getNumCapabilities() - (int)n_gc_idle_threads,
+             N, (int)n_capabilities - (int)n_gc_idle_threads,
              work_stealing ? "with": "without");
 
 #if defined(DEBUG)
@@ -500,7 +499,7 @@ GarbageCollect (uint32_t collect_gen,
   // call back into the GC via mark_root() (due to the gct register
   // variable).
   if (!is_par_gc()) {
-      for (n = 0; n < getNumCapabilities(); n++) {
+      for (n = 0; n < n_capabilities; n++) {
 #if defined(THREADED_RTS)
           scavenge_capability_mut_Lists1(capabilities[n]);
 #else
@@ -509,7 +508,7 @@ GarbageCollect (uint32_t collect_gen,
       }
   } else {
       scavenge_capability_mut_lists(gct->cap);
-      for (n = 0; n < getNumCapabilities(); n++) {
+      for (n = 0; n < n_capabilities; n++) {
           if (idle_cap[n]) {
               markCapability(mark_root, gct, capabilities[n],
                              true/*don't mark sparks*/);
@@ -525,7 +524,7 @@ GarbageCollect (uint32_t collect_gen,
   // follow all the roots that the application knows about.
   gct->evac_gen_no = 0;
   if (!is_par_gc()) {
-      for (n = 0; n < getNumCapabilities(); n++) {
+      for (n = 0; n < n_capabilities; n++) {
           markCapability(mark_root, gct, capabilities[n],
                          true/*don't mark sparks*/);
       }
@@ -557,9 +556,6 @@ GarbageCollect (uint32_t collect_gen,
   StgTSO *resurrected_threads = END_TSO_QUEUE;
   // must be last...  invariant is that everything is fully
   // scavenged at this point.
-#if defined(THREADED_RTS)
-  gc_sparks_all_caps = !work_stealing || !is_par_gc();
-#endif
   work_stealing = false;
   while (traverseWeakPtrList(&dead_weak_ptr_list, &resurrected_threads))
   {
@@ -572,13 +568,12 @@ GarbageCollect (uint32_t collect_gen,
   gcStableNameTable();
 
 #if defined(THREADED_RTS)
-  // See Note [Pruning the spark pool]
-  if (gc_sparks_all_caps) {
-      for (n = 0; n < getNumCapabilities(); n++) {
+  if (!is_par_gc()) {
+      for (n = 0; n < n_capabilities; n++) {
           pruneSparkQueue(false, capabilities[n]);
       }
   } else {
-      for (n = 0; n < getNumCapabilities(); n++) {
+      for (n = 0; n < n_capabilities; n++) {
           if (n == cap->no || idle_cap[n]) {
               pruneSparkQueue(false, capabilities[n]);
          }
@@ -684,7 +679,7 @@ GarbageCollect (uint32_t collect_gen,
     // stats.  Every mutable list is copied during every GC.
     if (g > 0) {
         W_ mut_list_size = 0;
-        for (n = 0; n < getNumCapabilities(); n++) {
+        for (n = 0; n < n_capabilities; n++) {
             mut_list_size += countOccupied(capabilities[n]->mut_lists[g]);
         }
         copied +=  mut_list_size;
@@ -837,7 +832,7 @@ GarbageCollect (uint32_t collect_gen,
     // add in the partial blocks in the gen_workspaces
     {
         uint32_t i;
-        for (i = 0; i < getNumCapabilities(); i++) {
+        for (i = 0; i < n_capabilities; i++) {
             live_words  += gcThreadLiveWords(i, gen->no);
             live_blocks += gcThreadLiveBlocks(i, gen->no);
         }
@@ -847,9 +842,11 @@ GarbageCollect (uint32_t collect_gen,
   // Flush the update remembered sets. See Note [Eager update remembered set
   // flushing] in NonMovingMark.c
   if (RtsFlags.GcFlags.useNonmoving) {
-      for (n = 0; n < getNumCapabilities(); n++) {
-          nonmovingAddUpdRemSetBlocks(&capabilities[n]->upd_rem_set);
+      RELEASE_SM_LOCK;
+      for (n = 0; n < n_capabilities; n++) {
+          nonmovingAddUpdRemSetBlocks(&capabilities[n]->upd_rem_set.queue);
       }
+      ACQUIRE_SM_LOCK;
   }
 
   // Mark and sweep the oldest generation.
@@ -870,6 +867,8 @@ GarbageCollect (uint32_t collect_gen,
       // old_weak_ptr_list should be empty.
       ASSERT(oldest_gen->old_weak_ptr_list == NULL);
 
+      // we may need to take the lock to allocate mark queue blocks
+      RELEASE_SM_LOCK;
       // dead_weak_ptr_list contains weak pointers with dead keys. Those need to
       // be kept alive because we'll use them in finalizeSchedulers(). Similarly
       // resurrected_threads are also going to be used in resurrectedThreads()
@@ -879,9 +878,10 @@ GarbageCollect (uint32_t collect_gen,
 #if !defined(THREADED_RTS)
       // In the non-threaded runtime this is the only time we push to the
       // upd_rem_set
-      nonmovingAddUpdRemSetBlocks(&gct->cap->upd_rem_set);
+      nonmovingAddUpdRemSetBlocks(&gct->cap->upd_rem_set.queue);
 #endif
       nonmovingCollect(&dead_weak_ptr_list, &resurrected_threads);
+      ACQUIRE_SM_LOCK;
   }
 
   // Update the max size of older generations after a major GC:
@@ -1063,7 +1063,7 @@ GarbageCollect (uint32_t collect_gen,
 
   // restore enclosing cost centre
 #if defined(PROFILING)
-  for (n = 0; n < getNumCapabilities(); n++) {
+  for (n = 0; n < n_capabilities; n++) {
       capabilities[n]->r.rCCCS = save_CCS[n];
   }
 #endif
@@ -1212,7 +1212,7 @@ freeGcThreads (void)
     if (gc_threads != NULL) {
 #if defined(THREADED_RTS)
         uint32_t i;
-        for (i = 0; i < getNumCapabilities(); i++) {
+        for (i = 0; i < n_capabilities; i++) {
             for (g = 0; g < RtsFlags.GcFlags.generations; g++)
             {
                 freeWSDeque(gc_threads[i]->gens[g].todo_q);
@@ -1372,6 +1372,7 @@ void
 gcWorkerThread (Capability *cap)
 {
     gc_thread *saved_gct;
+
     // necessary if we stole a callee-saves register for gct:
     saved_gct = gct;
 
@@ -1402,10 +1403,13 @@ gcWorkerThread (Capability *cap)
     scavenge_until_all_done();
 
 #if defined(THREADED_RTS)
-    // See Note [Pruning the spark pool]
-    if(work_stealing && is_par_gc()) {
-        pruneSparkQueue(false, cap);
-    }
+    // Now that the whole heap is marked, we discard any sparks that
+    // were found to be unreachable.  The main GC thread is currently
+    // marking heap reachable via weak pointers, so it is
+    // non-deterministic whether a spark will be retained if it is
+    // only reachable via weak pointers.  To fix this problem would
+    // require another GC barrier, which is too high a price.
+    pruneSparkQueue(false, cap);
 #endif
 
     // Wait until we're told to continue
@@ -1438,26 +1442,26 @@ void
 waitForGcThreads (Capability *cap, bool idle_cap[])
 {
     // n_gc_threads is not valid here, we're too early
-    uint32_t n_threads = getNumCapabilities();
+    uint32_t n_threads = n_capabilities;
     const uint32_t me = cap->no;
     uint32_t i, cur_n_gc_entered;
     Time t0, t1, t2;
 
     t0 = t1 = t2 = getProcessElapsedTime();
 
-    for(i = 0; i < getNumCapabilities(); ++i) {
+    for(i = 0; i < n_capabilities; ++i) {
         if (i == me || idle_cap[i]) {
             --n_threads;
         }
     }
 
-    ASSERT(n_threads < getNumCapabilities()); // must be less because we don't count ourself
+    ASSERT(n_threads < n_capabilities); // must be less because we don't count ourself
     if(n_threads == 0) { return; }
 
     ACQUIRE_LOCK(&gc_entry_mutex);
     while((cur_n_gc_entered = SEQ_CST_LOAD(&n_gc_entered)) != n_threads) {
         ASSERT(cur_n_gc_entered < n_threads);
-        for(i = 0; i < getNumCapabilities(); ++i) {
+        for(i = 0; i < n_capabilities; ++i) {
             if (i == me || idle_cap[i]) { continue; }
             if (SEQ_CST_LOAD(&gc_threads[i]->wakeup) != GC_THREAD_STANDING_BY) {
                 prodCapability(capabilities[i], cap->running_task);
@@ -1547,7 +1551,7 @@ shutdown_gc_threads (uint32_t me USED_IF_THREADS USED_IF_DEBUG,
     }
 #if defined(DEBUG)
     uint32_t i;
-    for (i=0; i < getNumCapabilities(); i++) {
+    for (i=0; i < n_capabilities; i++) {
         if (i == me || idle_cap[i]) continue;
         ASSERT(SEQ_CST_LOAD(&gc_threads[i]->wakeup) == GC_THREAD_WAITING_TO_CONTINUE);
     }
@@ -1560,11 +1564,11 @@ shutdown_gc_threads (uint32_t me USED_IF_THREADS USED_IF_DEBUG,
 void
 releaseGCThreads (Capability *cap USED_IF_THREADS, bool idle_cap[])
 {
-    const uint32_t n_threads = getNumCapabilities();
+    const uint32_t n_threads = n_capabilities;
     const uint32_t me = cap->no;
     uint32_t i;
-#if defined(DEBUG)
     uint32_t num_idle = 0;
+#if defined(ASSERTS_ENABLED)
     for(i=0; i < n_threads; ++i) {
         ASSERT(!(i==me && idle_cap[i]));
         if (idle_cap[i]) { ++num_idle;}
@@ -1612,14 +1616,14 @@ prepare_collected_gen (generation *gen)
 
     if (RtsFlags.GcFlags.useNonmoving && g == oldest_gen->no) {
         // Nonmoving heap's mutable list is always a root.
-        for (i = 0; i < getNumCapabilities(); i++) {
+        for (i = 0; i < n_capabilities; i++) {
             stash_mut_list(capabilities[i], g);
         }
     } else if (g != 0) {
         // Otherwise throw away the current mutable list. Invariant: the
         // mutable list always has at least one block; this means we can avoid
         // a check for NULL in recordMutable().
-        for (i = 0; i < getNumCapabilities(); i++) {
+        for (i = 0; i < n_capabilities; i++) {
             bdescr *old = RELAXED_LOAD(&capabilities[i]->mut_lists[g]);
             freeChain(old);
 
@@ -1656,7 +1660,7 @@ prepare_collected_gen (generation *gen)
 
     // grab all the partial blocks stashed in the gc_thread workspaces and
     // move them to the old_blocks list of this gen.
-    for (n = 0; n < getNumCapabilities(); n++) {
+    for (n = 0; n < n_capabilities; n++) {
         ws = &gc_threads[n]->gens[gen->no];
 
         for (bd = ws->part_list; bd != NULL; bd = next) {
@@ -1755,7 +1759,7 @@ prepare_uncollected_gen (generation *gen)
     // save the current mutable lists for this generation, and
     // allocate a fresh block for each one.  We'll traverse these
     // mutable lists as roots early on in the GC.
-    for (i = 0; i < getNumCapabilities(); i++) {
+    for (i = 0; i < n_capabilities; i++) {
         stash_mut_list(capabilities[i], gen->no);
     }
 
@@ -1829,7 +1833,7 @@ collect_pinned_object_blocks (void)
     const bool use_nonmoving = RtsFlags.GcFlags.useNonmoving;
     generation *const gen = (use_nonmoving && major_gc) ? oldest_gen : g0;
 
-    for (uint32_t n = 0; n < getNumCapabilities(); n++) {
+    for (uint32_t n = 0; n < n_capabilities; n++) {
         bdescr *last = NULL;
         if (use_nonmoving && gen == oldest_gen) {
             // Mark objects as belonging to the nonmoving heap
@@ -1945,7 +1949,7 @@ resizeGenerations (void)
     // minimum size for generation zero
     min_alloc = stg_max((RtsFlags.GcFlags.pcFreeHeap * max) / 200,
                         RtsFlags.GcFlags.minAllocAreaSize
-                        * (W_)getNumCapabilities());
+                        * (W_)n_capabilities);
 
     // Auto-enable compaction when the residency reaches a
     // certain percentage of the maximum heap size (default: 30%).
@@ -1981,7 +1985,7 @@ resizeGenerations (void)
             heapOverflow();
         }
 
-        if (oldest_gen->compact) {
+        if (oldest_gen->compact || RtsFlags.GcFlags.useNonmoving) {
             if ( (size + (size - 1) * (gens - 2) * 2) + min_alloc > max ) {
                 size = (max - min_alloc) / ((gens - 1) * 2 - 1);
             }
@@ -2017,7 +2021,7 @@ static void
 resize_nursery (void)
 {
     const StgWord min_nursery =
-      RtsFlags.GcFlags.minAllocAreaSize * (StgWord)getNumCapabilities();
+      RtsFlags.GcFlags.minAllocAreaSize * (StgWord)n_capabilities;
 
     if (RtsFlags.GcFlags.generations == 1)
     {   // Two-space collector:
@@ -2201,7 +2205,7 @@ bool doIdleGCWork(Capability *cap STG_UNUSED, bool all)
 
 
 /* Note [Synchronising work stealing]
- *
+ * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  * During parallel garbage collections, idle gc threads will steal work from
  * other threads. If they see no work to steal then they will wait on a
  * condition variabl(gc_running_cv).
@@ -2247,6 +2251,7 @@ bool doIdleGCWork(Capability *cap STG_UNUSED, bool all)
  * */
 
 /* Note [Scaling retained memory]
+ * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  * Tickets: #19381 #19359 #14702
  *
  * After a spike in memory usage we have been conservative about returning
