@@ -111,11 +111,6 @@ typedef struct {
     MarkQueue queue;
 } UpdRemSet;
 
-// How much marking work we are allowed to perform
-// See Note [Sync phase marking budget] in NonMoving.c
-typedef int64_t MarkBudget;
-#define UNLIMITED_MARK_BUDGET INT64_MIN
-
 // Number of blocks to allocate for a mark queue
 #define MARK_QUEUE_BLOCKS 16
 
@@ -140,13 +135,15 @@ extern MarkQueue *current_mark_queue;
 extern bdescr *upd_rem_set_block_list;
 
 
-void nonmovingMarkInit(void);
+void nonmovingMarkInitUpdRemSet(void);
 
-void nonmovingInitUpdRemSet(UpdRemSet *rset);
+void init_upd_rem_set(UpdRemSet *rset);
+void reset_upd_rem_set(UpdRemSet *rset);
 void updateRemembSetPushClosure(Capability *cap, StgClosure *p);
 void updateRemembSetPushThunk(Capability *cap, StgThunk *p);
 void updateRemembSetPushTSO(Capability *cap, StgTSO *tso);
 void updateRemembSetPushStack(Capability *cap, StgStack *stack);
+void updateRemembSetPushMessageThrowTo(Capability *cap, MessageThrowTo *m);
 
 #if defined(THREADED_RTS)
 void nonmovingFlushCapUpdRemSetBlocks(Capability *cap);
@@ -159,13 +156,8 @@ void markQueueAddRoot(MarkQueue* q, StgClosure** root);
 
 void initMarkQueue(MarkQueue *queue);
 void freeMarkQueue(MarkQueue *queue);
-void nonmovingMark(MarkBudget *budget, struct MarkQueue_ *restrict queue);
-INLINE_HEADER void nonmovingMarkUnlimitedBudget(struct MarkQueue_ *restrict queue) {
-    MarkBudget budget = UNLIMITED_MARK_BUDGET;
-    nonmovingMark(&budget, queue);
-}
+void nonmovingMark(struct MarkQueue_ *restrict queue);
 
-void nonmovingMarkWeakPtrList(struct MarkQueue_ *queue);
 bool nonmovingTidyWeaks(struct MarkQueue_ *queue);
 void nonmovingTidyThreads(void);
 void nonmovingMarkDeadWeaks(struct MarkQueue_ *queue, StgWeak **dead_weak_ptr_list);
@@ -173,7 +165,7 @@ void nonmovingResurrectThreads(struct MarkQueue_ *queue, StgTSO **resurrected_th
 bool nonmovingIsAlive(StgClosure *p);
 void nonmovingMarkDeadWeak(struct MarkQueue_ *queue, StgWeak *w);
 void nonmovingMarkLiveWeak(struct MarkQueue_ *queue, StgWeak *w);
-void nonmovingAddUpdRemSetBlocks(UpdRemSet *rset);
+void nonmovingAddUpdRemSetBlocks(struct MarkQueue_ *rset);
 
 void markQueuePush(MarkQueue *q, const MarkQueueEnt *ent);
 void markQueuePushClosureGC(MarkQueue *q, StgClosure *p);
