@@ -13,7 +13,7 @@
  *
  * ---------------------------------------------------------------------------*/
 
-#include "PosixSource.h"
+#include "rts/PosixSource.h"
 #include "Rts.h"
 
 #if defined(DEBUG)                                                   /* whole file */
@@ -619,12 +619,11 @@ static void checkNonmovingSegments (struct NonmovingSegment *seg)
 void checkNonmovingHeap (const struct NonmovingHeap *heap)
 {
     for (unsigned int i=0; i < NONMOVING_ALLOCA_CNT; i++) {
-        const struct NonmovingAllocator *alloc = &heap->allocators[i];
+        const struct NonmovingAllocator *alloc = heap->allocators[i];
         checkNonmovingSegments(alloc->filled);
         checkNonmovingSegments(alloc->active);
-        for (unsigned int cap_n=0; cap_n < getNumCapabilities(); cap_n++) {
-            Capability *cap = capabilities[cap_n];
-            checkNonmovingSegments(cap->current_segments[i]);
+        for (unsigned int cap=0; cap < n_capabilities; cap++) {
+            checkNonmovingSegments(alloc->current[cap]);
         }
     }
 }
@@ -827,7 +826,7 @@ static void
 checkMutableLists (void)
 {
     uint32_t i;
-    for (i = 0; i < getNumCapabilities(); i++) {
+    for (i = 0; i < n_capabilities; i++) {
         checkLocalMutableLists(i);
     }
 }
@@ -910,7 +909,6 @@ static void checkGeneration (generation *gen,
 #if defined(THREADED_RTS)
     // Note [heap sanity checking with SMP]
     // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-    //
     // heap sanity checking doesn't work with SMP for two reasons:
     //
     //   * We can't zero the slop. However, we can sanity-check the heap after a
@@ -947,7 +945,7 @@ static void checkGeneration (generation *gen,
 
     checkHeapChain(gen->blocks);
 
-    for (n = 0; n < getNumCapabilities(); n++) {
+    for (n = 0; n < n_capabilities; n++) {
         ws = &gc_threads[n]->gens[gen->no];
         checkHeapChain(ws->todo_bd);
         checkHeapChain(ws->part_list);
@@ -972,7 +970,7 @@ static void checkFullHeap (bool after_major_gc)
     for (g = 0; g < RtsFlags.GcFlags.generations; g++) {
         checkGeneration(&generations[g], after_major_gc);
     }
-    for (n = 0; n < getNumCapabilities(); n++) {
+    for (n = 0; n < n_capabilities; n++) {
         checkNurserySanity(&nurseries[n]);
     }
 }
@@ -1020,7 +1018,7 @@ findMemoryLeak (void)
 {
     uint32_t g, i, j;
     for (g = 0; g < RtsFlags.GcFlags.generations; g++) {
-        for (i = 0; i < getNumCapabilities(); i++) {
+        for (i = 0; i < n_capabilities; i++) {
             markBlocks(capabilities[i]->mut_lists[g]);
             markBlocks(gc_threads[i]->gens[g].part_list);
             markBlocks(gc_threads[i]->gens[g].scavd_list);
@@ -1035,7 +1033,7 @@ findMemoryLeak (void)
         markBlocks(nurseries[i].blocks);
     }
 
-    for (i = 0; i < getNumCapabilities(); i++) {
+    for (i = 0; i < n_capabilities; i++) {
         markBlocks(gc_threads[i]->free_blocks);
         markBlocks(capabilities[i]->pinned_object_block);
         markBlocks(capabilities[i]->upd_rem_set.queue.blocks);
@@ -1048,12 +1046,11 @@ findMemoryLeak (void)
         markBlocks(nonmoving_compact_objects);
         markBlocks(nonmoving_marked_compact_objects);
         for (i = 0; i < NONMOVING_ALLOCA_CNT; i++) {
-            struct NonmovingAllocator *alloc = &nonmovingHeap.allocators[i];
+            struct NonmovingAllocator *alloc = nonmovingHeap.allocators[i];
             markNonMovingSegments(alloc->filled);
             markNonMovingSegments(alloc->active);
-            for (j = 0; j < getNumCapabilities(); j++) {
-                Capability *cap = capabilities[j];
-                markNonMovingSegments(cap->current_segments[i]);
+            for (j = 0; j < n_capabilities; j++) {
+                markNonMovingSegments(alloc->current[j]);
             }
         }
         markNonMovingSegments(nonmovingHeap.sweep_list);
@@ -1158,18 +1155,23 @@ countNonMovingSegments(struct NonmovingSegment *segs)
     return ret;
 }
 
+static W_
+countNonMovingAllocator(struct NonmovingAllocator *alloc)
+{
+    W_ ret = countNonMovingSegments(alloc->filled)
+           + countNonMovingSegments(alloc->active);
+    for (uint32_t i = 0; i < n_capabilities; ++i) {
+        ret += countNonMovingSegments(alloc->current[i]);
+    }
+    return ret;
+}
+
 static W_
 countNonMovingHeap(struct NonmovingHeap *heap)
 {
     W_ ret = 0;
     for (int alloc_idx = 0; alloc_idx < NONMOVING_ALLOCA_CNT; alloc_idx++) {
-        struct NonmovingAllocator *alloc = &heap->allocators[alloc_idx];
-        ret += countNonMovingSegments(alloc->filled);
-        ret += countNonMovingSegments(alloc->active);
-        for (uint32_t c = 0; c < getNumCapabilities(); ++c) {
-            Capability *cap = capabilities[c];
-            ret += countNonMovingSegments(cap->current_segments[alloc_idx]);
-        }
+        ret += countNonMovingAllocator(heap->allocators[alloc_idx]);
     }
     ret += countNonMovingSegments(heap->sweep_list);
     ret += countNonMovingSegments(heap->free);
@@ -1199,7 +1201,7 @@ memInventory (bool show)
 
   for (g = 0; g < RtsFlags.GcFlags.generations; g++) {
       gen_blocks[g] = 0;
-      for (i = 0; i < getNumCapabilities(); i++) {
+      for (i = 0; i < n_capabilities; i++) {
           gen_blocks[g] += countBlocks(capabilities[i]->mut_lists[g]);
           gen_blocks[g] += countBlocks(gc_threads[i]->gens[g].part_list);
           gen_blocks[g] += countBlocks(gc_threads[i]->gens[g].scavd_list);
@@ -1212,7 +1214,7 @@ memInventory (bool show)
       ASSERT(countBlocks(nurseries[i].blocks) == nurseries[i].n_blocks);
       nursery_blocks += nurseries[i].n_blocks;
   }
-  for (i = 0; i < getNumCapabilities(); i++) {
+  for (i = 0; i < n_capabilities; i++) {
       W_ n = countBlocks(gc_threads[i]->free_blocks);
       gc_free_blocks += n;
       if (capabilities[i]->pinned_object_block != NULL) {
@@ -1238,7 +1240,7 @@ memInventory (bool show)
   free_blocks = countFreeList();
 
   // count UpdRemSet blocks
-  for (i = 0; i < getNumCapabilities(); ++i) {
+  for (i = 0; i < n_capabilities; ++i) {
       upd_rem_set_blocks += countBlocks(capabilities[i]->upd_rem_set.queue.blocks);
   }
   upd_rem_set_blocks += countBlocks(upd_rem_set_block_list);
@@ -1269,7 +1271,7 @@ memInventory (bool show)
       debugBelch("  nursery      : %5" FMT_Word " blocks (%6.1lf MB)\n",
                  nursery_blocks, MB(nursery_blocks));
       debugBelch("  empty pinned : %5" FMT_Word " blocks (%6.1lf MB)\n",
-                 nursery_blocks, MB(free_pinned_blocks));
+                 free_pinned_blocks, MB(free_pinned_blocks));
       debugBelch("  retainer     : %5" FMT_Word " blocks (%6.1lf MB)\n",
                  retainer_blocks, MB(retainer_blocks));
       debugBelch("  arena blocks : %5" FMT_Word " blocks (%6.1lf MB)\n",
