@@ -15,15 +15,15 @@
 #if RTS_LINKER_USE_MMAP
 /*
  * On FreeBSD and Darwin, when _XOPEN_SOURCE is defined, MAP_ANONYMOUS is not
- * exposed from <sys/mman.h>.  Include <sys/mman.h> before "PosixSource.h".
+ * exposed from <sys/mman.h>.  Include <sys/mman.h> before "rts/PosixSource.h".
  *
- * Alternatively, we could drop "PosixSource.h" from this file, but for just
+ * Alternatively, we could drop "rts/PosixSource.h" from this file, but for just
  * one non-POSIX macro, that seems a needless price to pay.
  */
 #include <sys/mman.h>
 #endif
 
-#include "PosixSource.h"
+#include "rts/PosixSource.h"
 #include "Rts.h"
 
 #include "Storage.h"
@@ -109,6 +109,7 @@ Mutex sm_mutex;
 
 static void allocNurseries (uint32_t from, uint32_t to);
 static void assignNurseriesToCapabilities (uint32_t from, uint32_t to);
+static StgInd * lockCAF (StgRegTable *reg, StgIndStatic *caf);
 
 void
 initGeneration (generation *gen, int g)
@@ -214,14 +215,17 @@ initStorage (void)
   }
   oldest_gen->to = oldest_gen;
 
+  // Nonmoving heap uses oldest_gen so initialize it after initializing oldest_gen
+  nonmovingInit();
+
 #if defined(THREADED_RTS)
   // nonmovingAddCapabilities allocates segments, which requires taking the gc
   // sync lock, so initialize it before nonmovingAddCapabilities
   initSpinLock(&gc_alloc_block_sync);
 #endif
 
-  // Nonmoving heap uses oldest_gen so initialize it after initializing oldest_gen
-  nonmovingInit();
+  if (RtsFlags.GcFlags.useNonmoving)
+      nonmovingAddCapabilities(n_capabilities);
 
   /* The oldest generation has one step. */
   if (RtsFlags.GcFlags.compact || RtsFlags.GcFlags.sweep) {
@@ -253,7 +257,7 @@ initStorage (void)
   for (n = 0; n < n_numa_nodes; n++) {
       next_nursery[n] = n;
   }
-  storageAddCapabilities(0, getNumCapabilities());
+  storageAddCapabilities(0, n_capabilities);
 
   IF_DEBUG(gc, statDescribeGens());
 
@@ -317,10 +321,11 @@ void storageAddCapabilities (uint32_t from, uint32_t to)
         }
     }
 
-    // Initialize non-moving collector
+    // Initialize NonmovingAllocators and UpdRemSets
     if (RtsFlags.GcFlags.useNonmoving) {
-        for (i = from; i < to; i++) {
-            nonmovingInitCapability(capabilities[i]);
+        nonmovingAddCapabilities(to);
+        for (i = 0; i < to; ++i) {
+            init_upd_rem_set(&capabilities[i]->upd_rem_set);
         }
     }
 
@@ -370,7 +375,7 @@ void listAllBlocks (ListBlocksCb cb, void *user)
 {
   uint32_t g, i;
   for (g = 0; g < RtsFlags.GcFlags.generations; g++) {
-      for (i = 0; i < getNumCapabilities(); i++) {
+      for (i = 0; i < n_capabilities; i++) {
           cb(user, capabilities[i]->mut_lists[g]);
           cb(user, gc_threads[i]->gens[g].part_list);
           cb(user, gc_threads[i]->gens[g].scavd_list);
@@ -382,7 +387,7 @@ void listAllBlocks (ListBlocksCb cb, void *user)
   for (i = 0; i < n_nurseries; i++) {
       cb(user, nurseries[i].blocks);
   }
-  for (i = 0; i < getNumCapabilities(); i++) {
+  for (i = 0; i < n_capabilities; i++) {
       if (capabilities[i]->pinned_object_block != NULL) {
           cb(user, capabilities[i]->pinned_object_block);
       }
@@ -395,7 +400,6 @@ void listAllBlocks (ListBlocksCb cb, void *user)
 /* -----------------------------------------------------------------------------
    Note [CAF management]
    ~~~~~~~~~~~~~~~~~~~~~
-
    The entry code for every CAF does the following:
 
       - calls newCAF, which builds a CAF_BLACKHOLE on the heap and atomically
@@ -430,7 +434,6 @@ void listAllBlocks (ListBlocksCb cb, void *user)
    ------------------
    Note [atomic CAF entry]
    ~~~~~~~~~~~~~~~~~~~~~~~
-
    With THREADED_RTS, newCAF() is required to be atomic (see
    #5558). This is because if two threads happened to enter the same
    CAF simultaneously, they would create two distinct CAF_BLACKHOLEs,
@@ -444,7 +447,6 @@ void listAllBlocks (ListBlocksCb cb, void *user)
    ------------------
    Note [GHCi CAFs]
    ~~~~~~~~~~~~~~~~
-
    For GHCI, we have additional requirements when dealing with CAFs:
 
       - we must *retain* all dynamically-loaded CAFs ever entered,
@@ -466,7 +468,6 @@ void listAllBlocks (ListBlocksCb cb, void *user)
    ------------------
    Note [Static objects under the nonmoving collector]
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
    Static object management is a bit tricky under the nonmoving collector as we
    need to maintain a bit more state than in the moving collector. In
    particular, the moving collector uses the low bits of the STATIC_LINK field
@@ -510,7 +511,7 @@ void listAllBlocks (ListBlocksCb cb, void *user)
 
    -------------------------------------------------------------------------- */
 
-STATIC_INLINE StgInd *
+static StgInd *
 lockCAF (StgRegTable *reg, StgIndStatic *caf)
 {
     const StgInfoTable *orig_info;
@@ -593,6 +594,7 @@ newCAF(StgRegTable *reg, StgIndStatic *caf)
     if(keepCAFs && !(highMemDynamic && (void*) caf > (void*) 0x80000000))
     {
         // Note [dyn_caf_list]
+        // ~~~~~~~~~~~~~~~~~~~
         // If we are in GHCi _and_ we are using dynamic libraries,
         // then we can't redirect newCAF calls to newRetainedCAF (see below),
         // so we make newCAF behave almost like newRetainedCAF.
@@ -814,7 +816,7 @@ resetNurseries (void)
     for (n = 0; n < n_numa_nodes; n++) {
         next_nursery[n] = n;
     }
-    assignNurseriesToCapabilities(0, getNumCapabilities());
+    assignNurseriesToCapabilities(0, n_capabilities);
 
 #if defined(DEBUG)
     bdescr *bd;
@@ -986,7 +988,6 @@ accountAllocation(Capability *cap, W_ n)
 
 /* Note [slop on the heap]
  * ~~~~~~~~~~~~~~~~~~~~~~~
- *
  * We use the term "slop" to refer to allocated memory on the heap which isn't
  * occupied by any closure. Usually closures are packet tightly into the heap
  * blocks, storage for one immediately following another. However there are
@@ -1397,20 +1398,22 @@ allocatePinned (Capability *cap, W_ n /*words*/, W_ alignment /*bytes*/, W_ alig
    MUT_VAR_CLEAN object is not on the mutable list; a MUT_VAR_DIRTY
    is.  When written to, a MUT_VAR_CLEAN turns into a MUT_VAR_DIRTY
    and is put on the mutable list.
+   Note that it is responsibility of the caller to do the
+   stg_MUT_VAR_CLEAN comparison.
 */
 void
 dirty_MUT_VAR(StgRegTable *reg, StgMutVar *mvar, StgClosure *old)
 {
+    ASSERT(RELAXED_LOAD(&mvar->header.info) == &stg_MUT_VAR_CLEAN_info);
+
     Capability *cap = regTableToCapability(reg);
     // No barrier required here as no other heap object fields are read. See
     // note [Heap memory barriers] in SMP.h.
-    if (RELAXED_LOAD(&mvar->header.info) == &stg_MUT_VAR_CLEAN_info) {
-        SET_INFO((StgClosure*) mvar, &stg_MUT_VAR_DIRTY_info);
-        recordClosureMutated(cap, (StgClosure *) mvar);
-        IF_NONMOVING_WRITE_BARRIER_ENABLED {
-            // See Note [Dirty flags in the non-moving collector] in NonMoving.c
-            updateRemembSetPushClosure_(reg, old);
-        }
+    SET_INFO((StgClosure*) mvar, &stg_MUT_VAR_DIRTY_info);
+    recordClosureMutated(cap, (StgClosure *) mvar);
+    IF_NONMOVING_WRITE_BARRIER_ENABLED {
+        // See Note [Dirty flags in the non-moving collector] in NonMoving.c
+        updateRemembSetPushClosure_(reg, old);
     }
 }
 
@@ -1543,7 +1546,7 @@ dirty_MVAR(StgRegTable *reg, StgClosure *p, StgClosure *old_val)
 
 /* -----------------------------------------------------------------------------
  * Note [allocation accounting]
- *
+ * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  *   - When cap->r.rCurrentNusery moves to a new block in the nursery,
  *     we add the size of the used portion of the previous block to
  *     cap->total_allocated. (see finishedNurseryBlock())
@@ -1565,7 +1568,7 @@ calcTotalAllocated (void)
     uint64_t tot_alloc = 0;
     W_ n;
 
-    for (n = 0; n < getNumCapabilities(); n++) {
+    for (n = 0; n < n_capabilities; n++) {
         tot_alloc += capabilities[n]->total_allocated;
 
         traceEventHeapAllocated(capabilities[n],
@@ -1586,7 +1589,7 @@ updateNurseriesStats (void)
     uint32_t i;
     bdescr *bd;
 
-    for (i = 0; i < getNumCapabilities(); i++) {
+    for (i = 0; i < n_capabilities; i++) {
         // The current nursery block and the current allocate block have not
         // yet been accounted for in cap->total_allocated, so we add them here.
         bd = capabilities[i]->r.rCurrentNursery;
@@ -1819,7 +1822,6 @@ _bdescr (StgPtr p)
 /*
 Note [Sources of Block Level Fragmentation]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
 Block level fragmentation is when there is unused space in megablocks.
 The amount of fragmentation can be calculated as the difference between the
 total size of allocated blocks and the total size of allocated megablocks.
