@@ -24,6 +24,7 @@
 #include <stdio.h>
 
 /* Note [Non-Threaded WINIO design]
+   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    Compared to Async MIO, Async WINIO does all of the heavy processing at the
    Haskell side of things.  The same code as the threaded WINIO is re-used for
    the Non-threaded version.  Of course since we are in a non-threaded rts we
@@ -122,10 +123,8 @@
    See also Note [WINIO Manager design].
 
 
+  Note [Notifying the RTS/Haskell of completed events]
   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-    Note [Notifying the RTS/Haskell of completed events]
-  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
   The C side runner can't directly create a haskell thread.
   With the current API of the haskell runtime this would be terrible
   unsound. In particular the GC assumes no heap objects are generated,
@@ -137,17 +136,15 @@
   ensures there is only one OS thread at a time making use of the haskell
   heap.
 
+  Note [Non-Threaded IO Manager startup sequence]
   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-    Note [Non-Threaded IO Manager startup sequence]
-  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
   Under the new IO Manager we run a bit of initialization under
   hs_init(). The first call into actual IO manager code is a
   invocation of startupAsyncWinIO();
 
   There we initialize IO manager locale variables and.
   * call ioManagerStart()
-  * Creat a thread to execute "runner"
+  * Create a thread to execute "runner"
 
   We never truly shut down the IO Manager. While this means we
   might block forever on the IOPort if the IO Manager is no longer
@@ -274,6 +271,7 @@ void shutdownAsyncWinIO(bool wait_threads)
           WaitForSingleObject (workerThread, INFINITE);
         }
       completionPortHandle = INVALID_HANDLE_VALUE;
+      CloseHandle (workerThread);
       workerThread = NULL;
       workerThreadId = 0;
       free (entries);
