@@ -2,19 +2,23 @@
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE StandaloneDeriving #-}
 {-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE BangPatterns #-}
 {-# OPTIONS_GHC -Wno-incomplete-uni-patterns #-}
 {-# OPTIONS_GHC -Wno-incomplete-patterns #-}
 {-# OPTIONS_GHC -Wno-orphans #-}
 
-import Data.List (intercalate)
 import Data.Data
+import Data.List (intercalate)
+import GHC hiding (moduleName)
+import GHC.Data.Bag
+import GHC.Driver.Errors.Types
+import GHC.Driver.Ppr
+import GHC.Hs.Dump
+import GHC.Types.Error
 import GHC.Types.Name.Occurrence
 import GHC.Types.Name.Reader
-import GHC hiding (moduleName)
-import GHC.Driver.Session
-import GHC.Utils.Outputable hiding ( (<>) )
-import GHC.Hs.Dump
-import GHC.Data.Bag
+import GHC.Utils.Error
+import GHC.Utils.Outputable
 import System.Environment( getArgs )
 import System.Exit
 import System.FilePath
@@ -28,7 +32,6 @@ import Parsers
 
 import GHC.Parser.Lexer
 import GHC.Data.FastString
-import GHC.Types.SrcLoc
 
 -- ---------------------------------------------------------------------
 
@@ -142,7 +145,7 @@ _tt = testOneFile changers "/home/alanz/mysrc/git.haskell.org/worktree/exactprin
  -- "../../testsuite/tests/printer/Ppr055.hs" Nothing
  -- "../../testsuite/tests/printer/PprRecordDotSyntax1.hs" Nothing
  -- "../../testsuite/tests/printer/PprRecordDotSyntax2.hs" Nothing
- "../../testsuite/tests/printer/PprRecordDotSyntax3.hs" Nothing
+ -- "../../testsuite/tests/printer/PprRecordDotSyntax3.hs" Nothing
  -- "../../testsuite/tests/printer/PprRecordDotSyntax4.hs" Nothing
  -- "../../testsuite/tests/printer/PprRecordDotSyntaxA.hs" Nothing
  -- "../../testsuite/tests/printer/StarBinderAnns.hs" Nothing
@@ -195,6 +198,7 @@ _tt = testOneFile changers "/home/alanz/mysrc/git.haskell.org/worktree/exactprin
  -- "../../testsuite/tests/printer/PprSemis.hs" Nothing
  -- "../../testsuite/tests/printer/PprEmptyMostly.hs" Nothing
  -- "../../testsuite/tests/parser/should_compile/DumpSemis.hs" Nothing
+ "../../testsuite/tests/ghc-api/exactprint/Test20239.hs" Nothing
 
 -- cloneT does not need a test, function can be retired
 
@@ -271,9 +275,6 @@ main = do
    _ -> putStrLn usage
 
 deriving instance Data Token
-deriving instance Data PsSpan
-deriving instance Data BufSpan
-deriving instance Data BufPos
 
 writeBinFile :: FilePath -> String -> IO()
 writeBinFile fpath x = withBinaryFile fpath WriteMode (\h -> hSetEncoding h utf8 >> hPutStr h x)
@@ -283,8 +284,8 @@ testOneFile _ libdir fileName mchanger = do
        (p,_toks) <- parseOneFile libdir fileName
        -- putStrLn $ "\n\ngot p" ++ showAst (take 4 $ reverse _toks)
        let
-         origAst = ppAst (pm_parsed_source p)
-         pped    = exactPrint (pm_parsed_source p)
+         origAst = ppAst p
+         pped    = exactPrint p
 
          newFile         = dropExtension fileName <.> "ppr"      <.> takeExtension fileName
          newFileChanged  = dropExtension fileName <.> "changed"  <.> takeExtension fileName
@@ -298,7 +299,7 @@ testOneFile _ libdir fileName mchanger = do
 
        (changedSourceOk, expectedSource, changedSource) <- case mchanger of
          Just changer -> do
-           (pped', ast') <- exactprintWithChange libdir changer (pm_parsed_source p)
+           (pped', ast') <- exactprintWithChange libdir changer p
            writeBinFile changedAstFile (ppAst ast')
            writeBinFile newFileChanged pped'
 
@@ -310,7 +311,7 @@ testOneFile _ libdir fileName mchanger = do
 
        (p',_) <- parseOneFile libdir newFile
        let newAstStr :: String
-           newAstStr = ppAst (pm_parsed_source p')
+           newAstStr = ppAst p'
        writeBinFile newAstFile newAstStr
 
        let origAstOk = origAst == newAstStr
@@ -339,30 +340,23 @@ testOneFile _ libdir fileName mchanger = do
 ppAst :: Data a => a -> String
 ppAst ast = showSDocUnsafe $ showAstData BlankSrcSpanFile NoBlankEpAnnotations ast
 
-parseOneFile :: FilePath -> FilePath -> IO (ParsedModule, [Located Token])
-parseOneFile libdir fileName = do
-       let modByFile m =
-             case ml_hs_file $ ms_location m of
-               Nothing -> False
-               Just fn -> fn == fileName
-       runGhc (Just libdir) $ do
-         dflags <- getSessionDynFlags
-         let dflags2 = dflags `gopt_set` Opt_KeepRawTokenStream
-         _ <- setSessionDynFlags dflags2
-         addTarget Target { targetId = TargetFile fileName Nothing
-                          , targetAllowObjCode = True
-                          , targetContents = Nothing }
-         _ <- load LoadAllTargets
-         graph <- getModuleGraph
-         let
-           modSum = case filter modByFile (mgModSummaries graph) of
-                     [x] -> x
-                     xs -> error $ "Can't find module, got:"
-                              ++ show (map (ml_hs_file . ms_location) xs)
-         pm <- GHC.parseModule modSum
-         toks <- getTokenStream (ms_mod modSum)
-         return (pm, toks)
 
+parseOneFile :: FilePath -> FilePath -> IO (ParsedSource, [Located Token])
+parseOneFile libdir fileName = do
+  res <- parseModuleEpAnnsWithCpp libdir defaultCppOptions fileName
+  case res of
+    Left m -> error (showErrorMessages m)
+    Right (injectedComments, _dflags, pmod) -> do
+      let !pmodWithComments = insertCppComments pmod injectedComments
+      return (pmodWithComments, [])
+
+showErrorMessages :: Messages GhcMessage -> String
+showErrorMessages msgs =
+  renderWithContext defaultSDocContext
+    $ vcat
+    $ pprMsgEnvelopeBagWithLoc
+    $ getMessages
+    $ msgs
 
 -- ---------------------------------------------------------------------
 
@@ -437,7 +431,7 @@ changeLetIn1 _libdir parsed
   = return (everywhere (mkT replace) parsed)
   where
     replace :: HsExpr GhcPs -> HsExpr GhcPs
-    replace (HsLet (EpAnn anc (AnnsLet l _i) cs) localDecls expr)
+    replace (HsLet an tkLet localDecls _ expr)
       =
          let (HsValBinds x (ValBinds xv bagDecls sigs)) = localDecls
              [l2,_l1] = map wrapDecl $ bagToList bagDecls
@@ -445,8 +439,9 @@ changeLetIn1 _libdir parsed
              (L (SrcSpanAnn _ le) e) = expr
              a = (SrcSpanAnn (EpAnn (Anchor (realSrcSpan le) (MovedAnchor (SameLine 1))) mempty emptyComments) le)
              expr' = L a e
-         in (HsLet (EpAnn anc (AnnsLet l (EpaDelta (DifferentLine 1 0) [])) cs)
-                (HsValBinds x (ValBinds xv bagDecls' sigs)) expr')
+             tkIn' = L (TokenLoc (EpaDelta (DifferentLine 1 0) [])) HsTok
+         in (HsLet an tkLet
+                (HsValBinds x (ValBinds xv bagDecls' sigs)) tkIn' expr')
 
     replace x = x
 
@@ -517,7 +512,7 @@ changeLocalDecls libdir (L l p) = do
             os' = setEntryDP' os (DifferentLine 2 0)
         let sortKey = captureOrder decls
         let (EpAnn anc (AnnList (Just (Anchor anc2 _)) a b c dd) cs) = van
-        let van' = (EpAnn anc (AnnList (Just (Anchor anc2 (MovedAnchor (DifferentLine 1 4)))) a b c dd) cs)
+        let van' = (EpAnn anc (AnnList (Just (Anchor anc2 (MovedAnchor (DifferentLine 1 5)))) a b c dd) cs)
         let binds' = (HsValBinds van'
                           (ValBinds sortKey (listToBag $ decl':oldBinds)
                                           (sig':os':oldSigs)))
@@ -541,8 +536,8 @@ changeLocalDecls2 libdir (L l p) = do
                         -> Transform (LMatch GhcPs (LHsExpr GhcPs))
       replaceLocalBinds (L lm (Match ma mln pats (GRHSs _ rhs EmptyLocalBinds{}))) = do
         newSpan <- uniqueSrcSpanT
-        let anc = (Anchor (rs newSpan) (MovedAnchor (DifferentLine 1 2)))
-        let anc2 = (Anchor (rs newSpan) (MovedAnchor (DifferentLine 1 4)))
+        let anc = (Anchor (rs newSpan) (MovedAnchor (DifferentLine 1 3)))
+        let anc2 = (Anchor (rs newSpan) (MovedAnchor (DifferentLine 1 5)))
         let an = EpAnn anc
                         (AnnList (Just anc2) Nothing Nothing
                                  [(undeltaSpan (rs newSpan) AnnWhere (SameLine 0))] [])
@@ -589,7 +584,7 @@ changeWhereIn3b _libdir (L l p) = do
 addLocaLDecl1 :: Changer
 addLocaLDecl1 libdir lp = do
   Right (L ld (ValD _ decl)) <- withDynFlags libdir (\df -> parseDecl df "decl" "nn = 2")
-  let decl' = setEntryDP' (L ld decl) (DifferentLine 1 4)
+  let decl' = setEntryDP' (L ld decl) (DifferentLine 1 5)
       doAddLocal = do
         (de1:d2:d3:_) <- hsDecls lp
         (de1'',d2') <- balanceComments de1 d2
@@ -791,12 +786,12 @@ rmDecl5 _libdir lp = do
       doRmDecl = do
         let
           go :: HsExpr GhcPs -> Transform (HsExpr GhcPs)
-          go (HsLet a lb expr) = do
+          go (HsLet a tkLet lb tkIn expr) = do
             decs <- hsDeclsValBinds lb
             let dec = last decs
             _ <- transferEntryDPT (head decs) dec
             lb' <- replaceDeclsValbinds WithoutWhere lb [dec]
-            return (HsLet a lb' expr)
+            return (HsLet a tkLet lb' tkIn expr)
           go x = return x
 
         everywhereM (mkM go) lp
